@@ -1,18 +1,18 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
 
 #include "db_base.h"
+#include "func.h"
 #include "exception.h"
-#include <qmessagebox.h>
-#include <qdir.h>
-#include <qlistview.h>
-#include <qdir.h>
-
+#include <QtGui/QMessageBox>
+#include <QtGui/QListView>
+#include <QtGui/QClipboard>
+#include <QtCore/QDir>
 #include "widgets/MainWindow.h"
 #include "widgets/ImportMulti.h"
 
@@ -21,24 +21,37 @@ db_base::db_base(QString db, MainWindow *mw)
 {
 	dbName = db;
 	rootItem = newPKI();
-	headertext.clear();
 	mainwin = mw;
+	colResizing = 0;
 	currentIdx = QModelIndex();
-	view = NULL;
 	class_name = "base";
-	for (size_t i =0; i<ARRAY_SIZE(pkitype); i++)
-		pkitype[i] = none;
+	allHeaders << new dbheader(HD_internal_name, true, tr("Internal name"));
 }
 
 db_base::~db_base()
 {
+	saveHeaderState();
 	delete rootItem;
 }
 
-pki_base *db_base::newPKI(db_header_t *head){
+pki_base *db_base::newPKI(db_header_t *)
+{
 	return new pki_base("rootItem");
 }
 
+void db_base::createSuccess(pki_base *pki)
+{
+	if (!pki)
+		return;
+
+	if (pki_base::suppress_messages)
+		return;
+
+	QMessageBox::information(mainwin, XCA_TITLE,
+		pki->getMsg(pki_base::msg_create).
+			arg(pki->getIntName()));
+}
+
 void db_base::remFromCont(QModelIndex &idx)
 {
 	if (!idx.isValid())
@@ -50,6 +63,7 @@ void db_base::remFromCont(QModelIndex &idx)
 	beginRemoveRows(parent(idx), row, row);
 	parent_pki->takeChild(pki);
 	endRemoveRows();
+	emit columnsContentChanged();
 }
 
 void db_base::loadContainer()
@@ -59,7 +73,7 @@ void db_base::loadContainer()
 	db_header_t head;
 	pki_base *pki;
 
-	for (int i=0; pkitype[i] != none; i++) {
+	for (int i=0; i < pkitype.count(); i++) {
 		mydb.first();
 		while (mydb.find(pkitype[i], QString()) == 0) {
 			QString s;
@@ -91,33 +105,108 @@ void db_base::loadContainer()
 				pki = NULL;
 			}
 			free(p);
-			if (pki)
+			if (pki) {
 				inToCont(pki);
+			}
 next:
 			if (mydb.next())
 				break;
 		}
 	}
-	view->columnsResize();
+
+	mydb.first();
+	if (!mydb.find(setting, class_name + "_hdView")) {
+		QByteArray ba;
+		char *p;
+		if ((p = (char *)mydb.load(&head))) {
+			ba = QByteArray(p, head.len - sizeof(db_header_t));
+			free(p);
+		}
+		if (head.version != 5)
+			return;
+		try {
+			allHeaders.fromData(ba);
+		}  catch (errorEx()) {
+			for (int i=0; i< allHeaders.count(); i++) {
+				allHeaders[i]->reset();
+			}
+		}
+	}
+	emit columnsContentChanged();
+	return;
+}
+
+void db_base::saveHeaderState()
+{
+	QByteArray ba;
+	if (dbName.isEmpty())
+		return;
+	ba = allHeaders.toData();
+	db mydb(dbName);
+	mydb.set((const unsigned char *)ba.constData(), ba.size(), 5,
+		setting, class_name + "_hdView");
+}
+
+void db_base::setVisualIndex(int i, int visualIndex)
+{
+	if (colResizing)
+		return;
+	allHeaders[i]->visualIndex = visualIndex;
+}
+
+void db_base::sectionResized(int i, int,  int newSize)
+{
+	if (!allHeaders[i]->show || newSize <= 0 || colResizing)
+		return;
+	allHeaders[i]->size = newSize;
+}
+
+bool db_base::fixedHeaderSize(int sect)
+{
+	return allHeaders[sect]->size != -1;
+}
+
+void db_base::initHeaderView(QHeaderView *hv)
+{
+	int max = allHeaders.count();
+	colResizeStart();
+	for (int i=0; i<max; i++) {
+		allHeaders[i]->setupHeaderView(i, hv);
+	}
+	for (int i=0; i<max; i++) {
+		if (allHeaders[i]->visualIndex == -1)
+			continue;
+		if (hv->visualIndex(i) != allHeaders[i]->visualIndex) {
+			hv->moveSection(hv->visualIndex(i),
+					allHeaders[i]->visualIndex);
+		}
+	}
+	colResizeEnd();
+}
+
+void db_base::sortIndicatorChanged(int logicalIndex, Qt::SortOrder order)
+{
+	int max = allHeaders.count();
+	for (int i=0; i<max; i++) {
+		allHeaders[i]->sortIndicator = -1;
+	}
+	allHeaders[logicalIndex]->sortIndicator = order;
 }
 
 void db_base::insertPKI(pki_base *pki)
 {
-	unsigned char *p;
-	int size;
 	QString name;
 	db mydb(dbName);
+	QByteArray ba = pki->toData();
 
-	p = pki->toData(&size);
-
-	if (p) {
-		name = mydb.uniq_name(pki->getIntName(), pki->getType());
+	if (ba.count() > 0) {
+		name = mydb.uniq_name(pki->getIntName(), pkitype);
 		pki->setIntName(name);
-		mydb.add(p, size, pki->getVersion(), pki->getType(), name);
-		OPENSSL_free(p);
+		mydb.add((const unsigned char*)ba.constData(), ba.count(),
+			pki->getVersion(), pki->getType(), name);
 	}
 	inToCont(pki);
-	view->columnsResize();
+	emit columnsContentChanged();
 }
 
 void db_base::delete_ask()
@@ -126,39 +215,70 @@ void db_base::delete_ask()
 		return;
 	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
 
-	if (QMessageBox::information(mainwin, tr(XCA_TITLE),
-				delete_txt + ": " + pki->getIntName() + " ?\n" ,
-				tr("Delete"), tr("Cancel"))
-		) return;
+	if (QMessageBox::question(mainwin, XCA_TITLE,
+				pki->getMsg(pki_base::msg_delete)
+					.arg(pki->getIntName()),
+				QMessageBox::Ok | QMessageBox::Cancel) !=
+				QMessageBox::Ok)
+		 return;
 
 	deletePKI();
 }
 
+void db_base::pem2clipboard()
+{
+	long l;
+        const char *p;
+	QString msg;
+	QClipboard *cb = QApplication::clipboard();
+
+	BIO *bio = BIO_new(BIO_s_mem());
+	if (!currentIdx.isValid())
+		return;
+	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
+	pki->pem(bio);
+	openssl_error();
+	l = BIO_get_mem_data(bio, &p);
+	msg = QString::fromUtf8(p, l);
+	BIO_free(bio);
+	if (cb->supportsSelection())
+		cb->setText(msg, QClipboard::Selection);
+	else
+		cb->setText(msg);
+}
+
 void db_base::deletePKI()
 {
 	if (!currentIdx.isValid())
 		return;
 	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
+	try {
+		try {
+			pki->deleteFromToken();
+		} catch (errorEx &err) {
+			MainWindow::Error(err);
+		}
 
-	remFromCont(currentIdx);
+		remFromCont(currentIdx);
 
-	db mydb(dbName);
-	mydb.find(pki->getType(), pki->getIntName());
-	mydb.erase();
-	delete pki;
+		db mydb(dbName);
+		mydb.find(pki->getType(), pki->getIntName());
+		mydb.erase();
+		delete pki;
+	} catch (errorEx &err) {
+		MainWindow::Error(err);
+	}
 }
 
 void db_base::updatePKI(pki_base *pki)
 {
-	unsigned char *p;
-	int size;
 	db mydb(dbName);
 
-	p = pki->toData(&size);
+	QByteArray ba = pki->toData();
 
-	if (p) {
-		mydb.set(p, size, pki->getVersion(), pki->getType(), pki->getIntName());
-		OPENSSL_free(p);
+	if (ba.count() > 0) {
+		mydb.set((const unsigned char*)ba.constData(), ba.count(),
+			pki->getVersion(), pki->getType(), pki->getIntName());
 	}
 }
 
@@ -166,7 +286,9 @@ void db_base::deleteSelectedItems(XcaTreeView* view)
 {
 	QModelIndexList indexes = view->getSelectedIndexes();
 	QModelIndex index;
-	QString items;
+	QString items, msg;
+	int count = 0;
+	pki_base *pki = NULL;
 
 	if (indexes.count() == 0)
 		return;
@@ -174,14 +296,20 @@ void db_base::deleteSelectedItems(XcaTreeView* view)
 	foreach(index, indexes) {
 		if (index.column() != 0)
 			continue;
-		pki_base *pki = static_cast<pki_base*>(index.internalPointer());
+		pki = static_cast<pki_base*>(index.internalPointer());
 		items += "'" + pki->getIntName() + "' ";
+		count++;
 	}
+	if (count == 1)
+		msg = pki->getMsg(pki_base::msg_delete).arg(pki->getIntName());
+	else
+		msg = pki->getMsg(pki_base::msg_delete_multi).arg(count).
+				arg(items);
 
-	if (QMessageBox::information(mainwin, tr(XCA_TITLE),
-				delete_txt + ": " + items + " ?\n" ,
-				tr("Delete"), tr("Cancel"))
-        ) return;
+	if (QMessageBox::question(mainwin, XCA_TITLE, msg,
+				QMessageBox::Ok | QMessageBox::Cancel) !=
+				QMessageBox::Ok)
+		return;
 
 	foreach(index, indexes) {
 		if (index.column() != 0)
@@ -373,41 +501,55 @@ int db_base::rowCount(const QModelIndex &parent) const
 	return parentItem->childCount();
 }
 
-int db_base::columnCount(const QModelIndex &parent) const
+int db_base::columnCount(const QModelIndex &) const
 {
-	pki_base *item;
-	if (!parent.isValid()) {
-		if (headertext.count())
-			return headertext.count();
-		item = rootItem;
-	} else {
-		item = static_cast<pki_base*>(parent.internalPointer());
-	}
-	return item->columns();
+	return allHeaders.count();
 }
 
 QVariant db_base::data(const QModelIndex &index, int role) const
 {
 	if (!index.isValid())
 		return QVariant();
-
+	dbheader *hd = allHeaders[index.column()];
 	pki_base *item = static_cast<pki_base*>(index.internalPointer());
 	switch (role) {
 		case Qt::EditRole:
 		case Qt::DisplayRole:
-			return item->column_data(index.column());
+			return item->column_data(hd);
 		case Qt::DecorationRole:
-			return item->getIcon(index.column());
+			return item->getIcon(hd);
+		case Qt::TextAlignmentRole:
+			return hd->isNumeric() ? Qt::AlignRight : Qt::AlignLeft;
+		case Qt::FontRole: {
+			if (hd->isNumeric())
+				return QVariant(QFont("Monospace"));
+			return QVariant(QApplication::font());
+		}
 	}
 	return QVariant();
 }
-
+#if 0
+static QString getHeaderViewInfo(int sect, dbheader *h)
+{
+	return QString("H[%1] Show:%2%3 Size:%4 VI:%5 Indi:%6").
+		arg(sect).arg(h->show).arg(h->showDefault).arg(h->size).
+		arg(h->visualIndex).arg(h->sortIndicator);
+}
+#endif
 QVariant db_base::headerData(int section, Qt::Orientation orientation,
 		int role) const
 {
-	if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
-		return headertext[section];
-
+	if (orientation == Qt::Horizontal) {
+		switch (role) {
+		case Qt::DisplayRole:
+			return QVariant(allHeaders[section]->name);
+		case Qt::ToolTipRole:
+#if 0
+			return getHeaderViewInfo(section, allHeaders[section]);
+#endif
+			return QVariant(allHeaders[section]->tooltip);
+		}
+	}
 	return QVariant();
 }
 
@@ -428,9 +570,11 @@ bool db_base::setData(const QModelIndex &index, const QVariant &value, int role)
 	pki_base *item;
 	if (index.isValid() && role == Qt::EditRole) {
 		nn = value.toString();
-		db mydb(dbName);
 		item = static_cast<pki_base*>(index.internalPointer());
 		on = item->getIntName();
+		if (nn == on)
+			return true;
+		db mydb(dbName);
 		try {
 			mydb.rename(item->getType(), on, nn);
 			item->setIntName(nn);
@@ -445,19 +589,18 @@ bool db_base::setData(const QModelIndex &index, const QVariant &value, int role)
 
 void db_base::load_default(load_base &load)
 {
+	QString s;
 	QStringList slist = QFileDialog::getOpenFileNames(mainwin, load.caption,
 			mainwin->getPath(), load.filter);
 
 	if (!slist.count())
 		return;
 
-	QString fn = QDir::convertSeparators(slist[0]);
-	mainwin->setPath(fn.mid(0, fn.lastIndexOf(QRegExp("[/\\\\]")) ));
+	QString fn = slist[0];
+	mainwin->setPath(fn.mid(0, fn.lastIndexOf("/")));
 
 	ImportMulti *dlgi = new ImportMulti(mainwin);
-	for (QStringList::Iterator it = slist.begin(); it != slist.end(); ++it) {
-		QString s = *it;
-		s = QDir::convertSeparators(s);
+	foreach(s, slist) {
 		pki_base *item = NULL;
 		try {
 			item = load.loadItem(s);
@@ -476,7 +619,7 @@ void db_base::edit()
 {
 	if (!currentIdx.isValid())
 		return;
-	view->edit(view->getProxyIndex(currentIdx));
+	emit editItem(currentIdx);
 }
 
 void db_base::showItem()
@@ -486,3 +629,86 @@ void db_base::showItem()
 	showItem(currentIdx);
 }
 
+bool db_base::columnHidden(int col) const
+{
+	return !allHeaders[col]->show;
+}
+
+void db_base::columnResetDefaults()
+{
+	dbheader *hd;
+	foreach(hd, allHeaders) {
+		hd->reset();
+	}
+	emit resetHeader();
+}
+
+bool db_base::isNumericCol(int col) const
+{
+	return allHeaders[col]->isNumeric();
+}
+
+void db_base::showHeaderMenu(QContextMenuEvent *e, int sect)
+{
+	contextMenu(e, NULL, sect);
+}
+
+void db_base::contextMenu(QContextMenuEvent *e, QMenu *parent, int)
+{
+	int shown = 0;
+	QMenu *menu, *dn, *v3ext, *current;
+	QAction *a, *sep;
+	dbheader *hd;
+
+	menu = new QMenu(mainwin);
+	dn = new QMenu(tr("Subject entries"));
+	v3ext = new QMenu(tr("X509v3 Extensions"));
+	menu->addAction(tr("Reset"), this, SLOT(columnResetDefaults()));
+	sep = menu->addSeparator();
+	foreach(hd, allHeaders) {
+		switch (hd->type) {
+			case dbheader::hd_x509name:
+				current = dn;
+				break;
+			case dbheader::hd_v3ext:
+				current = v3ext;
+				break;
+			default:
+				current = menu;
+				break;
+		}
+		a = current->addAction(hd->name);
+		a->setCheckable(true);
+		a->setChecked(hd->show);
+		if (!hd->tooltip.isEmpty())
+			a->setToolTip(hd->tooltip);
+		hd->action = a;
+	}
+	if (!dn->isEmpty() || !v3ext->isEmpty())
+		menu->insertSeparator(sep);
+	if (!dn->isEmpty())
+		menu->insertMenu(sep, dn);
+	if (!v3ext->isEmpty())
+		menu->insertMenu(sep, v3ext);
+
+	if (parent) {
+		parent->addAction(tr("Paste PEM data"), mainwin,
+				SLOT(pastePem()));
+		parent->addMenu(menu)->setText(tr("Columns"));
+		parent->exec(e->globalPos());
+	} else {
+		menu->exec(e->globalPos());
+	}
+	foreach(hd, allHeaders) {
+		if (hd->action)
+			hd->show = hd->action->isChecked();
+		shown += hd->show ? 1 : 0;
+		hd->action = NULL;
+	}
+	if (!shown)
+		allHeaders[0]->show = true;
+        delete menu;
+	if (parent)
+		delete parent;
+	emit updateHeader();
+}
