@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -10,23 +10,29 @@
 #include "exception.h"
 #include "widgets/MainWindow.h"
 #include "widgets/CrlDetail.h"
-#include <qmessagebox.h>
-#include <qevent.h>
+#include "widgets/NewCrl.h"
+#include <QtGui/QMessageBox>
+#include <QtGui/QContextMenuEvent>
 #include "ui_NewCrl.h"
 
 db_crl::db_crl(QString db, MainWindow *mw)
-	:db_base(db,mw)
+	:db_x509name(db,mw)
 {
-	headertext << tr("Name") << tr("Signer") << tr("Common name") <<
-		tr("No. revoked") << tr("Next update");
-	delete_txt = tr("Delete the revokation list(s)");
-	view = mw->crlView;
+	allHeaders <<
+		new dbheader(HD_crl_signer,	true, tr("Signer"),
+			tr("Internal name of the signer")) <<
+		new dbheader(HD_crl_revoked,	true, tr("No. revoked"),
+			tr("Number of revoked certificates")) <<
+		new dbheader(HD_crl_lastUpdate, false,tr("Last update")) <<
+		new dbheader(HD_crl_nextUpdate,	true, tr("Next update")) <<
+		new dbheader(HD_crl_crlnumber,	false,tr("CRL number"));
+
 	class_name = "crls";
-	pkitype[0] = revokation;
+	pkitype << revokation;
 	loadContainer();
 }
 
-pki_base *db_crl::newPKI(db_header_t *head)
+pki_base *db_crl::newPKI(db_header_t *)
 {
 	return new pki_crl();
 }
@@ -40,19 +46,16 @@ void db_crl::load()
 void db_crl::revokeCerts(pki_crl *crl)
 {
 	int numc, i;
-	certs = mainwin->certs;
-	if (! certs)
+	if (!mainwin->certs)
 		return;
 	x509rev revok;
-	pki_x509 *rev;
+	pki_x509 *signer = crl->getIssuer();
+	if (!signer)
+		return;
 	numc = crl->numRev();
-
 	for (i=0; i<numc; i++) {
 		revok = crl->getRev(i);
-		rev = certs->getByIssSerial(crl->getIssuer(), revok.getSerial());
-		if (rev) {
-			rev->setRevoked(revok.getDate());
-		}
+		mainwin->certs->revokeCert(revok, signer);
 	}
 }
 
@@ -67,21 +70,23 @@ void db_crl::removeSigner(pki_base *signer)
 void db_crl::inToCont(pki_base *pki)
 {
 	pki_crl *crl = (pki_crl *)pki;
-	revokeCerts(crl);
 	if (crl->getIssuer() == NULL) {
 		pki_x509 *iss = NULL, *last = NULL;
-		x509name issname = crl->getIssuerName();
+		x509name issname = crl->getSubject();
 		while ((iss = mainwin->certs->getBySubject(issname, last)) != NULL) {
 			pki_key *key = iss->getPubKey();
-			if (crl->verify(key)) {
+			if (key) {
+				if (crl->verify(key)) {
+					delete key;
+					break;
+				}
 				delete key;
-				break;
 			}
-			delete key;
 			last = iss;
 		}
 		crl->setIssuer(iss);
 	}
+	revokeCerts(crl);
 	db_base::inToCont(pki);
 }
 
@@ -90,12 +95,10 @@ pki_base *db_crl::insert(pki_base *item)
 	pki_crl *crl = (pki_crl *)item;
 	pki_crl *oldcrl = (pki_crl *)getByReference(crl);
 	if (oldcrl) {
-		QMessageBox::information(NULL, XCA_TITLE,
-			tr("The revokation list already exists in the database as") +
-			":\n'" + oldcrl->getIntName() +
-			"'\n" + tr("and so it was not imported"), "OK");
+		QMessageBox::information(mainwin, XCA_TITLE,
+			tr("The revokation list already exists in the database as:\n'%1'\nand so it was not imported").arg(oldcrl->getIntName()));
 		delete(crl);
-		return oldcrl;
+		return NULL;
 	}
 	insertPKI(crl);
 	return crl;
@@ -156,67 +159,58 @@ pki_crl *db_crl::newItem(pki_x509 *cert)
 {
 	if (!cert)
 		return NULL;
-	QList<pki_x509*> list;
-	a1time time;
+
 	pki_crl *crl = NULL;
-	Ui::NewCrl ui;
+	NewCrl *dlg = new NewCrl(mainwin, cert);
 
-	x509v3ext e;
-	X509V3_CTX ext_ctx;
-	X509V3_set_ctx(&ext_ctx, cert->getCert() , NULL, NULL, NULL, 0);
-	X509V3_set_ctx_nodb((&ext_ctx));
-	QDialog *dlg = new QDialog(mainwin);
-
-	ui.setupUi(dlg);
-	ui.image->setPixmap(*MainWindow::revImg);
-	ui.lastUpdate->setDate(time.now());
-	ui.nextUpdate->setDate(time.now(cert->getCrlDays() * SECONDS_PER_DAY));
-
-	pki_key *key = cert->getRefKey();
-	ui.hashAlgo->setKeyType(key->getKeyType());
-	ui.hashAlgo->setupHashes(key->possibleHashNids());
-
-	if (cert->hasExtension(NID_subject_alt_name))
-		ui.subAltName->setEnabled(true);
-	else
-		ui.subAltName->setEnabled(false);
 	if (!dlg->exec()) {
 		delete dlg;
 		return NULL;
 	}
 	try {
+		x509v3ext e;
+		X509V3_CTX ext_ctx;
+		X509V3_set_ctx(&ext_ctx, cert->getCert(), NULL, NULL, NULL, 0);
+		X509V3_set_ctx_nodb(&ext_ctx);
+
 		crl = new pki_crl();
 		crl->createCrl(cert->getIntName(), cert);
 
-		list = mainwin->certs->getIssuedCerts(cert);
+		QList<pki_x509*> list = mainwin->certs->getIssuedCerts(cert);
+		bool reason = dlg->revokationReasons->isChecked();
 		for (int i =0; i<list.size(); i++) {
 			if (list.at(i)->isRevoked() ) {
-				crl->addRev(list.at(i)->getRev());
+				crl->addRev(list.at(i)->getRev(reason));
 			}
 		}
 
-		if (ui.authKeyId->isChecked()) {
+		if (dlg->authKeyId->isChecked()) {
 			crl->addV3ext(e.create(NID_authority_key_identifier,
 				"keyid,issuer", &ext_ctx));
 		}
-		if (ui.subAltName->isChecked()) {
+		if (dlg->subAltName->isChecked()) {
 			if (cert->hasExtension(NID_subject_alt_name)) {
 				crl->addV3ext(e.create(NID_issuer_alt_name,
 					"issuer:copy", &ext_ctx));
 			}
 		}
-
-		crl->setLastUpdate(ui.lastUpdate->getDate());
-		crl->setNextUpdate(ui.nextUpdate->getDate());
-		cert->setCrlExpiry(ui.nextUpdate->getDate());
-
-		crl->sign(cert->getRefKey(), ui.hashAlgo->currentHash());
+		if (dlg->crlNumber->isChecked()) {
+			crl->setCrlNumber(cert->getIncCrlNumber());
+		}
+		crl->setLastUpdate(dlg->lastUpdate->getDate());
+		crl->setNextUpdate(dlg->nextUpdate->getDate());
+		crl->sign(cert->getRefKey(), dlg->hashAlgo->currentHash());
+		cert->setCrlExpiry(dlg->nextUpdate->getDate());
 		mainwin->certs->updatePKI(cert);
-		insert(crl);
+		createSuccess(insert(crl));
 	}
 	catch (errorEx &err) {
 		MainWindow::Error(err);
+		if (crl)
+			delete crl;
+		crl = NULL;
 	}
+	delete dlg;
 	return crl;
 }
 
@@ -224,15 +218,18 @@ void db_crl::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 {
 	QMenu *menu = new QMenu(mainwin);
 	currentIdx = index;
+	QMenu *subExport;
 
 	menu->addAction(tr("Import"), this, SLOT(load()));
 	if (index != QModelIndex()) {
 		menu->addAction(tr("Rename"), this, SLOT(edit()));
-		menu->addAction(tr("Export"), this, SLOT(store()));
+		subExport = menu->addMenu(tr("Export"));
+		subExport->addAction(tr("Clipboard"), this,
+					SLOT(pem2clipboard()));
+		subExport->addAction(tr("File"), this, SLOT(store()));
 		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
 	}
-	menu->exec(e->globalPos());
-	delete menu;
+	contextMenu(e, menu);
 	currentIdx = QModelIndex();
 	return;
 }
