@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -10,15 +10,20 @@
 #include "pki_evp.h"
 
 #include "pki_scard.h"
-#include <qmessagebox.h>
-#include <qprogressbar.h>
-#include <qstatusbar.h>
-#include <qevent.h>
+#include <QtGui/QDialog>
+#include <QtGui/QLabel>
+#include <QtGui/QPushButton>
+
+#include <QtGui/QMessageBox>
+#include <QtGui/QProgressBar>
+#include <QtGui/QStatusBar>
+#include <QtGui/QContextMenuEvent>
+
 #include "exception.h"
 #include "ui_NewKey.h"
 #include "pkcs11.h"
 
-#include "widgets/MainWindow.h"
+#include "widgets/PwDialog.h"
 #include "widgets/ExportKey.h"
 #include "widgets/KeyDetail.h"
 #include "widgets/NewKey.h"
@@ -27,12 +32,13 @@ db_key::db_key(QString db, MainWindow *mw)
 	:db_base(db, mw)
 {
 	rootItem->setIntName("[key root]");
-	headertext << "Name" << "Type" << "Size" << "Use" << "Password";
-	delete_txt = tr("Delete the key(s)");
-	view = mw->keyView;
 	class_name = "keys";
-	pkitype[0] = asym_key;
-	pkitype[1] = smartCard;
+	pkitype << asym_key << smartCard;
+	allHeaders <<
+		new dbheader(HD_key_type,   true, tr("Type")) <<
+		new dbheader(HD_key_size,   true, tr("Size")) <<
+		new dbheader(HD_key_use,    true, tr("Use")) <<
+		new dbheader(HD_key_passwd, true, tr("Password"));
 	loadContainer();
 }
 
@@ -54,12 +60,11 @@ QStringList db_key::getPrivateDesc()
 	return x;
 }
 
-QStringList db_key::get0PrivateDesc(bool all)
+QStringList db_key::get0KeyDesc(bool all)
 {
 	QStringList x;
-	x.clear();
 	FOR_ALL_pki(pki, pki_key) {
-		if (pki->isPrivKey() && ((pki->getUcount() == 0) || all))
+		if ((pki->getUcount() == 0) || all)
 			x.append(pki->getIntNameWithType());
 	}
 	return x;
@@ -86,18 +91,14 @@ pki_base* db_key::insert(pki_base *item)
 	oldkey = (pki_key *)getByReference(lkey);
 	if (oldkey != NULL) {
 		if ((oldkey->isPrivKey() && lkey->isPrivKey()) || lkey->isPubKey()){
-			QMessageBox::information(NULL, tr(XCA_TITLE),
-			tr("The key is already in the database as") +":\n'" +
-			oldkey->getIntName() +
-			"'\n" + tr("and is not going to be imported"), "OK");
+			QMessageBox::information(mainwin, XCA_TITLE,
+			tr("The key is already in the database as:\n'%1'\nand is not going to be imported").arg(oldkey->getIntName()));
 			delete(lkey);
-			return oldkey;
+			return NULL;
 		}
 		else {
-			QMessageBox::information(NULL,tr(XCA_TITLE),
-			tr("The database already contains the public part of the imported key as") +":\n'" +
-			oldkey->getIntName() +
-			"'\n" + tr("and will be completed by the new, private part of the key"), "OK");
+			QMessageBox::information(mainwin, XCA_TITLE,
+			tr("The database already contains the public part of the imported key as\n'%1\nand will be completed by the new, private part of the key").arg(oldkey->getIntName()));
 			lkey->setIntName(oldkey->getIntName());
 			currentIdx = index(oldkey->row(), 0, QModelIndex());
 			deletePKI();
@@ -119,6 +120,8 @@ void db_key::newItem(QString name)
 	QProgressBar *bar;
 	QStatusBar *status = mainwin->statusBar();
 	pki_evp *nkey = NULL;
+	pki_scard *cardkey = NULL;
+	pki_key *key = NULL;
 
 	if (!dlg->exec()) {
 		delete dlg;
@@ -133,25 +136,34 @@ void db_key::newItem(QString name)
 			return;
 		}
 		if (ksize < 1024 || ksize > 8192)
-			if (!QMessageBox::warning(NULL, XCA_TITLE,
-				tr("You are sure to create a key of the size: ")
-				+QString::number(ksize) + " ?", tr("Cancel"),
-				tr("Create") ))
+			if (QMessageBox::warning(NULL, XCA_TITLE,
+				tr("You are sure to create a key of the size: %1 ?").arg(ksize),
+				QMessageBox::Yes | QMessageBox::No) !=
+				QMessageBox::Yes)
 			{
 				delete dlg;
 				return;
 			}
 	}
 	mainwin->repaint();
-	nkey = new pki_evp(dlg->keyDesc->text());
 	bar = new QProgressBar();
 	status->addPermanentWidget(bar, 1);
 	try {
-		nkey->generate(ksize, dlg->getKeytype(), bar,dlg->getKeyCurve_nid());
-		nkey = (pki_evp*)insert(nkey);
-		emit keyDone(nkey->getIntNameWithType());
+		if (dlg->isToken()) {
+			key = cardkey = new pki_scard(dlg->keyDesc->text());
+			cardkey->generateKey_card(dlg->getKeyCardSlot(),
+						 ksize, bar);
+		} else {
+			key = nkey = new pki_evp(dlg->keyDesc->text());
+			nkey->generate(ksize, dlg->getKeytype(), bar,
+				dlg->getKeyCurve_nid());
+		}
+		key = (pki_key*)insert(key);
+		emit keyDone(key->getIntNameWithType());
+		createSuccess(key);
+
 	} catch (errorEx &err) {
-		delete nkey;
+		delete key;
 		mainwin->Error(err);
 	}
 	status->removeWidget(bar);
@@ -165,6 +177,37 @@ void db_key::load(void)
 	load_default(l);
 }
 
+void db_key::toToken()
+{
+	pki_key *key = static_cast<pki_scard*>(currentIdx.internalPointer());
+	if (!key || !pkcs11::loaded() || key->isToken())
+		return;
+
+	pki_scard *card = NULL;
+	try {
+		pkcs11 p11;
+		slotid slot;
+
+		if (!p11.selectToken(&slot, mainwin))
+			return;
+		card = new pki_scard(key->getIntName());
+		card->store_token(slot, key->decryptKey());
+		QString msg = tr("Shall the original key '%1' be replaced by the key on the token?\nThis will delete the key '%1' and make it unexportable").
+			arg(key->getIntName());
+		if (QMessageBox::question(mainwin, XCA_TITLE, msg,
+		    QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
+		{
+			deletePKI();
+			insertPKI(card);
+			card = NULL;
+		}
+	} catch (errorEx &err) {
+		mainwin->Error(err);
+        }
+	if (card)
+		delete card;
+}
+
 void db_key::showPki(pki_base *pki)
 {
 	pki_evp *key = (pki_evp *)pki;
@@ -190,7 +233,7 @@ void db_key::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 		menu->addAction(tr("Show Details"), this, SLOT(showItem()));
 		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
 		menu->addAction(tr("Export"), this, SLOT(store()));
-		if (key->isPrivKey() && !key->isScard()) {
+		if (key->isPrivKey() && !key->isToken()) {
 			switch (key->getOwnPass()) {
 			case pki_key::ptCommon:
 				menu->addAction(tr("Change password"), this,
@@ -202,19 +245,20 @@ void db_key::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 				break;
 			}
 		}
-		if (key->isScard() && pkcs11::loaded()) {
+		if (key->isToken() && pkcs11::loaded()) {
 			menu->addAction(tr("Change PIN"), this,
 				SLOT(changePin()));
-#if 0
 			menu->addAction(tr("Init PIN with SO PIN (PUK)"), this,
 				SLOT(initPin()));
 			menu->addAction(tr("Change SO PIN (PUK)"), this,
 				SLOT(changeSoPin()));
-#endif
+		}
+		if (!key->isToken() && pkcs11::loaded()) {
+			menu->addAction(tr("Store on Security token"),
+				this, SLOT(toToken()));
 		}
 	}
-	menu->exec(e->globalPos());
-	delete menu;
+	contextMenu(e, menu);
 	currentIdx = QModelIndex();
 	return;
 }
@@ -233,8 +277,12 @@ void db_key::store()
 			targetKey->getUnderlinedName() + ".pem";
 
 	ExportKey *dlg = new ExportKey(mainwin, fn,
-		targetKey->isPubKey() || targetKey->isScard());
-	dlg->image->setPixmap(*MainWindow::keyImg);
+		targetKey->isPubKey() || targetKey->isToken());
+	if (targetKey->isToken())
+		dlg->image->setPixmap(*MainWindow::scardImg);
+	else
+		dlg->image->setPixmap(*MainWindow::keyImg);
+
 
 	if (!dlg->exec()) {
 		delete dlg;
@@ -250,12 +298,12 @@ void db_key::store()
 		pem = dlg->exportFormat->currentText() == "PEM" ? true : false;
 		if (dlg->encryptKey->isChecked())
 			enc = EVP_des_ede3_cbc();
-		if (dlg->exportPrivate->isChecked() && !targetKey->isScard()) {
+		if (dlg->exportPrivate->isChecked() && !targetKey->isToken()) {
 			pki_evp *evpKey = (pki_evp *)targetKey;
 			if (dlg->exportPkcs8->isChecked()) {
-				evpKey->writePKCS8(fname, enc, &MainWindow::passWrite, pem);
+				evpKey->writePKCS8(fname, enc, PwDialog::pwCallback, pem);
 			} else {
-				evpKey->writeKey(fname, enc, &MainWindow::passWrite, pem);
+				evpKey->writeKey(fname, enc, PwDialog::pwCallback, pem);
 			}
 		} else {
 			targetKey->writePublic(fname, pem);
@@ -294,8 +342,8 @@ void db_key::__setOwnPass(enum pki_key::passType x)
 	if (!currentIdx.isValid())
 		        return;
 	targetKey = static_cast<pki_evp*>(currentIdx.internalPointer());
-	if (targetKey->isScard()) {
-		throw errorEx(tr("Tried to change password of a smart card"));
+	if (targetKey->isToken()) {
+		throw errorEx(tr("Tried to change password of a token"));
 	}
 	targetKey->setOwnPass(x);
 	updatePKI(targetKey);
@@ -308,7 +356,7 @@ void db_key::changePin()
 		        return;
 	scard = static_cast<pki_scard*>(currentIdx.internalPointer());
 	try {
-		if (!scard->isScard()) {
+		if (!scard->isToken()) {
 			throw errorEx(tr("Tried to change PIN of a key"));
 		}
 		scard->changePin();
@@ -324,7 +372,7 @@ void db_key::initPin()
 		        return;
 	scard = static_cast<pki_scard*>(currentIdx.internalPointer());
 	try {
-		if (!scard->isScard()) {
+		if (!scard->isToken()) {
 			throw errorEx(tr("Tried to init PIN of a key"));
 		}
 		scard->initPin();
@@ -340,7 +388,7 @@ void db_key::changeSoPin()
 		        return;
 	scard = static_cast<pki_scard*>(currentIdx.internalPointer());
 	try {
-		if (!scard->isScard()) {
+		if (!scard->isToken()) {
 			throw errorEx(tr("Tried to change SO PIN of a key"));
 		}
 		scard->changeSoPin();
