@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -9,17 +9,20 @@
 #include "pki_pkcs12.h"
 #include "pki_pkcs7.h"
 #include "pki_evp.h"
+#include "pki_scard.h"
+#include "pass_info.h"
 #include "widgets/CertDetail.h"
 #include "widgets/CertExtend.h"
-#include "widgets/ExportCert.h"
+#include "widgets/ExportDialog.h"
 #include "widgets/MainWindow.h"
+#include "widgets/PwDialog.h"
 #include "ui_TrustState.h"
 #include "ui_CaProperties.h"
-#include "ui_PassWrite.h"
 #include "ui_About.h"
-#include <qmessagebox.h>
-#include <qevent.h>
-#include <qaction.h>
+#include "ui_Revoke.h"
+#include <QtGui/QMessageBox>
+#include <QtGui/QContextMenuEvent>
+#include <QtGui/QAction>
 
 bool db_x509::treeview = true;
 
@@ -27,17 +30,25 @@ db_x509::db_x509(QString DBfile, MainWindow *mw)
 	:db_x509super(DBfile, mw)
 {
 	rootItem->setIntName("[x509 root]");
-	headertext << tr("Internal name") << tr("Common name") << tr("Serial") <<
-			tr("not After") << tr("Trust state") << tr("Revocation");
+	allHeaders <<
+		new dbheader(HD_cert_ca,	true, tr("CA"),
+			tr("reflects the basic Constraints extension")) <<
+		new dbheader(HD_cert_serial,	true, tr("Serial")) <<
+		new dbheader(HD_cert_md5fp,	false,tr("md5 fingerprint")) <<
+		new dbheader(HD_cert_sha1fp,	false,tr("sha1 fingerprint")) <<
+		new dbheader(HD_cert_notBefore, false,tr("Start date"),
+				tr("not Before")) <<
+		new dbheader(HD_cert_notAfter,	true, tr("Expiry date"),
+				tr("not After")) <<
+		new dbheader(HD_cert_trust,	false,tr("Trust state")) <<
+		new dbheader(HD_cert_revokation,true, tr("Revocation"));
 
-	delete_txt = tr("Delete the certificate(s)");
-	view = mw->certView;
 	class_name = "certificates";
-	pkitype[0] = x509;
+	pkitype << x509;
 	loadContainer();
 }
 
-pki_base *db_x509::newPKI(db_header_t *head)
+pki_base *db_x509::newPKI(db_header_t *)
 {
 	return new pki_x509();
 }
@@ -99,6 +110,16 @@ void db_x509::remFromCont(QModelIndex &idx)
 		insertChild(new_parent, child);
 	}
 	mainwin->crls->removeSigner(pki);
+	pki_key *pub = ((pki_x509*)pki)->getPubKey();
+	if (pub) {
+		if (mainwin->certs->findByPubKey(pub).count() == 0) {
+			QList<pki_x509super *> reqs;
+			reqs = mainwin->reqs->findByPubKey(pub);
+			foreach(pki_x509super *r, reqs)
+				((pki_x509req*)r)->setDone(false);
+		}
+		delete pub;
+	}
 	return;
 }
 
@@ -111,6 +132,7 @@ void db_x509::changeView()
 		return;
 
 	temproot = new pki_base();
+	mainwin->certView->hide();
 	mainwin->certView->setModel(NULL);
 	beginRemoveRows(QModelIndex(), 0, rows -1);
 	pki_base *pki = rootItem;
@@ -137,6 +159,7 @@ void db_x509::changeView()
 	}
 	delete temproot;
 	mainwin->certView->setModel(this);
+	mainwin->certView->show();
 }
 
 void db_x509::calcEffTrust()
@@ -188,10 +211,13 @@ void db_x509::inToCont(pki_base *pki)
 	}
 	findKey(cert);
 	pki_key *pub = cert->getPubKey();
-	pki_x509req *req = (pki_x509req *)mainwin->reqs->findByByPubKey(pub);
-	delete pub;
-	if (req) {
-		req->setDone();
+	if (pub) {
+		QList<pki_x509super *> reqs = mainwin->reqs->findByPubKey(pub);
+		delete pub;
+
+		foreach (pki_x509super *r, reqs) {
+			((pki_x509req*)r)->setDone();
+		}
 	}
 	calcEffTrust();
 }
@@ -271,29 +297,49 @@ a1int db_x509::searchSerial(pki_x509 *signer)
 	// returns the highest certificate serial
 	// of all certs with this signer (itself too)
 	a1int sserial, myserial;
-	if (!signer) return sserial;
+	if (!signer)
+		return sserial;
 	sserial = signer->getCaSerial();
 	FOR_ALL_pki(pki, pki_x509)
 		if (pki->getSigner() == signer)  {
 			myserial = pki->getSerial();
-			if (sserial < myserial ) {
+			if (sserial < myserial) {
 				sserial = myserial;
 			}
 		}
 	return sserial;
 }
 
+a1int db_x509::getUniqueSerial(pki_x509 *signer)
+{
+	// returnes an unused unique serial
+	a1int serial;
+	bool dup;
+	do {
+		dup = false;
+		serial = signer->getIncCaSerial();
+		FOR_ALL_pki(pki, pki_x509)
+			if (pki->getSigner() == signer)  {
+				if (serial == pki->getSerial()) {
+					dup = true;
+					break;
+				}
+			}
+	} while (dup);
+	if (!signer->usesRandomSerial())
+		updatePKI(signer);
+	return serial;
+}
+
 pki_base *db_x509::insert(pki_base *item)
 {
 	pki_x509 *cert = (pki_x509 *)item;
 	pki_x509 *oldcert = (pki_x509 *)getByReference(cert);
 	if (oldcert) {
 		QMessageBox::information(mainwin, XCA_TITLE,
-		tr("The certificate already exists in the database as") +":\n'" +
-		oldcert->getIntName() +
-		"'\n" + tr("and so it was not imported"), "OK");
+		tr("The certificate already exists in the database as:\n'%1'\nand so it was not imported").arg(oldcert->getIntName()));
 		delete(cert);
-		return oldcert;
+		return NULL;
 	}
 	cert->setCaSerial((cert->getSerial()));
 	insertPKI(cert);
@@ -400,29 +446,29 @@ void db_x509::newCert(NewX509 *dlg)
 
 	// Step 1 - Subject and key
 	if (!dlg->fromReqCB->isChecked()) {
-	    clientkey = dlg->getSelectedKey();
+		clientkey = dlg->getSelectedKey();
 		if (!clientkey)
 			return;
-	    subject = dlg->getX509name();
-	    intname = dlg->description->text();
-	}
-	else {
-	    // A PKCS#10 Request was selected
-	    req = dlg->getSelectedReq();
-	    if (!req)
-			return;
-	    clientkey = req->getRefKey();
-	    if (clientkey == NULL) {
-		    clientkey = req->getPubKey();
-		    tempkey = clientkey;
-	    }
-	    if (dlg->reqSubChange->isChecked())
 		subject = dlg->getX509name();
-	    else
-	        subject = req->getSubject();
-	    intname = req->getIntName();
+		intname = dlg->description->text();
+	} else {
+		// A PKCS#10 Request was selected
+		req = dlg->getSelectedReq();
+		if (!req)
+			return;
+		clientkey = req->getRefKey();
+		if (clientkey == NULL) {
+			clientkey = req->getPubKey();
+			tempkey = clientkey;
+		}
+		if (dlg->reqSubChange->isChecked())
+			subject = dlg->getX509name();
+		else
+			subject = req->getSubject();
+		intname = req->getIntName();
 	}
-
+	if (clientkey == NULL)
+		throw errorEx(tr("Invalid public key"));
 	// initially create cert
 	cert = new pki_x509();
 	cert->setIntName(intname);
@@ -434,39 +480,24 @@ void db_x509::newCert(NewX509 *dlg)
 		signcert = dlg->getSelectedSigner();
 		if (!signcert)
 			return;
-		serial = signcert->getIncCaSerial();
+		serial = getUniqueSerial(signcert);
 		signkey = signcert->getRefKey();
 		cert->setTrust(1);
 #ifdef WG_QA_SERIAL
 	} else if (dlg->selfQASignRB->isChecked()){
-		Ui::PassWrite ui;
-		QDialog *dlg1 = new QDialog(mainwin);
-		ui.setupUi(dlg1);
-		ui.image->setPixmap( *MainWindow::keyImg );
-		ui.description->setText(tr("Please enter the new hexadecimal secret number for the QA process."));
-		dlg1->setWindowTitle(XCA_TITLE);
-		ui.passA->setFocus();
+		Passwd pass;
+		pass_info p(XCA_TITLE, tr("Please enter the new hexadecimal secret number for the QA process."));
+#if 0
 		ui.passA->setValidator(new QRegExpValidator(QRegExp("[0-9a-fA-F]*"),
 				ui.passA));
 		ui.passB->setValidator(new QRegExpValidator(QRegExp("[0-9a-fA-F]*"),
 				ui.passB));
-		QString A = "x", B="";
-
-		while (dlg1->exec()) {
-			A = ui.passA->text();
-			B = ui.passB->text();
-			if (A==B)
-				break;
-			else
-				QMessageBox::warning(mainwin, XCA_TITLE,
-						tr("The two secret numbers don't match."));
-		}
-		delete dlg1;
-		if (A!=B)
+#endif
+		if (PwDialog::execute(&p, &pass, true) != 1)
 			throw errorEx(tr("The QA process has been terminated by the user."));
 		signcert = cert;
 		signkey = clientkey;
-		serial.setHex(A);
+		serial.setHex(pass);
 		cert->setTrust(2);
 #endif
 	} else {
@@ -479,6 +510,7 @@ void db_x509::newCert(NewX509 *dlg)
 	dlg->initCtx(cert, signcert, NULL);
 	// if we can not sign
 	if (! signkey || signkey->isPubKey()) {
+		free(cert);
 		throw errorEx(tr("The key you selected for signing is not a private one."));
 	}
 
@@ -497,14 +529,6 @@ void db_x509::newCert(NewX509 *dlg)
 
 	cert->setNotAfter(a);
 
-	if (cert->resetTimes(signcert) > 0) {
-		if (QMessageBox::information(mainwin,tr(XCA_TITLE),
-			tr("The validity times for the certificate need to get adjusted to not exceed those of the signer"),
-			tr("Continue creation"), tr("Abort")
-		))
-			throw errorEx("");
-	}
-
 	// STEP 4 handle extensions
 	if (dlg->copyReqExtCB->isChecked() && dlg->fromReqCB->isChecked()) {
 		extList el = req->getV3ext();
@@ -528,10 +552,27 @@ void db_x509::newCert(NewX509 *dlg)
 #endif
 	// and finally sign the request
 	cert->sign(signkey, hashAlgo);
-	insert(cert);
+
+	cert = (pki_x509*)insert(cert);
+	createSuccess(cert);
 	updatePKI(signcert);
+	if (cert && clientkey->isToken()) {
+		pki_scard *card = (pki_scard*)clientkey;
+		if (QMessageBox::information(mainwin, XCA_TITLE,
+			tr("Store the certificate to the key on the token '%1 (#%2)' ?").
+			arg(card->getCardLabel()).arg(card->getSerial()),
+			QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
+		{
+			try {
+				cert->store_token(false);
+			} catch (errorEx &err) {
+				mainwin->Error(err);
+			}
+		}
+	}
 	if (tempkey != NULL)
 		delete(tempkey);
+		tempkey = NULL;
     }
 
     catch (errorEx &err) {
@@ -551,7 +592,7 @@ void db_x509::showPki(pki_base *pki)
 		dlg->setCert(crt);
 		connect( dlg->privKey, SIGNAL( doubleClicked(QString) ),
 			mainwin->keys, SLOT( showItem(QString) ));
-		connect( dlg->signCert, SIGNAL( doubleClicked(QString) ),
+		connect( dlg->signature, SIGNAL( doubleClicked(QString) ),
 			this, SLOT( showItem(QString) ));
 		dlg->exec();
 		delete dlg;
@@ -562,8 +603,10 @@ void db_x509::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 {
 	QMenu *menu = new QMenu(mainwin);
 	QMenu *subExport, *subCa;
-	QAction *itemReq, *itemRevoke, *itemExtend, *itemTrust, *itemScard;
-	bool parentCanSign, canSign, hasTemplates;
+	QAction *itemReq, *itemRevoke, *itemExtend, *itemTrust, *action;
+	QList<QAction*> scardItems;
+
+	bool parentCanSign, canSign, hasScard;
 	currentIdx = index;
 	pki_key *privkey;
 
@@ -574,18 +617,31 @@ void db_x509::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 	menu->addAction(tr("Import PKCS#12"), this, SLOT(loadPKCS12()));
 	menu->addAction(tr("Import from PKCS#7"), this, SLOT(loadPKCS7()));
 	if (index != QModelIndex()) {
+		privkey = cert->getRefKey();
 		menu->addAction(tr("Rename"), this, SLOT(edit()));
 		menu->addAction(tr("Show Details"), this, SLOT(showItem()));
+		if (!privkey)
+			menu->addAction(tr("Extract public Key"),
+				this, SLOT(extractPubkey()));
 		subExport = menu->addMenu(tr("Export"));
+		subExport->addAction(tr("Clipboard"), this,
+					SLOT(pem2clipboard()));
 		subExport->addAction(tr("File"), this, SLOT(store()));
 		itemReq = subExport->addAction(tr("Request"),
 				this, SLOT(toRequest()));
-		itemScard = subExport->addAction(tr("Smart Card"),
-				this, SLOT(toScard()));
+		scardItems += subExport->addAction(tr("Security token"),
+				this, SLOT(toToken()));
+		scardItems += subExport->addAction(tr("Other token"),
+				this, SLOT(toOtherToken()));
 		subExport->addAction(tr("Template"), this, SLOT(toTemplate()));
+		subExport->addAction(tr("OpenSSL config"),
+				this, SLOT(toOpenssl()));
 
 		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
-		itemTrust = menu->addAction(tr("Trust"), this, SLOT(setTrust()));
+		scardItems += menu->addAction(
+				tr("Delete from Security token"),
+				this, SLOT(deleteFromToken()));
+		itemTrust = menu->addAction(tr("Trust"), this,SLOT(setTrust()));
 		menu->addSeparator();
 		subCa = menu->addMenu(tr("CA"));
 		subCa->addAction(tr("Properties"), this, SLOT(caProperties()));
@@ -609,21 +665,20 @@ void db_x509::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 		parentCanSign = (cert->getSigner() && cert->getSigner()->canSign()
 					&& (cert->getSigner() != cert));
 		canSign = cert->canSign();
-		hasTemplates = mainwin->temps->getDesc().count() > 0 ;
-		privkey = cert->getRefKey();
+		hasScard = pkcs11::loaded();
+
 		itemRevoke->setEnabled(parentCanSign);
 		itemExtend->setEnabled(parentCanSign);
 		subCa->setEnabled(canSign);
-		itemReq->setEnabled(privkey);
-		itemScard->setEnabled(privkey &&
-				privkey->isScard() &&
-				pkcs11::loaded());
+		itemReq->setEnabled(privkey && privkey->isPrivKey());
+
+		foreach(action, scardItems)
+			action->setEnabled(hasScard);
 #if 0
 		subP7->setEnabled(privkey);
 #endif
 	}
-	menu->exec(e->globalPos());
-	delete menu;
+	contextMenu(e, menu);
 	currentIdx = QModelIndex();
 	return;
 }
@@ -650,7 +705,7 @@ void db_x509::store()
 	QString fn = mainwin->getPath() + QDir::separator() +
 			crt->getUnderlinedName() + ".crt";
 	ExportCert *dlg = new ExportCert(mainwin, fn,
-		(privkey && privkey->isPrivKey()) && !privkey->isScard());
+		(privkey && privkey->isPrivKey()) && !privkey->isToken());
 	dlg->image->setPixmap(*MainWindow::certImg);
 	int dlgret = dlg->exec();
 
@@ -710,15 +765,15 @@ void db_x509::store()
 		case 11: // Certificate and Key in PEM format for apache
 			pki_evp *privkey = (pki_evp *)crt->getRefKey();
 			if (!privkey || privkey->isPubKey()) {
-				QMessageBox::warning(mainwin, tr(XCA_TITLE),
-					tr("There was no key found for the Certificate: ") +
-					crt->getIntName() );
+				QMessageBox::warning(mainwin, XCA_TITLE,
+					tr("There was no key found for the Certificate: '%1'").
+					arg(crt->getIntName()));
 				return;
 			}
-			if (privkey->isScard()) {
-				QMessageBox::warning(mainwin, tr(XCA_TITLE),
-					tr("Not possible for smart card key:") +
-                                        crt->getIntName() );
+			if (privkey->isToken()) {
+				QMessageBox::warning(mainwin, XCA_TITLE,
+					tr("Not possible for a token key: '%1'").
+					arg(crt->getIntName()));
                                 return;
                         }
 
@@ -743,22 +798,20 @@ void db_x509::writePKCS12(pki_x509 *cert, QString s, bool chain)
     try {
 		pki_evp *privkey = (pki_evp *)cert->getRefKey();
 		if (!privkey || privkey->isPubKey()) {
-			QMessageBox::warning(mainwin, tr(XCA_TITLE),
-				tr("There was no key found for the Certificate: ") +
-				cert->getIntName() );
+			QMessageBox::warning(mainwin, XCA_TITLE,
+				tr("There was no key found for the Certificate: '%1'").arg(cert->getIntName()));
 			return;
 		}
-		if (privkey->isScard()) {
-			QMessageBox::warning(mainwin, tr(XCA_TITLE),
-				tr("Not possible for Smart card key for the Certificate: ") +
-				cert->getIntName() );
+		if (privkey->isToken()) {
+			QMessageBox::warning(mainwin, XCA_TITLE,
+				tr("Not possible for the token-key Certificate '%1'").
+				arg(cert->getIntName()));
 			return;
 		}
 		if (s.isEmpty())
 			return;
 		s = QDir::convertSeparators(s);
-		pki_pkcs12 *p12 = new pki_pkcs12(cert->getIntName(), cert, privkey,
-				MainWindow::passWrite);
+		pki_pkcs12 *p12 = new pki_pkcs12(cert->getIntName(), cert, privkey);
 		pki_x509 *signer = cert->getSigner();
 		while ((signer != NULL ) && (signer != cert) && chain) {
 			p12->addCaCert(signer);
@@ -903,6 +956,18 @@ void db_x509::setMultiTrust(QAbstractItemView* view)
 	currentIdx = QModelIndex();
 }
 
+void db_x509::deleteFromToken()
+{
+	if (!currentIdx.isValid())
+		return;
+	pki_x509 *pki = static_cast<pki_x509*>(currentIdx.internalPointer());
+	try {
+		pki->deleteFromToken();
+	} catch (errorEx &err) {
+		mainwin->Error(err);
+	}
+}
+
 void db_x509::setTrust()
 {
 	int state, newstate = 0;
@@ -942,50 +1007,47 @@ void db_x509::extendCert()
 	pki_key *signkey = NULL;
 	a1time time;
 	a1int serial;
+	CertExtend *dlg = NULL;
+
 	try {
-		CertExtend *dlg = new CertExtend(mainwin);
-		if (!dlg->exec()) {
-			delete dlg;
-			return;
-		}
 		oldcert = static_cast<pki_x509*>(currentIdx.internalPointer());
 		if (!oldcert ||
 				!(signer = oldcert->getSigner()) ||
 				!(signkey = signer->getRefKey()) ||
 				signkey->isPubKey())
 			return;
-		newcert = new pki_x509(oldcert);
-		serial = signer->getIncCaSerial();
 
-		// get signers own serial to avoid having the same
-		if (serial == signer->getSerial()) {
-			serial = signer->getIncCaSerial(); // just take the next one
+		CertExtend *dlg = new CertExtend(mainwin, signer);
+		if (!dlg->exec()) {
+			delete dlg;
+			return;
 		}
-		updatePKI(signer);
+		newcert = new pki_x509(oldcert);
+		serial = getUniqueSerial(signer);
 
 		// change date and serial
 		newcert->setSerial(serial);
 		newcert->setNotBefore(dlg->notBefore->getDate());
-		newcert->setNotAfter(dlg->notAfter->getDate());
+		a1time a;
+		if (dlg->noWellDefinedExpDate->isChecked())
+			a.setUndefined();
+		else
+			a = dlg->notAfter->getDate();
 
-		if (newcert->resetTimes(signer) > 0) {
-			if (QMessageBox::information(mainwin, tr(XCA_TITLE),
-				tr("The validity times for the certificate need to get adjusted to not exceed those of the signer"),
-				tr("Continue creation"), tr("Abort")
-			))
-				throw errorEx("");
-		}
+		newcert->setNotAfter(a);
 
-		// and finally sign the request
+		// and finally sign the cert
 		newcert->sign(signkey, oldcert->getDigest());
-		insert(newcert);
-		delete dlg;
+		newcert = (pki_x509 *)insert(newcert);
+		createSuccess(newcert);
 	}
 	catch (errorEx &err) {
 		MainWindow::Error(err);
 		if (newcert)
 			delete newcert;
 	}
+	if (dlg)
+		delete dlg;
 }
 
 
@@ -994,8 +1056,16 @@ void db_x509::revoke()
 	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
 	if (!cert)
 		return;
-	cert->setRevoked(true);
-	updatePKI(cert);
+	Ui::Revoke ui;
+	QDialog *revoke = new QDialog(mainwin, 0);
+	ui.setupUi(revoke);
+	ui.invalid->setNow();
+	ui.reason->addItems(x509rev::crlreasons());
+	if (revoke->exec()) {
+		cert->setRevoked(true, ui.invalid->getDate(),
+					ui.reason->currentText());
+		updatePKI(cert);
+	}
 }
 
 void db_x509::unRevoke()
@@ -1026,25 +1096,35 @@ void db_x509::toRequest()
 		req->setIntName(cert->getIntName());
 		req->createReq(cert->getRefKey(), cert->getSubject(),
 			cert->getRefKey()->getDefaultMD(), cert->getV3ext());
-		mainwin->reqs->insert(req);
+		createSuccess(mainwin->reqs->insert(req));
 	}
 	catch (errorEx &err) {
 		mainwin->Error(err);
 	}
 }
 
-void db_x509::toScard()
+void db_x509::myToToken(bool alwaysSelect)
 {
 	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
 	if (!cert)
 		return;
 	try {
-		cert->store_token();
+		cert->store_token(alwaysSelect);
 	} catch (errorEx &err) {
 		mainwin->Error(err);
         }
 }
 
+void db_x509::toToken()
+{
+	myToToken(false);
+}
+
+void db_x509::toOtherToken()
+{
+	myToToken(true);
+}
+
 void db_x509::caProperties()
 {
 	Ui::CaProperties ui;
@@ -1055,6 +1135,8 @@ void db_x509::caProperties()
 	QDialog *dlg = new QDialog(mainwin);
 	ui.setupUi(dlg);
 	ui.serial->setText(cert->getCaSerial().toHex());
+	ui.serial->setDisabled(cert->usesRandomSerial());
+	ui.randomSerial->setChecked(cert->usesRandomSerial());
 	ui.days->setSuffix(tr(" days"));
 	ui.days->setMaximum(1000000);
 	ui.days->setValue(cert->getCrlDays());
@@ -1067,14 +1149,14 @@ void db_x509::caProperties()
 	}
 	ui.temp->addItems(tempList);
 	ui.temp->setCurrentIndex(i);
-	ui.certName->setText(cert->getIntName());
+	ui.certName->setTitle(cert->getIntName());
 	if (dlg->exec()) {
 		a1int nserial;
 		cert->setCrlDays(ui.days->value());
 		nserial.setHex(ui.serial->text());
-		if (nserial > cert->getCaSerial())
-			cert->setCaSerial(nserial);
+		cert->setCaSerial(nserial);
 		cert->setTemplate(ui.temp->currentText());
+		cert->setUseRandomSerial(ui.randomSerial->isChecked());
 		updatePKI(cert);
 	}
 	delete dlg;
