@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -8,25 +8,28 @@
 
 #include "db_x509req.h"
 #include "pki_x509req.h"
-#include "widgets/ReqDetail.h"
+#include "widgets/CertDetail.h"
 #include "widgets/MainWindow.h"
-#include <qmessagebox.h>
-#include <qevent.h>
-#include <qaction.h>
+#include <QtGui/QMessageBox>
+#include <QtGui/QContextMenuEvent>
+#include <QtGui/QAction>
 
 
 db_x509req::db_x509req(QString DBfile, MainWindow *mw)
 	:db_x509super(DBfile, mw)
 {
-	headertext << tr("Name") << tr("Subject") << tr("Signed");
-	delete_txt = tr("Delete the request(s)");
-	view = mw->reqView;
+	allHeaders << new dbheader(HD_req_signed, true, tr("Signed"),
+			tr("whether the request is already signed or not")) <<
+		new dbheader(HD_req_unstr_name, false, tr("Unstructured name"),
+			QString(OBJ_nid2ln(NID_pkcs9_unstructuredName))) <<
+		new dbheader(HD_req_chall_pass, false, tr("Challenge password"),
+			 QString(OBJ_nid2ln(NID_pkcs9_challengePassword)));
 	class_name = "requests";
-	pkitype[0] = x509_req;
+	pkitype << x509_req;
 	loadContainer();
 }
 
-pki_base *db_x509req::newPKI(db_header_t *head)
+pki_base *db_x509req::newPKI(db_header_t *)
 {
 	return new pki_x509req();
 }
@@ -37,12 +40,10 @@ pki_base *db_x509req::insert(pki_base *item)
 	req = (pki_x509req *)item;
 	oldreq = (pki_x509req *)getByReference(req);
 	if (oldreq) {
-		QMessageBox::information(NULL,tr(XCA_TITLE),
-		tr("The certificate signing request already exists in the database as") +":\n'" +
-		oldreq->getIntName() +
-		"'\n" + tr("and thus was not stored"), "OK");
+		QMessageBox::information(mainwin, XCA_TITLE,
+		tr("The certificate signing request already exists in the database as\n'%1'\nand thus was not stored").arg(oldreq->getIntName()));
 		delete(req);
-		return oldreq;
+		return NULL;
 	}
 	insertPKI(req);
 	return req;
@@ -72,7 +73,7 @@ void db_x509req::newItem(pki_temp *temp)
 		dlg->initCtx(NULL, NULL, req);
 		dlg->addReqAttributes(req);
 		req->createReq(key, xn, dlg->hashAlgo->currentHash(), dlg->getAllExt());
-		insert(req);
+		createSuccess(insert(req));
 	}
 	catch (errorEx &err) {
 		MainWindow::Error(err);
@@ -89,10 +90,12 @@ void db_x509req::inToCont(pki_base *pki)
 	if (!mainwin->certs)
 		return;
 	pki_key *pub = req->getPubKey();
-	pki_x509super *cert = mainwin->certs->findByByPubKey(pub);
-	delete pub;
-	if (cert) {
-		req->setDone();
+	if (pub) {
+		int certs = mainwin->certs->findByPubKey(pub).count();
+		delete pub;
+		if (certs > 0) {
+			req->setDone();
+		}
 	}
 }
 
@@ -105,11 +108,11 @@ void db_x509req::load(void)
 void db_x509req::showPki(pki_base *pki)
 {
 	pki_x509req *req = (pki_x509req *)pki;
-	ReqDetail *dlg;
-	dlg = new ReqDetail(mainwin);
+	CertDetail *dlg;
+	dlg = new CertDetail(mainwin);
 	if (dlg) {
 		dlg->setReq(req);
-		connect( dlg->privKey, SIGNAL( doubleClicked(QString) ),
+		connect(dlg->privKey, SIGNAL( doubleClicked(QString) ),
 			mainwin->keys, SLOT( showItem(QString) ));
 		dlg->exec();
 		delete dlg;
@@ -165,7 +168,6 @@ void db_x509req::signReq()
 void db_x509req::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 {
 	QMenu *menu = new QMenu(mainwin);
-	QAction *expItem;
 	QMenu *subExport;
 	currentIdx = index;
 
@@ -174,18 +176,24 @@ void db_x509req::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
 	menu->addAction(tr("New Request"), this, SLOT(newItem()));
 	menu->addAction(tr("Import"), this, SLOT(load()));
 	if (index != QModelIndex()) {
+		if (!req->getRefKey())
+			menu->addAction(tr("Extract public Key"),
+				this, SLOT(extractPubkey()));
 		menu->addAction(tr("Rename"), this, SLOT(edit()));
 		menu->addAction(tr("Show Details"), this, SLOT(showItem()));
 		menu->addAction(tr("Sign"), this, SLOT(signReq()));
 		subExport = menu->addMenu(tr("Export"));
-		expItem = subExport->addAction(tr("File"), this, SLOT(store()));
+		subExport->addAction(tr("Clipboard"), this,
+					SLOT(pem2clipboard()));
+		subExport->addAction(tr("File"), this, SLOT(store()));
 		subExport->addAction(tr("Template"), this, SLOT(toTemplate()));
+		subExport->addAction(tr("OpenSSL config"), this,
+					SLOT(toOpenssl()));
 		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
 
-		expItem->setEnabled(!req->isSpki());
+		subExport->setEnabled(!req->isSpki());
 	}
-	menu->exec(e->globalPos());
-	delete menu;
+	contextMenu(e, menu);
 	currentIdx = QModelIndex();
 	return;
 }
