@@ -1,30 +1,36 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
-#ifndef PKI_EXCEPTION_H
-#define PKI_EXCEPTION_H
+#ifndef __PKI_EXCEPTION_H
+#define __PKI_EXCEPTION_H
 
-#include <qstring.h>
+#include <QtCore/QString>
+#include <QtCore/QObject>
 #include "base.h"
 
+#define E_PASSWD 1
+
 class errorEx
 {
 	private:
 		QString msg;
 	public:
-		errorEx(QString txt, QString className = "")
+		int info;
+		errorEx(QString txt = "", QString className = "", int inf = 0)
 		{
 			msg = txt;
 			if (!className.isEmpty())
 				msg += " (" + className + ")";
+			info = inf;
 		}
 		errorEx(const errorEx &e)
 		{
 			msg = e.msg;
+			info = e.info;
 		}
 		void appendString(QString s)
 		{
@@ -44,5 +50,10 @@ class errorEx
 		}
 };
 
-#define check_oom(ptr) if(!ptr){throw errorEx(QObject::tr("Out of Memory"));}
+#define check_oom(ptr) \
+	if(!ptr) { \
+		throw errorEx(QObject::tr("Out of Memory at %1:%2").\
+			arg(__FILE__).arg(__LINE__)); \
+	}
+
 #endif
