@@ -1,13 +1,17 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
+#ifndef __OID_H
+#define __OID_H
 
 class QString;
-#include <qlist.h>
+#include <QtCore/QList>
+
+extern int first_additional_oid;
 
 typedef QList<int> NIDlist;
 /* reads additional OIDs from a file: oid, sn, ln */
@@ -20,3 +24,4 @@ void initOIDs();
  */
 
 NIDlist readNIDlist(QString fname);
+#endif
