@@ -1,16 +1,18 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2007 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
 
+#include "func.h"
 #include "pki_base.h"
 #include "exception.h"
 #include <QtCore/QString>
 
 int pki_base::pki_counter = 0;
+int pki_base::suppress_messages = 0;
 
 pki_base::pki_base(const QString name, pki_base *p)
 {
@@ -21,7 +23,6 @@ pki_base::pki_base(const QString name, pki_base *p)
 	childItems.clear();
 	dataVersion=0;
 	pkiType=none;
-	cols=1;
 }
 
 int pki_base::getVersion()
@@ -37,7 +38,7 @@ enum pki_type pki_base::getType()
 pki_base::~pki_base(void)
 {
 	while((childCount()))
-			delete takeFirst();
+		delete takeFirst();
 	pki_counter--;
 }
 
@@ -78,58 +79,33 @@ void pki_base::setIntName(const QString &d)
 
 void pki_base::fopen_error(const QString fname)
 {
-	QString txt = tr("Error opening file:") + " '" + fname + "' " +
-			strerror(errno);
-	throw errorEx(txt, class_name);
+	my_error(tr("Error opening file: '%1': %2").
+			arg(fname).
+			arg(strerror(errno)));
+}
+
+void pki_base::fwrite_ba(FILE *fp, QByteArray ba, QString fname)
+{
+	if (fwrite(ba.constData(), 1, ba.size(), fp) != (size_t)ba.size()) {
+		my_error(tr("Error writing to file: '%1': %2").
+			arg(fname).
+			arg(strerror(errno)));
+        }
 }
 
 void pki_base::my_error(const QString error) const
 {
 	if (!error.isEmpty()) {
-		fprintf(stderr, "%s\n", CCHAR(tr("Error :") + error));
+		fprintf(stderr, "%s\n", CCHAR(tr("Error: ") + error));
 		throw errorEx(error, class_name);
 	}
 }
 
-void pki_base::openssl_error(const QString txt)
-{
-	QString errtxt;
-	QString error;
-
-	while (int i = ERR_get_error() ) {
-		errtxt = ERR_error_string(i ,NULL);
-		fprintf(stderr, "OpenSSL error: %s\n", ERR_error_string(i ,NULL) );
-		error += errtxt + "\n";
-	}
-	if (!error.isEmpty()) {
-		if (!txt.isEmpty())
-			error = txt + "\n" + error;
-		throw errorEx(error);
-	}
-}
-
-bool pki_base::ign_openssl_error()
-{
-	// ignore openssl errors
-	QString errtxt;
-	while (int i = ERR_get_error() ) {
-		errtxt = ERR_error_string(i ,NULL);
-		fprintf(stderr,"IGNORED: %s\n", CCHAR(errtxt));
-	}
-	return !errtxt.isEmpty();
-}
-
 QString pki_base::rmslashdot(const QString &s)
 {
 	QByteArray a = s.toAscii();
 	int r = a.lastIndexOf('.');
-#ifdef WIN32
-	int l = a.lastIndexOf('\\');
-#else
 	int l = a.lastIndexOf('/');
-#endif
-	//printf("r=%d, l=%d, s='%s', mid='%s'\n",r,l,(const char*)a,
-	//		CCHAR(s.mid(l+1,r-l-1)));
 	return s.mid(l+1,r-l-1);
 }
 
@@ -198,30 +174,37 @@ pki_base *pki_base::takeFirst()
 	return childItems.takeFirst();
 }
 
-int pki_base::columns(void)
+QVariant pki_base::column_data(dbheader *hd)
 {
-	return cols;
+	switch (hd->id) {
+	case HD_internal_name:
+		return QVariant(getIntName());
+	}
+	return QVariant();
 }
 
-QVariant pki_base::column_data(int col)
-{
-	return QVariant("invalid");
-}
-QVariant pki_base::getIcon(int column)
+QVariant pki_base::getIcon(dbheader *hd)
 {
 	return QVariant();
 }
-void pki_base::oldFromData(unsigned char *p, int size)
-{
-}
 
-uint32_t pki_base::intFromData(const unsigned char **p)
+uint32_t pki_base::intFromData(QByteArray &ba)
 {
 	/* For import "oldFromData" use the endian dependent version */
-	int s = sizeof(uint32_t);
 	uint32_t ret;
-	memcpy(&ret, *p, s);
-	*p += s;
+	if ((unsigned)(ba.count()) < sizeof(uint32_t)) {
+		ba.clear();
+		return 0;
+	}
+	memcpy(&ret, ba.constData(), sizeof(uint32_t));
+	ba = ba.mid(sizeof(uint32_t));
 	return ret;
 }
 
+bool pki_base::compare(pki_base *refcrl)
+{
+	bool ret;
+	ret = (i2d() == refcrl->i2d());
+	pki_openssl_error();
+	return ret;
+}
