@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -8,20 +8,20 @@
 
 #include "pki_crl.h"
 #include "func.h"
-#include <qdir.h>
+#include "exception.h"
+#include <QtCore/QDir>
 
 QPixmap *pki_crl::icon = NULL;
 
 pki_crl::pki_crl(const QString name )
-	:pki_base(name)
+	:pki_x509name(name)
 {
 	issuer = NULL;
 	crl = X509_CRL_new();
 	class_name="pki_crl";
-	openssl_error();
+	pki_openssl_error();
 	dataVersion=1;
 	pkiType=revokation;
-	cols=3;
 }
 
 void pki_crl::fromPEM_BIO(BIO *bio, QString name)
@@ -34,24 +34,46 @@ void pki_crl::fromPEM_BIO(BIO *bio, QString name)
 	setIntName(rmslashdot(name));
 }
 
-void pki_crl::fload(const QString fname )
+QString pki_crl::getMsg(msg_type msg)
+{
+	/*
+	 * We do not construct english sentences from fragments
+	 * to allow proper translations.
+	 *
+	 * %1 will be replaced by the internal name of the CRL
+	 */
+	switch (msg) {
+	case msg_import: return tr("Successfully imported the revokation list '%1'");
+	case msg_delete: return tr("Delete the revokation list '%1'?");
+	case msg_create: return tr("Successfully created the revokation list '%1'");
+	/* %1: Number of CRLs; %2: list of CRL names */
+	case msg_delete_multi: return tr("Delete the %1 revokation lists: %2?");
+	}
+	return pki_base::getMsg(msg);
+}
+
+void pki_crl::fload(const QString fname)
 {
 	FILE *fp = fopen(QString2filename(fname), "r");
 	X509_CRL *_crl;
 	if (fp != NULL) {
 		_crl = PEM_read_X509_CRL(fp, NULL, NULL, NULL);
 		if (!_crl) {
-			ign_openssl_error();
+			pki_ign_openssl_error();
 			rewind(fp);
 			_crl = d2i_X509_CRL_fp(fp, NULL);
 		}
 		fclose(fp);
-		openssl_error();
+		if (pki_ign_openssl_error()) {
+			if (_crl)
+				X509_CRL_free(_crl);
+			throw errorEx(tr("Unable to load the revokation list in file %1. Tried PEM and DER formatted CRL.").arg(fname));
+		}
 		if (crl)
 			X509_CRL_free(crl);
 		crl = _crl;
 		setIntName(rmslashdot(fname));
-		openssl_error(fname);
+		pki_openssl_error();
 	} else
 		fopen_error(fname);
 }
@@ -69,10 +91,9 @@ void pki_crl::createCrl(const QString d, pki_x509 *iss )
 	if (!iss)
 		my_error(tr("No issuer given"));
 	crl->crl->issuer = issuer->getSubject().get();
-	crl->crl->revoked = sk_X509_REVOKED_new_null();
 	a1int version = 1; /* version 2 CRL */
 	crl->crl->version = version.get();
-	openssl_error();
+	pki_openssl_error();
 }
 
 a1int pki_crl::getVersion()
@@ -83,18 +104,12 @@ a1int pki_crl::getVersion()
 
 void pki_crl::setLastUpdate(const a1time &t)
 {
-	if (crl->crl->lastUpdate != NULL)
-		ASN1_TIME_free(crl->crl->lastUpdate);
-
-	crl->crl->lastUpdate = t.get_utc();
+	t.set_date(&crl->crl->lastUpdate);
 }
 
 void pki_crl::setNextUpdate(const a1time &t)
 {
-	if (crl->crl->nextUpdate != NULL)
-		ASN1_TIME_free(crl->crl->nextUpdate);
-
-	crl->crl->nextUpdate = t.get_utc();
+	t.set_date(&crl->crl->nextUpdate);
 }
 
 pki_crl::~pki_crl()
@@ -102,49 +117,45 @@ pki_crl::~pki_crl()
 	X509_CRL_free(crl);
 }
 
+void pki_crl::d2i(QByteArray &ba)
+{
+	X509_CRL *c = (X509_CRL*)d2i_bytearray(D2I_VOID(d2i_X509_CRL), ba);
+	if (c) {
+		X509_CRL_free(crl);
+		crl = c;
+	}
+}
+
+QByteArray pki_crl::i2d()
+{
+	return i2d_bytearray(I2D_VOID(i2d_X509_CRL), crl);
+}
+
 void pki_crl::fromData(const unsigned char *p, db_header_t *head)
 {
-	X509_CRL *crl_sik = crl;
-	int version, size;
+	int size;
 
 	size = head->len - sizeof(db_header_t);
-	version = head->version;
 
-	crl = D2I_CLASH(d2i_X509_CRL, NULL, &p, size);
-	if (crl)
-		X509_CRL_free(crl_sik);
-	else
-		crl = crl_sik;
-	openssl_error();
+	QByteArray ba((const char*)p, size);
+	d2i(ba);
+
+	if (ba.count() > 0) {
+		my_error(tr("Wrong Size %1").arg(ba.count()));
+	}
 }
 
-unsigned char *pki_crl::toData(int *size)
+QByteArray pki_crl::toData()
 {
-	unsigned char *p, *p1;
-	*size = i2d_X509_CRL(crl, NULL);
-	openssl_error();
-	p = (unsigned char*)OPENSSL_malloc(*size);
-	p1 = p;
-	i2d_X509_CRL(crl, &p1);
-	openssl_error();
-	return p;
+	QByteArray ba = i2d();
+	pki_openssl_error();
+	return ba;
 }
 
-bool pki_crl::compare(pki_base *refcrl)
-{
-	bool ret;
-	ret = X509_CRL_cmp(crl, ((pki_crl *)refcrl)->crl) == 0 &&
-		getLastUpdate() == ((pki_crl *)refcrl)->getLastUpdate() &&
-		getNextUpdate() == ((pki_crl *)refcrl)->getNextUpdate() ;
-	openssl_error();
-	return ret;
-}
-
-
 void pki_crl::addRev(const x509rev &xrev)
 {
-	sk_X509_REVOKED_push(crl->crl->revoked, xrev.get());
-	openssl_error();
+	X509_CRL_add0_revoked(crl, xrev.get());
+	pki_openssl_error();
 }
 
 void pki_crl::addV3ext(const x509v3ext &e)
@@ -152,7 +163,7 @@ void pki_crl::addV3ext(const x509v3ext &e)
 	X509_EXTENSION *ext = e.get();
 	X509_CRL_add_ext(crl, ext, -1);
 	X509_EXTENSION_free(ext);
-	openssl_error();
+	pki_openssl_error();
 }
 
 
@@ -161,10 +172,11 @@ void pki_crl::sign(pki_key *key, const EVP_MD *md)
 	EVP_PKEY *pkey;
 	if (!key || key->isPubKey())
 		return;
+	X509_CRL_sort(crl);
 	pkey = key->decryptKey();
 	X509_CRL_sign(crl, pkey, md);
 	EVP_PKEY_free(pkey);
-	openssl_error();
+	pki_openssl_error();
 }
 
 void pki_crl::writeDefault(const QString fname)
@@ -183,14 +195,18 @@ void pki_crl::writeCrl(const QString fname, bool pem)
 				i2d_X509_CRL_fp(fp, crl);
 		}
 		fclose(fp);
-		openssl_error();
+		pki_openssl_error();
 	} else
 		fopen_error(fname);
 }
 
-pki_x509 *pki_crl::getIssuer() { return issuer; }
-void pki_crl::setIssuer(pki_x509 *iss) { issuer = iss; }
-
+BIO *pki_crl::pem(BIO *b)
+{
+	if (!b)
+		b = BIO_new(BIO_s_mem());
+	PEM_write_bio_X509_CRL(b, crl);
+	return b;
+}
 
 a1time pki_crl::getLastUpdate()
 {
@@ -221,12 +237,12 @@ x509rev pki_crl::getRev(int num)
 	x509rev ret;
 	if (crl && crl->crl && crl->crl->revoked) {
 		ret.set(sk_X509_REVOKED_value(crl->crl->revoked, num));
-		openssl_error();
+		pki_openssl_error();
 	}
 	return ret;
 }
 
-x509name pki_crl::getIssuerName()
+x509name pki_crl::getSubject() const
 {
 	x509name x;
 	if (crl && crl->crl && crl->crl->issuer) {
@@ -240,11 +256,33 @@ bool pki_crl::verify(pki_key *key)
 	bool ret=false;
 	if (crl && crl->crl && key) {
 		ret = (X509_CRL_verify(crl , key->getPubKey()) == 1);
-		ign_openssl_error();
+		pki_ign_openssl_error();
 	}
 	return ret ;
 }
 
+void pki_crl::setCrlNumber(a1int num)
+{
+	ASN1_INTEGER *tmpser = num.get();
+	pki_openssl_error();
+	X509_CRL_add1_ext_i2d(crl, NID_crl_number, tmpser, 0, 0);
+	ASN1_INTEGER_free(tmpser);
+	pki_openssl_error();
+}
+
+bool pki_crl::getCrlNumber(a1int *num)
+{
+	int j;
+	ASN1_INTEGER *i;
+	i = (ASN1_INTEGER *)X509_CRL_get_ext_d2i(crl, NID_crl_number, &j, NULL);
+	pki_openssl_error();
+	if (j == -1)
+		return false;
+	num->set(i);
+	ASN1_INTEGER_free(i);
+	return true;
+}
+
 x509v3ext pki_crl::getExtByNid(int nid)
 {
 	extList el;
@@ -262,43 +300,44 @@ QString pki_crl::printV3ext()
 	extList el;
 	el.setStack(crl->crl->extensions);
 	QString text = el.getHtml("<br>");
-	openssl_error();
+	pki_openssl_error();
 	return text;
 }
 
-QVariant pki_crl::column_data(int col)
+QVariant pki_crl::column_data(dbheader *hd)
 {
-	switch (col) {
-		case 0:
-			return QVariant(getIntName());
-		case 1:
+	switch (hd->id) {
+		case HD_crl_signer:
 			if (issuer)
 				return QVariant(getIssuer()->getIntName());
 			else
-				 return QVariant(tr("unknown"));
-		case 2:
-			return QVariant(getIssuerName().getEntryByNid(NID_commonName));
-		case 3:
+				return QVariant(tr("unknown"));
+		case HD_crl_revoked:
 			return QVariant(numRev());
-		case 4:
+		case HD_crl_lastUpdate:
+			return QVariant(getLastUpdate().toSortable());
+		case HD_crl_nextUpdate:
 			return QVariant(getNextUpdate().toSortable());
+		case HD_crl_crlnumber:
+			a1int a;
+			if (getCrlNumber(&a))
+				return QVariant(a.toDec());
+			return QVariant();
 	}
-	return QVariant();
+	return pki_x509name::column_data(hd);
 }
 
-QVariant pki_crl::getIcon(int column)
+QVariant pki_crl::getIcon(dbheader *hd)
 {
-	return column == 0 ? QVariant(*icon) : QVariant();
+	return hd->id == HD_internal_name ? QVariant(*icon) : QVariant();
 }
 
 void pki_crl::oldFromData(unsigned char *p, int size)
 {
-	X509_CRL *crl_sik = crl;
-	const unsigned char *p1 = p;
-	crl = D2I_CLASH(d2i_X509_CRL, NULL, &p1, size);
-	if (crl)
-		X509_CRL_free(crl_sik);
-	else
-		crl = crl_sik;
-	openssl_error();
+	QByteArray ba((const char *)p, size);
+	d2i(ba);
+
+	if (ba.count() > 0) {
+		my_error(tr("Wrong Size %1").arg(ba.count()));
+	}
 }
