@@ -1,13 +1,13 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
 
-#ifndef PKI_CRL_H
-#define PKI_CRL_H
+#ifndef __PKI_CRL_H
+#define __PKI_CRL_H
 
 #include <iostream>
 #include <openssl/pem.h>
@@ -16,8 +16,9 @@
 #include "asn1time.h"
 #include "asn1int.h"
 
-class pki_crl: public pki_base
+class pki_crl: public pki_x509name
 {
+		Q_OBJECT
 	friend class pki_x509;
 	protected:
 		pki_x509 *issuer;
@@ -38,25 +39,36 @@ class pki_crl: public pki_base
 		void addV3ext(const x509v3ext &e);
 		void sign(pki_key *key, const EVP_MD *md = EVP_md5());
 		void writeCrl(const QString fname, bool pem = true);
-		pki_x509 *getIssuer();
-		void setIssuer(pki_x509 *iss);
-		x509name getIssuerName();
+		pki_x509 *getIssuer()
+		{
+			return issuer;
+		}
+		void setIssuer(pki_x509 *iss)
+		{
+			 issuer = iss;
+		}
+		x509name getSubject() const;
 		void setLastUpdate(const a1time &t);
 		void setNextUpdate(const a1time &t);
 		a1time getNextUpdate();
 		a1time getLastUpdate();
 		void fromData(const unsigned char *p, db_header_t *head);
 		void oldFromData(unsigned char *p, int size);
-		unsigned char *toData(int *size);
-		bool compare(pki_base *refcrl);
+		QByteArray toData();
 		int numRev();
 		bool verify(pki_key *pkey);
 		x509rev getRev(int num);
 		QString printV3ext();
 		x509v3ext getExtByNid(int nid);
 		a1int getVersion();
-		QVariant column_data(int col);
-		QVariant getIcon(int column);
+		QVariant column_data(dbheader *hd);
+		QVariant getIcon(dbheader *hd);
+		virtual QString getMsg(msg_type msg);
+		void d2i(QByteArray &ba);
+		QByteArray i2d();
+		void setCrlNumber(a1int num);
+		bool getCrlNumber(a1int *num);
+		BIO *pem(BIO *);
 };
 
 #endif
