@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -8,70 +8,45 @@
 
 #include "pki_evp.h"
 #include "pass_info.h"
+#include "Passwd.h"
 #include "func.h"
 #include "db.h"
+#include "widgets/PwDialog.h"
+
 #include <openssl/rand.h>
 #include <openssl/evp.h>
 #include <openssl/pem.h>
-#include <qprogressdialog.h>
-#include <qapplication.h>
-#include <qdir.h>
-#include <widgets/MainWindow.h>
 
-char pki_evp::passwd[MAX_PASS_LENGTH]={0,};
-char pki_evp::oldpasswd[MAX_PASS_LENGTH]={0,};
+#include <QtGui/QProgressDialog>
+#include <QtGui/QApplication>
+#include <QtCore/QDir>
+
+Passwd pki_evp::passwd;
+Passwd pki_evp::oldpasswd;
 
 QString pki_evp::passHash = QString();
 
 QPixmap *pki_evp::icon[2]= { NULL, NULL };
 
+#ifndef OPENSSL_NO_EC
 EC_builtin_curve *pki_evp::curves = NULL;
 size_t pki_evp::num_curves = 0;
 unsigned char *pki_evp::curve_flags = NULL;
-
-void pki_evp::erasePasswd()
-{
-	memset(passwd, 0, MAX_PASS_LENGTH);
-}
-
-void pki_evp::eraseOldPasswd()
-{
-	memset(oldpasswd, 0, MAX_PASS_LENGTH);
-}
-
-void pki_evp::setPasswd(const char *pass)
-{
-	strncpy(passwd, pass, MAX_PASS_LENGTH);
-	passwd[MAX_PASS_LENGTH-1] = '\0';
-}
-
-void pki_evp::setOldPasswd(const char *pass)
-{
-	strncpy(oldpasswd, pass, MAX_PASS_LENGTH);
-	oldpasswd[MAX_PASS_LENGTH-1] = '\0';
-}
+#endif
 
 void pki_evp::init(int type)
 {
 	key->type = type;
 	class_name = "pki_evp";
-	encKey = NULL;
-	encKey_len = 0;
 	ownPass = ptCommon;
 	dataVersion=2;
 	pkiType=asym_key;
-	cols=5;
-}
-
-static void incProgress(int, int, void *progress)
-{
-	int i = ((QProgressBar *)progress)->value();
-			((QProgressBar *)progress)->setValue(++i);
 }
 
 QString pki_evp::removeTypeFromIntName(QString n)
 {
-	if (n.right(1) != ")" ) return n;
+	if (n.right(1) != ")" )
+		return n;
 	n.truncate(n.length() - 6);
 	return n;
 }
@@ -105,29 +80,30 @@ void pki_evp::setOwnPass(enum passType x)
 
 void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid)
 {
-	RSA *rsakey;
-	DSA *dsakey;
-	EC_KEY *eckey;
-
 	progress->setMinimum(0);
 	progress->setMaximum(100);
 	progress->setValue(50);
 
 	switch (type) {
 	case EVP_PKEY_RSA:
-		rsakey = RSA_generate_key(bits, 0x10001, &incProgress,progress);
+		RSA *rsakey;
+		rsakey = RSA_generate_key(bits, 0x10001, inc_progress_bar,
+			progress);
 		if (rsakey)
-			EVP_PKEY_set1_RSA(key, rsakey);
+			EVP_PKEY_assign_RSA(key, rsakey);
 		break;
 	case EVP_PKEY_DSA:
+		DSA *dsakey;
 		progress->setMaximum(500);
 		dsakey = DSA_generate_parameters(bits, NULL, 0, NULL, NULL,
-				&incProgress, progress);
+				inc_progress_bar, progress);
 		DSA_generate_key(dsakey);
 		if (dsakey)
-			EVP_PKEY_set1_DSA(key, dsakey);
+			EVP_PKEY_assign_DSA(key, dsakey);
 		break;
+#ifndef OPENSSL_NO_EC
 	case EVP_PKEY_EC:
+		EC_KEY *eckey;
 		EC_GROUP *group = EC_GROUP_new_by_curve_name(curve_nid);
 		if (!group)
 			break;
@@ -139,7 +115,7 @@ void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid
 		EC_GROUP_set_asn1_flag(group, 1);
 		if (EC_KEY_set_group(eckey, group)) {
 			if (EC_KEY_generate_key(eckey)) {
-				EVP_PKEY_set1_EC_KEY(key, eckey);
+				EVP_PKEY_assign_EC_KEY(key, eckey);
 				EC_GROUP_free(group);
 				break;
 			}
@@ -147,8 +123,9 @@ void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid
 		EC_KEY_free(eckey);
 		EC_GROUP_free(group);
 		break;
+#endif
 	}
-	openssl_error();
+	pki_openssl_error();
 	encryptKey();
 }
 
@@ -156,21 +133,16 @@ pki_evp::pki_evp(const pki_evp *pk)
 	:pki_key(pk)
 {
 	init(pk->key->type);
-	openssl_error();
+	pki_openssl_error();
 	ownPass = pk->ownPass;
-	encKey_len = pk->encKey_len;
-	if (encKey_len) {
-		encKey = (unsigned char *)OPENSSL_malloc(encKey_len);
-		check_oom(encKey);
-		memcpy(encKey, pk->encKey, encKey_len);
-	}
+	encKey = pk->encKey;
 }
 
 pki_evp::pki_evp(const QString name, int type )
 	:pki_key(name)
 {
 	init(type);
-	openssl_error();
+	pki_openssl_error();
 }
 
 pki_evp::pki_evp(EVP_PKEY *pkey)
@@ -190,8 +162,10 @@ static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
 			return key->pkey.rsa->d ? true: false;
 		case EVP_PKEY_DSA:
 			return key->pkey.dsa->priv_key ? true: false;
+#ifndef OPENSSL_NO_EC
 		case EVP_PKEY_EC:
 			return EC_KEY_get0_private_key(key->pkey.ec) ? true: false;
+#endif
 	}
 	return false;
 }
@@ -212,18 +186,32 @@ QList<int> pki_evp::possibleHashNids()
 	return nids;
 };
 
+void pki_evp::openssl_pw_error(QString fname)
+{
+	switch (ERR_peek_error() & 0xff000fff) {
+	case ERR_PACK(ERR_LIB_PEM, 0, PEM_R_BAD_DECRYPT):
+	case ERR_PACK(ERR_LIB_PEM, 0, PEM_R_BAD_PASSWORD_READ):
+	case ERR_PACK(ERR_LIB_EVP, 0, EVP_R_BAD_DECRYPT):
+		pki_ign_openssl_error();
+		throw errorEx(tr("Failed to decrypt the key (bad password) ")+
+				fname, class_name, E_PASSWD);
+	}
+}
+
 void pki_evp::fromPEM_BIO(BIO *bio, QString name)
 {
 	EVP_PKEY *pkey;
 	int pos;
-	pass_info p(XCA_TITLE, qApp->translate("MainWindow",
-			"Please enter the password to decrypt the private key."));
+	pass_info p(XCA_TITLE,
+		tr("Please enter the password to decrypt the private key.") +
+		" " + name);
 	pos = BIO_tell(bio);
-	pkey = PEM_read_bio_PrivateKey(bio, NULL, MainWindow::passRead, &p);
+	pkey = PEM_read_bio_PrivateKey(bio, NULL, PwDialog::pwCallback, &p);
+	openssl_pw_error(name);
 	if (!pkey){
-		ign_openssl_error();
+		pki_ign_openssl_error();
 		pos = BIO_seek(bio, pos);
-		pkey = PEM_read_bio_PUBKEY(bio, NULL, MainWindow::passRead, &p);
+		pkey = PEM_read_bio_PUBKEY(bio, NULL, PwDialog::pwCallback, &p);
 	}
 	if (pkey){
 		if (key)
@@ -236,8 +224,14 @@ void pki_evp::fromPEM_BIO(BIO *bio, QString name)
 	openssl_error(name);
 }
 
-static void search_ec_oid(EC_KEY *ec)
+static void search_ec_oid(EVP_PKEY *pkey)
 {
+#ifndef OPENSSL_NO_EC
+	if (pkey->type != EVP_PKEY_EC)
+		return;
+
+	EC_KEY *ec = pkey->pkey.ec;
+
 	const EC_GROUP *ec_group = EC_KEY_get0_group(ec);
 	EC_GROUP *builtin;
 
@@ -259,50 +253,67 @@ static void search_ec_oid(EC_KEY *ec)
 			EC_GROUP_free(builtin);
 		}
 	}
+#endif
 }
 
 void pki_evp::fload(const QString fname)
 {
-	pass_info p(XCA_TITLE, qApp->translate("MainWindow",
-		"Please enter the password to decrypt the private key.") +
-		"\n'" + fname + "'");
-	pem_password_cb *cb = MainWindow::passRead;
+	pass_info p(XCA_TITLE, tr("Please enter the password to decrypt the private key from file:\n%1").
+		arg(compressFilename(fname)));
+	pem_password_cb *cb = PwDialog::pwCallback;
 	FILE *fp = fopen(QString2filename(fname), "r");
 	EVP_PKEY *pkey;
 
-	ign_openssl_error();
+	pki_ign_openssl_error();
 	if (!fp) {
 		fopen_error(fname);
 		return;
 	}
 	pkey = PEM_read_PrivateKey(fp, NULL, cb, &p);
-	if (!pkey) {
-		if (ERR_get_error() == 0x06065064) {
-			fclose(fp);
-			ign_openssl_error();
-			throw errorEx(tr("Failed to decrypt the key (bad password) ") +
-					fname, class_name);
-		}
+	try {
+		openssl_pw_error(fname);
+	} catch (errorEx &err) {
+		fclose(fp);
+		throw err;
 	}
 	if (!pkey) {
-		ign_openssl_error();
+		pki_ign_openssl_error();
 		rewind(fp);
 		pkey = d2i_PrivateKey_fp(fp, NULL);
 	}
 	if (!pkey) {
-		ign_openssl_error();
+		pki_ign_openssl_error();
+		rewind(fp);
+		pkey = d2i_PKCS8PrivateKey_fp(fp, NULL, cb, &p);
+	}
+	if (!pkey) {
+		PKCS8_PRIV_KEY_INFO *p8inf;
+		pki_ign_openssl_error();
+		rewind(fp);
+		p8inf = d2i_PKCS8_PRIV_KEY_INFO_fp(fp, NULL);
+		if (p8inf) {
+			pkey = EVP_PKCS82PKEY(p8inf);
+			PKCS8_PRIV_KEY_INFO_free(p8inf);
+		}
+	}
+	if (!pkey) {
+		pki_ign_openssl_error();
 		rewind(fp);
 		pkey = PEM_read_PUBKEY(fp, NULL, cb, &p);
 	}
 	if (!pkey) {
-		ign_openssl_error();
+		pki_ign_openssl_error();
 		rewind(fp);
 		pkey = d2i_PUBKEY_fp(fp, NULL);
 	}
 	fclose(fp);
+	if (pki_ign_openssl_error()) {
+		if (pkey)
+			EVP_PKEY_free(pkey);
+		throw errorEx(tr("Unable to load the private key in file %1. Tried PEM and DER private, public and PKCS#8 key types.").arg(fname));
+	}
 	if (pkey){
-		if (pkey->type == EVP_PKEY_EC)
-			search_ec_oid(pkey->pkey.ec);
+		search_ec_oid(pkey);
 		if (key)
 			EVP_PKEY_free(key);
 		key = pkey;
@@ -310,37 +321,31 @@ void pki_evp::fload(const QString fname)
 			bogusEncryptKey();
 		setIntName(rmslashdot(fname));
 	}
-	openssl_error(fname);
 }
 
 void pki_evp::fromData(const unsigned char *p, db_header_t *head )
 {
-	const unsigned char *p1;
 	int version, type, size;
 
-	p1 = p;
 	size = head->len - sizeof(db_header_t);
 	version = head->version;
 
+	QByteArray ba((const char*)p, size);
+
 	if (key)
 		EVP_PKEY_free(key);
 
 	key = NULL;
-	type = db::intFromData(&p1);
-	ownPass = db::intFromData(&p1);
+	type = db::intFromData(ba);
+	ownPass = db::intFromData(ba);
 	if (version < 2) {
-		D2I_CLASHT(d2i_PublicKey, type, &key, &p1, size - (2*sizeof(int)));
+		d2i_old(ba, type);
 	} else {
-		d2i_PUBKEY(&key, &p1, size - (2*sizeof(int)));
+		d2i(ba);
 	}
-	openssl_error();
+	pki_openssl_error();
 
-	encKey_len = size - (p1-p);
-	if (encKey_len) {
-		encKey = (unsigned char *)OPENSSL_malloc(encKey_len);
-		check_oom(encKey);
-		memcpy(encKey, p1 ,encKey_len);
-	}
+	encKey = ba;
 }
 
 EVP_PKEY *pki_evp::decryptKey() const
@@ -354,7 +359,8 @@ EVP_PKEY *pki_evp::decryptKey() const
 	EVP_PKEY *tmpkey;
 	EVP_CIPHER_CTX ctx;
 	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
-	char ownPassBuf[MAX_PASS_LENGTH] = "";
+	Passwd ownPassBuf;
+	int ret;
 
 	if (isPubKey()) {
 		unsigned char *q;
@@ -369,82 +375,67 @@ EVP_PKEY *pki_evp::decryptKey() const
 	}
 	/* This key has its own password */
 	if (ownPass == ptPrivate) {
-		int ret;
-		pass_info pi(XCA_TITLE, qApp->translate("MainWindow",
-			"Please enter the password to decrypt the private key: '") +
-			getIntName() + "'");
-		ret = MainWindow::passRead(ownPassBuf, MAX_PASS_LENGTH, 0, &pi);
-		if (ret < 0)
+		pass_info pi(XCA_TITLE, tr("Please enter the password to decrypt the private key: '%1'").arg(getIntName()));
+		ret = PwDialog::execute(&pi, &ownPassBuf, false);
+		if (ret != 1)
 			throw errorEx(tr("Password input aborted"), class_name);
 	} else if (ownPass == ptBogus) { // BOGUS pass
-		ownPassBuf[0] = '\0';
+		ownPassBuf = "Bogus";
 	} else {
-		memcpy(ownPassBuf, passwd, MAX_PASS_LENGTH);
-		//printf("Orig password: '%s' len:%d\n", passwd, strlen(passwd));
+		ownPassBuf = passwd;
 		while (md5passwd(ownPassBuf) != passHash &&
 			sha512passwd(ownPassBuf, passHash) != passHash)
 		{
-			int ret;
-			//printf("Passhash= '%s', new hash= '%s', passwd= '%s'\n",
-				//CCHAR(passHash), CCHAR(md5passwd(ownPassBuf)), ownPassBuf);
-			pass_info p(XCA_TITLE, qApp->translate("MainWindow",
-					"Please enter the database password for decrypting the key"));
-			ret = MainWindow::passRead(ownPassBuf, MAX_PASS_LENGTH, 0, &p);
-			if (ret < 0)
+			pass_info p(XCA_TITLE, tr("Please enter the database password for decrypting the key '%1'").arg(getIntName()));
+			ret = PwDialog::execute(&p, &ownPassBuf, false);
+			if (ret != 1)
 				throw errorEx(tr("Password input aborted"), class_name);
 		}
 	}
-	//printf("Using decrypt Pass: %s\n", ownPassBuf);
-	p = (unsigned char *)OPENSSL_malloc(encKey_len);
+	p = (unsigned char *)OPENSSL_malloc(encKey.count());
 	check_oom(p);
-	openssl_error();
+	pki_openssl_error();
 	p1 = p;
 	memset(iv, 0, EVP_MAX_IV_LENGTH);
 
-	memcpy(iv, encKey, 8); /* recover the iv */
+	memcpy(iv, encKey.constData(), 8); /* recover the iv */
 	/* generate the key */
-	EVP_BytesToKey(cipher, EVP_sha1(), iv, (unsigned char *)ownPassBuf,
-		strlen(ownPassBuf), 1, ckey,NULL);
+	EVP_BytesToKey(cipher, EVP_sha1(), iv,
+		ownPassBuf.constUchar(),
+		ownPassBuf.size(), 1, ckey, NULL);
 	/* we use sha1 as message digest,
 	 * because an md5 version of the password is
 	 * stored in the database...
 	 */
 	EVP_CIPHER_CTX_init(&ctx);
 	EVP_DecryptInit(&ctx, cipher, ckey, iv);
-	EVP_DecryptUpdate(&ctx, p , &outl, encKey +8, encKey_len -8);
+	EVP_DecryptUpdate(&ctx, p , &outl,
+		(const unsigned char*)encKey.constData() +8, encKey.count() -8);
+
 	decsize = outl;
 	EVP_DecryptFinal(&ctx, p + decsize , &outl);
 	decsize += outl;
-	//printf("Decrypt decsize=%d, encKey_len=%d\n", decsize, encKey_len);
-	openssl_error();
-	tmpkey = D2I_CLASHT(d2i_PrivateKey, key->type, NULL, &p1, decsize);
-	openssl_error();
+	//printf("Decrypt decsize=%d, encKey_len=%d\n", decsize, encKey.count() -8);
+	pki_openssl_error();
+	tmpkey = d2i_PrivateKey(key->type, NULL, &p1, decsize);
+	pki_openssl_error();
 	OPENSSL_free(p);
 	EVP_CIPHER_CTX_cleanup(&ctx);
-	openssl_error();
+	pki_openssl_error();
+	if (EVP_PKEY_type(tmpkey->type) == EVP_PKEY_RSA)
+		RSA_blinding_on(tmpkey->pkey.rsa, NULL);
 	return tmpkey;
 }
 
-unsigned char *pki_evp::toData(int *size)
+QByteArray pki_evp::toData()
 {
-	unsigned char *p, *p1;
-	int pubsize;
+	QByteArray ba;
 
-	pubsize = i2d_PUBKEY(key, NULL);
-	*size = pubsize + encKey_len + (2*sizeof(int));
-	p1 = p = (unsigned char *)OPENSSL_malloc(*size);
-	check_oom(p);
-	openssl_error();
-	db::intToData(&p1, key->type);
-	db::intToData(&p1, ownPass);
-	i2d_PUBKEY(key, &p1);
-	openssl_error();
-	if (encKey_len) {
-		memcpy(p1, encKey, encKey_len);
-	}
-	// printf("To data: pubsize=%d, encKey_len: %d, *size=%d\n",
-			//pubsize, encKey_len, *size);
-	return p;
+	ba += db::intToData(key->type);
+	ba += db::intToData(ownPass);
+	ba += i2d();
+	ba += encKey;
+	return ba;
 }
 
 EVP_PKEY *pki_evp::priv2pub(EVP_PKEY* key)
@@ -459,11 +450,11 @@ EVP_PKEY *pki_evp::priv2pub(EVP_PKEY* key)
 
 	/* convert rsa/dsa/ec to Pubkey */
 	keylen = i2d_PUBKEY(key, &p);
-	openssl_error();
+	pki_openssl_error();
 	p = p1;
 	pubkey = d2i_PUBKEY(NULL, (const unsigned char**)&p, keylen);
 	OPENSSL_free(p1);
-	openssl_error();
+	pki_openssl_error();
 	return pubkey;
 }
 
@@ -475,33 +466,32 @@ void pki_evp::encryptKey(const char *password)
 	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
 	unsigned char iv[EVP_MAX_IV_LENGTH], *punenc, *punenc1;
 	unsigned char ckey[EVP_MAX_KEY_LENGTH];
-	char ownPassBuf[MAX_PASS_LENGTH];
+	Passwd ownPassBuf;
 
 	/* This key has its own, private password */
 	if (ownPass == ptPrivate) {
 		int ret;
-		pass_info p(XCA_TITLE, qApp->translate("MainWindow",
-			"Please enter the password to protect the private key: '") +
-			getIntName() + "'");
-		ret = MainWindow::passWrite(ownPassBuf, MAX_PASS_LENGTH, 0, &p);
-		if (ret < 0)
+		pass_info p(XCA_TITLE, tr("Please enter the password to protect the private key: '%1'").
+			arg(getIntName()));
+		ret = PwDialog::execute(&p, &ownPassBuf, true);
+		if (ret != 1)
 			throw errorEx("Password input aborted", class_name);
 	} else if (ownPass == ptBogus) { // BOGUS password
-		ownPassBuf[0] = '\0';
+		ownPassBuf = "Bogus";
 	} else {
 		if (password) {
-			/* use the password parameter if this is a common password */
-			strncpy(ownPassBuf, password, MAX_PASS_LENGTH);
+			/* use the password parameter
+			 * if this is a common password */
+			ownPassBuf = password;
 		} else {
 			int ret = 0;
-			memcpy(ownPassBuf, passwd, MAX_PASS_LENGTH);
-			pass_info p(XCA_TITLE, qApp->translate("MainWindow",
-				"Please enter the database password for encrypting the key"));
+			ownPassBuf = passwd;
+			pass_info p(XCA_TITLE, tr("Please enter the database password for encrypting the key"));
 			while (md5passwd(ownPassBuf) != passHash &&
 				sha512passwd(ownPassBuf, passHash) != passHash )
 			{
-				ret = MainWindow::passRead(ownPassBuf, MAX_PASS_LENGTH, 0,&p);
-				if (ret < 0)
+				ret = PwDialog::execute(&p, &ownPassBuf, false);
+				if (ret != 1)
 					throw errorEx("Password input aborted", class_name);
 			}
 		}
@@ -510,24 +500,21 @@ void pki_evp::encryptKey(const char *password)
 	/* Prepare Encryption */
 	memset(iv, 0, EVP_MAX_IV_LENGTH);
 	RAND_pseudo_bytes(iv,8);      /* Generate a salt */
-	EVP_BytesToKey(cipher, EVP_sha1(), iv, (unsigned char *)ownPassBuf,
-			strlen(ownPassBuf), 1, ckey, NULL);
+	EVP_BytesToKey(cipher, EVP_sha1(), iv,
+			ownPassBuf.constUchar(),
+			ownPassBuf.size(), 1, ckey, NULL);
 	EVP_CIPHER_CTX_init (&ctx);
-	openssl_error();
-	if (encKey)
-		OPENSSL_free(encKey);
-	encKey_len = 0;
+	pki_openssl_error();
 
 	/* reserve space for unencrypted and encrypted key */
 	keylen = i2d_PrivateKey(key, NULL);
-	encKey = (unsigned char *)OPENSSL_malloc(keylen + EVP_MAX_KEY_LENGTH + 8);
-	check_oom(encKey);
+	encKey.resize(keylen + EVP_MAX_KEY_LENGTH + 8);
 	punenc1 = punenc = (unsigned char *)OPENSSL_malloc(keylen);
 	check_oom(punenc);
 	keylen = i2d_PrivateKey(key, &punenc1);
-	openssl_error();
+	pki_openssl_error();
 
-	memcpy(encKey, iv, 8); /* store the iv */
+	memcpy(encKey.data(), iv, 8); /* store the iv */
 	/*
 	 * Now DER version of privkey is in punenc
 	 * and privkey is still in key
@@ -535,24 +522,24 @@ void pki_evp::encryptKey(const char *password)
 
 	/* do the encryption */
 	/* store key right after the iv */
-	EVP_EncryptInit( &ctx, cipher, ckey, iv);
-	EVP_EncryptUpdate(&ctx, encKey + 8, &outl, punenc, keylen);
-	encKey_len = outl;
-	EVP_EncryptFinal(&ctx, encKey + encKey_len + 8, &outl);
-	encKey_len += outl + 8;
-
+	EVP_EncryptInit(&ctx, cipher, ckey, iv);
+	unsigned char *penc = (unsigned char *)encKey.data() +8;
+	EVP_EncryptUpdate(&ctx, penc, &outl, punenc, keylen);
+	int encKey_len = outl;
+	EVP_EncryptFinal(&ctx, penc + encKey_len, &outl);
+	encKey.resize(encKey_len + outl +8);
 	/* Cleanup */
 	EVP_CIPHER_CTX_cleanup(&ctx);
 	/* wipe out the memory */
 	memset(punenc, 0, keylen);
 	OPENSSL_free(punenc);
-	openssl_error();
+	pki_openssl_error();
 
 	pkey1 = priv2pub(key);
 	check_oom(pkey1);
 	EVP_PKEY_free(key);
 	key = pkey1;
-	openssl_error();
+	pki_openssl_error();
 
 	//CRYPTO_mem_ctrl(CRYPTO_MEM_CHECK_OFF);
 
@@ -575,8 +562,6 @@ void pki_evp::bogusEncryptKey()
 
 pki_evp::~pki_evp()
 {
-	if (encKey)
-		OPENSSL_free(encKey);
 }
 
 
@@ -584,7 +569,7 @@ void pki_evp::writePKCS8(const QString fname, const EVP_CIPHER *enc,
 		pem_password_cb *cb, bool pem)
 {
 	EVP_PKEY *pkey;
-	pass_info p(XCA_TITLE, tr("Please enter the password protecting the PKCS#8 key") + " '" + getIntName() + "'");
+	pass_info p(XCA_TITLE, tr("Please enter the password protecting the PKCS#8 key '%1'").arg(getIntName()));
 	FILE *fp = fopen(QString2filename(fname), "w");
 	if (fp != NULL) {
 		if (key) {
@@ -598,7 +583,7 @@ void pki_evp::writePKCS8(const QString fname, const EVP_CIPHER *enc,
 			}
 		}
 		fclose(fp);
-		openssl_error();
+		pki_openssl_error();
 	} else
 		fopen_error(fname);
 }
@@ -619,7 +604,7 @@ void pki_evp::writeKey(const QString fname, const EVP_CIPHER *enc,
 			pem_password_cb *cb, bool pem)
 {
 	EVP_PKEY *pkey;
-	pass_info p(XCA_TITLE, tr("Please enter the export password for the private key") + " '" + getIntName() + "'");
+	pass_info p(XCA_TITLE, tr("Please enter the export password for the private key '%1'").arg(getIntName()));
 	if (isPubKey()) {
 		writePublic(fname, pem);
 		return;
@@ -633,28 +618,38 @@ void pki_evp::writeKey(const QString fname, const EVP_CIPHER *enc,
 		pkey = decryptKey();
 		if (pkey) {
 			if (pem) {
-				PEM_write_PrivateKey(fp, pkey, enc, NULL, 0, cb, &p);
+				switch (pkey->type) {
+				case EVP_PKEY_RSA:
+					PEM_write_RSAPrivateKey(fp,
+					  pkey->pkey.rsa, enc, NULL, 0, cb, &p);
+					break;
+				case EVP_PKEY_DSA:
+					PEM_write_DSAPrivateKey(fp,
+					  pkey->pkey.dsa, enc, NULL, 0, cb, &p);
+					break;
+#ifndef OPENSSL_NO_EC
+				case EVP_PKEY_EC:
+					PEM_write_ECPrivateKey(fp,
+					  pkey->pkey.ec, enc, NULL, 0, cb, &p);
+					break;
+#endif
+				default:
+					PEM_write_PrivateKey(fp, pkey,
+							enc, NULL, 0, cb, &p);
+				}
 			} else {
 				i2d_PrivateKey_fp(fp, pkey);
 			}
 			EVP_PKEY_free(pkey);
 		}
-		openssl_error();
+		pki_openssl_error();
 	}
 	fclose(fp);
 }
 
-QString pki_evp::length()
-{
-	if (key->type == EVP_PKEY_DSA && key->pkey.dsa->p == NULL) {
-		return QString("???");
-	}
-	return QString("%1 bit").arg(EVP_PKEY_bits(key));
-}
-
 bool pki_evp::isPubKey() const
 {
-	if (encKey_len == 0 || encKey == NULL) {
+	if (encKey.count() == 0) {
 		return true;
 	}
 	return false;
@@ -670,7 +665,7 @@ int pki_evp::verify()
 	}
 	if (isPrivKey())
 		veri = true;
-	openssl_error();
+	pki_openssl_error();
 	return veri;
 }
 
@@ -680,44 +675,23 @@ const EVP_MD *pki_evp::getDefaultMD()
 	switch (key->type) {
 		case EVP_PKEY_RSA: md = EVP_sha1(); break;
 		case EVP_PKEY_DSA: md = EVP_dss1(); break;
+#ifndef OPENSSL_NO_EC
 		case EVP_PKEY_EC:  md = EVP_ecdsa(); break;
+#endif
 		default: md = NULL; break;
 	}
 	return md;
 }
 
-QVariant pki_evp::column_data(int col)
+QVariant pki_evp::getIcon(dbheader *hd)
 {
-	QStringList sl;
-	sl << tr("Common") << tr("Private") << tr("Bogus");
-	switch (col) {
-		case 0:
-			return QVariant(getIntName());
-		case 1:
-			return QVariant(getTypeString());
-		case 2:
-			return QVariant(length());
-		case 3:
-			return QVariant(getUcount());
-		case 4:
-			if (isPubKey())
-				return QVariant(tr("No password"));
-			if (ownPass<0 || ownPass>2)
-				return QVariant("Holla die Waldfee");
-			return QVariant(sl[ownPass]);
-	}
-	return QVariant();
-}
-
-QVariant pki_evp::getIcon(int column)
-{
-	if (column != 0)
+	if (hd->id != HD_internal_name)
 		return QVariant();
 	int pixnum= isPubKey() ? 1 : 0;
 	return QVariant(*icon[pixnum]);
 }
 
-QString pki_evp::md5passwd(const char *pass)
+QString pki_evp::md5passwd(QByteArray pass)
 {
 
 	EVP_MD_CTX mdctx;
@@ -726,7 +700,7 @@ QString pki_evp::md5passwd(const char *pass)
 	int j;
 	unsigned char m[EVP_MAX_MD_SIZE];
 	EVP_DigestInit(&mdctx, EVP_md5());
-	EVP_DigestUpdate(&mdctx, pass, strlen(pass));
+	EVP_DigestUpdate(&mdctx, pass.constData(), pass.size());
 	EVP_DigestFinal(&mdctx, m, (unsigned*)&n);
 	for (j=0; j<n; j++) {
 		char zs[4];
@@ -736,7 +710,7 @@ QString pki_evp::md5passwd(const char *pass)
 	return str;
 }
 
-QString pki_evp::sha512passwd(QString pass, QString salt)
+QString pki_evp::sha512passwd(QByteArray pass, QString salt)
 {
 
 	EVP_MD_CTX mdctx;
@@ -749,10 +723,10 @@ QString pki_evp::sha512passwd(QString pass, QString salt)
 		abort();
 
 	str = salt.left(5);
-	pass = str + pass;
+	pass = str.toAscii() + pass;
 
 	EVP_DigestInit(&mdctx, EVP_sha512());
-	EVP_DigestUpdate(&mdctx, CCHAR(pass), pass.size());
+	EVP_DigestUpdate(&mdctx, pass.constData(), pass.size());
 	EVP_DigestFinal(&mdctx, m, (unsigned*)&n);
 
 	for (j=0; j<n; j++) {
@@ -775,7 +749,7 @@ void pki_evp::veryOldFromData(unsigned char *p, int size )
 	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
 	sik = (unsigned char *)OPENSSL_malloc(size);
 	check_oom(sik);
-	openssl_error();
+	pki_openssl_error();
 	pdec = (unsigned char *)OPENSSL_malloc(size);
 	if (pdec == NULL ) {
 		OPENSSL_free(sik);
@@ -785,8 +759,8 @@ void pki_evp::veryOldFromData(unsigned char *p, int size )
 	sik1=sik;
 	memcpy(iv, p, 8); /* recover the iv */
 	/* generate the key */
-	EVP_BytesToKey(cipher, EVP_sha1(), iv, (unsigned char *)oldpasswd,
-		strlen(oldpasswd), 1, ckey,NULL);
+	EVP_BytesToKey(cipher, EVP_sha1(), iv, oldpasswd.constUchar(),
+		oldpasswd.size(), 1, ckey, NULL);
 	/* we use sha1 as message digest,
 	 * because an md5 version of the password is
 	 * stored in the database...
@@ -797,30 +771,30 @@ void pki_evp::veryOldFromData(unsigned char *p, int size )
 	decsize = outl;
 	EVP_DecryptFinal( &ctx, pdec + decsize , &outl );
 	decsize += outl;
-	openssl_error();
+	pki_openssl_error();
 	memcpy(sik, pdec, decsize);
 	if (key->type == EVP_PKEY_RSA) {
 		rsakey=d2i_RSAPrivateKey(NULL,(const unsigned char **)&pdec, decsize);
-		if (ign_openssl_error()) {
-			rsakey=D2I_CLASH(d2i_RSA_PUBKEY, NULL, &sik, decsize);
+		if (pki_ign_openssl_error()) {
+			rsakey = d2i_RSA_PUBKEY(NULL, (const unsigned char **)&sik, decsize);
 		}
-		openssl_error();
+		pki_openssl_error();
 		if (rsakey) EVP_PKEY_assign_RSA(key, rsakey);
 	}
 	OPENSSL_free(sik1);
 	OPENSSL_free(pdec1);
 	EVP_CIPHER_CTX_cleanup(&ctx);
-	openssl_error();
+	pki_openssl_error();
 	encryptKey();
 }
 
 void pki_evp::oldFromData(unsigned char *p, int size )
 {
-	const unsigned char *p1;
 	int version, type;
 
-	p1 = (const unsigned char*)p;
-	version = intFromData(&p1);
+	QByteArray ba;
+
+	version = intFromData(ba);
 	if (version != 1) { // backward compatibility
 		veryOldFromData(p, size);
 		return;
@@ -829,18 +803,12 @@ void pki_evp::oldFromData(unsigned char *p, int size )
 		EVP_PKEY_free(key);
 
 	key = NULL;
-	type = intFromData(&p1);
-	ownPass = intFromData(&p1);
+	type = intFromData(ba);
+	ownPass = intFromData(ba);
 
-	D2I_CLASHT(d2i_PublicKey, type, &key, &p1, size - (2*sizeof(int)));
-	openssl_error();
-
-	encKey_len = size - (p1-p);
-	if (encKey_len) {
-		encKey = (unsigned char *)OPENSSL_malloc(encKey_len);
-		check_oom(encKey);
-		memcpy(encKey, p1 ,encKey_len);
-	}
+	d2i_old(ba, type);
+	pki_openssl_error();
 
+	encKey = ba;
 }
 
