@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2009 Christian Hohnstaedt.
+ * Copyright (C) 2009 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -9,8 +9,11 @@
 #include "pki_key.h"
 #include "func.h"
 #include "db.h"
-#include <qapplication.h>
-#include <widgets/MainWindow.h>
+#include <openssl/rand.h>
+#include <QtGui/QProgressDialog>
+#include <QtGui/QApplication>
+#include <QtCore/QDir>
+#include "widgets/PwDialog.h"
 
 pki_key::pki_key(const QString name)
         :pki_base(name)
@@ -35,7 +38,7 @@ pki_key::pki_key(const pki_key *pk)
 	p = der_key;
 	key = d2i_PUBKEY(NULL, (const unsigned char**)&p, keylen);
 	OPENSSL_free(der_key);
-	openssl_error();
+	pki_openssl_error();
 }
 
 pki_key::~pki_key()
@@ -44,6 +47,42 @@ pki_key::~pki_key()
 		EVP_PKEY_free(key);
 }
 
+void pki_key::d2i(QByteArray &ba)
+{
+	EVP_PKEY *k = (EVP_PKEY*)d2i_bytearray(D2I_VOID(d2i_PUBKEY), ba);
+	if (k) {
+		EVP_PKEY_free(key);
+		key = k;
+	}
+}
+
+void pki_key::d2i_old(QByteArray &ba, int type)
+{
+        if (key)
+		EVP_PKEY_free(key);
+
+	const unsigned char *p, *p1;
+	p = p1 = (const unsigned char *)ba.constData();
+
+	key = d2i_PublicKey(type, NULL, &p1, ba.count());
+        ba = ba.mid(p1-p);
+        pki_openssl_error();
+}
+
+QByteArray pki_key::i2d()
+{
+        return i2d_bytearray(I2D_VOID(i2d_PUBKEY), key);
+}
+
+
+QString pki_key::length()
+{
+	if (key->type == EVP_PKEY_DSA && key->pkey.dsa->p == NULL) {
+		return QString("???");
+	}
+	return QString("%1 bit").arg(EVP_PKEY_bits(key));
+}
+
 QString pki_key::getTypeString()
 {
 	QString type;
@@ -63,9 +102,42 @@ QString pki_key::getTypeString()
 	return type;
 }
 
+QString pki_key::getMsg(msg_type msg)
+{
+	/*
+	 * We do not construct english sentences (just a little bit)
+	 * from fragments to allow proper translations.
+	 * The drawback are all the slightly different duplicated messages
+	 *
+	 * %1 will be replaced by "RSA", "DSA", "EC"
+	 * %2 is the internal name of the key
+	 */
+	QString ktype = getTypeString();
+	if (isPubKey()) {
+		switch (msg) {
+		case msg_import: return tr("Successfully imported the %1 public key '%2'").arg(ktype);
+		case msg_delete: return tr("Delete the %1 public key '%2'?").arg(ktype);
+		default: break;
+		}
+	} else {
+		switch (msg) {
+		case msg_import: return tr("Successfully imported the %1 private key '%2'").arg(ktype);
+		case msg_delete: return tr("Delete the %1 private key '%2'?").arg(ktype);
+		case msg_create: return tr("Successfully created the %1 private key '%2'").arg(ktype);
+		default: break;
+		}
+	}
+	if (msg == msg_delete_multi) {
+		/* %1: Number of keys; %2: list of keynames */
+		return tr("Delete the %1 keys: %2?");
+	}
+	return pki_base::getMsg(msg);
+}
+
 QString pki_key::getIntNameWithType()
 {
-	return getIntName() + " (" + getTypeString() + ")";
+	return QString("%1 (%2%3)").arg(getIntName()).arg(getTypeString()).
+		arg(isPubKey() ? QString(" ") + tr("public key") : QString(""));
 }
 
 QString pki_key::removeTypeFromIntName(QString n)
@@ -79,7 +151,7 @@ QString pki_key::removeTypeFromIntName(QString n)
 	return n;
 }
 
-bool pki_key::isScard()
+bool pki_key::isToken()
 {
 	return false;
 }
@@ -137,7 +209,7 @@ QString pki_key::pubkey()
 		return BN2QString(key->pkey.dsa->pub_key);
 	return QString();
 }
-
+#ifndef OPENSSL_NO_EC
 int pki_key::ecParamNid()
 {
 	if (key->type != EVP_PKEY_EC)
@@ -160,7 +232,7 @@ QString pki_key::ecPubKey()
 	}
 	return pub;
 }
-
+#endif
 bool pki_key::compare(pki_base *ref)
 {
 	pki_key *kref = (pki_key *)ref;
@@ -171,7 +243,7 @@ bool pki_key::compare(pki_base *ref)
 		return false;
 
 	int r = EVP_PKEY_cmp(key, kref->key);
-	openssl_error();
+	pki_openssl_error();
 	return r == 1;
 }
 
@@ -188,7 +260,7 @@ void pki_key::writePublic(const QString fname, bool pem)
 		i2d_PUBKEY_fp(fp, key);
 
 	fclose(fp);
-	openssl_error();
+	pki_openssl_error();
 }
 
 QString pki_key::BNoneLine(BIGNUM *bn) const
@@ -198,7 +270,7 @@ QString pki_key::BNoneLine(BIGNUM *bn) const
 		char *hex = BN_bn2hex(bn);
 		x = hex;
 		OPENSSL_free(hex);
-		openssl_error();
+		pki_openssl_error();
 	}
 	return x;
 }
@@ -220,30 +292,28 @@ QString pki_key::BN2QString(BIGNUM *bn) const
 		x += zs;
 	}
 	OPENSSL_free(buf);
-	openssl_error();
+	pki_openssl_error();
 	return x;
 }
 
-QVariant pki_key::column_data(int col)
+QVariant pki_key::column_data(dbheader *hd)
 {
 	QStringList sl;
 	sl << tr("Common") << tr("Private") << tr("Bogus") << tr("PIN");
-	switch (col) {
-		case 0:
-			return QVariant(getIntName());
-		case 1:
+	switch (hd->id) {
+		case HD_key_type:
 			return QVariant(getTypeString());
-		case 2:
+		case HD_key_size:
 			return QVariant(length());
-		case 3:
+		case HD_key_use:
 			return QVariant(getUcount());
-		case 4:
+		case HD_key_passwd:
 			if (isPubKey())
 				return QVariant(tr("No password"));
 			if (ownPass<0 || ownPass>3)
 				return QVariant("Holla die Waldfee");
 			return QVariant(sl[ownPass]);
 	}
-	return QVariant();
+	return pki_base::column_data(hd);
 }
 
