@@ -1,14 +1,14 @@
 /*
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
-#ifndef PKI_KEY_H
-#define PKI_KEY_H
+#ifndef __PKI_KEY_H
+#define __PKI_KEY_H
 
-#include <qstring.h>
-#include <qprogressbar.h>
+#include <QtCore/QString>
+#include <QtGui/QProgressBar>
 #include <openssl/rsa.h>
 #include <openssl/bn.h>
 #include <openssl/pem.h>
@@ -16,10 +16,10 @@
 #include "pki_base.h"
 
 #define MAX_KEY_LENGTH 4096
-#define MAX_PASS_LENGTH 40
 
 class pki_key: public pki_base
 {
+		Q_OBJECT
 	protected:
 		int ownPass;
 		EVP_PKEY *key;
@@ -34,16 +34,31 @@ class pki_key: public pki_base
 		virtual ~pki_key();
 		enum passType { ptCommon, ptPrivate, ptBogus, ptPin };
 
-		virtual EVP_PKEY *decryptKey() const { return NULL; };
-		virtual QString length() const { return QString(); };
-		virtual bool isPubKey() const { return true; };
-		virtual const EVP_MD *getDefaultMD() { return NULL; };
-		virtual bool isScard();
-		virtual QString length() { return tr("Unknown"); };
+		virtual EVP_PKEY *decryptKey() const
+		{
+			return NULL;
+		}
+		virtual QString length() const
+		{
+			return QString();
+		}
+		virtual bool isPubKey() const
+		{
+			return true;
+		}
+		virtual const EVP_MD *getDefaultMD()
+		{
+			return NULL;
+		}
+		virtual bool isToken();
 		virtual QString getTypeString(void);
 		virtual QString getIntNameWithType(void);
 		virtual QList<int> possibleHashNids()
-			{ return QList<int>(); };
+		{
+			return QList<int>();
+		}
+		virtual QString getMsg(msg_type msg);
+		virtual QString length();
 
 		void writePublic(const QString fname, bool pem);
 		bool compare(pki_base *ref);
@@ -53,15 +68,24 @@ class pki_key: public pki_base
 		int incUcount();
 		int decUcount();
 		int getUcount();
-		int getOwnPass(void) {return ownPass;};
-		EVP_PKEY *getPubKey() {return key;};
-		QVariant column_data(int col);
+		int getOwnPass(void)
+		{
+			return ownPass;
+		}
+		EVP_PKEY *getPubKey()
+		{
+			return key;
+		}
+		QVariant column_data(dbheader *hd);
 		QString modulus();
 		QString pubEx();
 		QString subprime();
 		QString pubkey();
 		int ecParamNid();
 		QString ecPubKey();
+		void d2i(QByteArray &ba);
+		void d2i_old(QByteArray &ba, int type);
+		QByteArray i2d();
 };
 
 #endif
