@@ -13,9 +13,11 @@
 #include "pki_pkcs7.h"
 #include "pki_pkcs12.h"
 #include "pki_crl.h"
+#include "load_obj.h"
 #include "exception.h"
 #include "func.h"
 #include "widgets/MainWindow.h"
+#include <QtCore/QList>
 
 pki_multi::pki_multi(const QString name)
 	:pki_base(name)
@@ -124,7 +126,7 @@ void pki_multi::fromPEM_BIO(BIO *bio, QString name)
 				continue;
 			}
 			pos += startpos;
-			if (BIO_seek(bio, pos))
+			if (BIO_seek(bio, pos) == -1)
 				throw errorEx(tr("Seek failed"));
 			item->fromPEM_BIO(bio, name);
 			if (pos == BIO_tell(bio)) {
@@ -146,3 +148,31 @@ void pki_multi::fromPEM_BIO(BIO *bio, QString name)
 		}
 	}
 }
+
+void pki_multi::probeAnything(const QString fname)
+{
+	pki_base *item = NULL;
+	load_base *lb;
+	QList<load_base *> lbs;
+
+	lbs <<  new load_cert() << new load_pkcs7() << new load_pkcs12() <<
+		new load_crl() <<  new load_req() <<   new load_key() <<
+		new load_temp();
+
+	foreach(lb, lbs) {
+		try {
+			item = lb->loadItem(fname);
+			if (item) {
+				multi.append(item);
+				break;
+			}
+		} catch (errorEx &err) {
+			if (err.info == E_PASSWD) {
+				MainWindow::Error(err);
+				break;
+			}
+		}
+	}
+	while (!lbs.isEmpty())
+		delete lbs.takeFirst();
+}
