@@ -1,20 +1,21 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
-#ifndef PKI_TEMP_H
-#define PKI_TEMP_H
+#ifndef __PKI_TEMP_H
+#define __PKI_TEMP_H
 
 #include "pki_base.h"
 #include "x509name.h"
 #include "asn1time.h"
 #include "pki_x509.h"
 
-class pki_temp: public pki_base
+class pki_temp: public pki_x509name
 {
+		Q_OBJECT
 	protected:
 		int dataSize();
 	public:
@@ -41,11 +42,13 @@ class pki_temp: public pki_base
 		void fromData(const unsigned char *p, db_header_t *head );
 		void oldFromData(unsigned char *p, int size);
 
-		unsigned char *toData(int *size);
+		QByteArray toData();
 		bool compare(pki_base *ref);
 		void writeTemp(QString fname);
-		QVariant column_data(int col);
-		QVariant getIcon(int column);
+		QVariant column_data(dbheader *hd);
+		QVariant getIcon(dbheader *hd);
+		virtual QString getMsg(msg_type msg);
+		x509name getSubject() const;
 };
 
 #endif
