@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -14,16 +14,18 @@
 #include "exception.h"
 #include "pass_info.h"
 #include "widgets/MainWindow.h"
-#include <qdir.h>
+#include <QtGui/QMessageBox>
+#include <QtCore/QDir>
+#include <openssl/rand.h>
 
-QPixmap *pki_x509::icon[5] = { NULL, NULL, NULL, NULL, NULL };
+QPixmap *pki_x509::icon[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
 
 pki_x509::pki_x509(X509 *c)
 	:pki_x509super()
 {
 	init();
 	cert = c;
-	openssl_error();
+	pki_openssl_error();
 }
 
 pki_x509::pki_x509(const pki_x509 *crt)
@@ -31,7 +33,7 @@ pki_x509::pki_x509(const pki_x509 *crt)
 {
 	init();
 	cert = X509_dup(crt->cert);
-	openssl_error();
+	pki_openssl_error();
 	psigner = crt->psigner;
 	setRefKey(crt->getRefKey());
 	trust = crt->trust;
@@ -42,7 +44,7 @@ pki_x509::pki_x509(const pki_x509 *crt)
 	crlDays = crt->crlDays;
 	crlExpiry = crt->crlExpiry;
 	isrevoked = isrevoked;
-	openssl_error();
+	pki_openssl_error();
 }
 
 pki_x509::pki_x509(const QString name)
@@ -51,7 +53,26 @@ pki_x509::pki_x509(const QString name)
 	init();
 	cert = X509_new();
 	X509_set_version(cert, 2);
-	openssl_error();
+	pki_openssl_error();
+}
+
+QString pki_x509::getMsg(msg_type msg)
+{
+	/*
+	 * We do not construct english sentences from fragments
+	 * to allow proper translations.
+	 * The drawback are all the slightly different duplicated messages
+	 *
+	 * %1 will be replaced by the internal name of the certificate
+	 */
+	switch (msg) {
+	case msg_import: return tr("Successfully imported the certificate '%1'");
+	case msg_delete: return tr("Delete the certificate '%1'?");
+	case msg_create: return tr("Successfully created the certificate '%1'");
+	/* %1: Number of certs; %2: list of cert ames */
+	case msg_delete_multi: return tr("Delete the %1 certificates: %2?");
+	}
+	return pki_base::getMsg(msg);
 }
 
 void pki_x509::fromPEM_BIO(BIO *bio, QString name)
@@ -78,18 +99,21 @@ void pki_x509::fload(const QString fname)
 	}
 	_cert = PEM_read_X509(fp, NULL, NULL, NULL);
 	if (!_cert) {
-		ign_openssl_error();
+		pki_ign_openssl_error();
 		rewind(fp);
 		_cert = d2i_X509_fp(fp, NULL);
 	}
 	fclose(fp);
-	openssl_error(fname);
-
+	if (pki_ign_openssl_error() ) {
+		if (_cert)
+			X509_free(_cert);
+		throw errorEx(tr("Unable to load the certificate in file %1. Tried PEM and DER certificate.").arg(fname));
+	}
 	X509_free(cert);
 	cert = _cert;
 	autoIntName();
 	if (getIntName().isEmpty())
-			setIntName(rmslashdot(fname));
+		setIntName(rmslashdot(fname));
 	trust = 1;
 	efftrust = 1;
 }
@@ -99,7 +123,7 @@ pki_x509::~pki_x509()
 	if (cert) {
 		X509_free(cert);
 	}
-	openssl_error();
+	pki_openssl_error();
 }
 
 void pki_x509::init()
@@ -115,9 +139,10 @@ void pki_x509::init()
 	class_name = "pki_x509";
 	cert = NULL;
 	isrevoked = false;
-	dataVersion=1;
-	pkiType=x509;
-	cols=6;
+	dataVersion = 3;
+	pkiType = x509;
+	randomSerial = false;
+	revoke_reason = "";
 }
 
 void pki_x509::setSerial(const a1int &serial)
@@ -126,7 +151,7 @@ void pki_x509::setSerial(const a1int &serial)
 		ASN1_INTEGER_free(cert->cert_info->serialNumber);
 	}
 	cert->cert_info->serialNumber = serial.get();
-	openssl_error();
+	pki_openssl_error();
 }
 
 a1int pki_x509::getSerial() const
@@ -135,13 +160,25 @@ a1int pki_x509::getSerial() const
 	return a;
 }
 
+#define SERIAL_LEN 8
+a1int pki_x509::getIncCaSerial()
+{
+	unsigned char buf[SERIAL_LEN];
+	if (!randomSerial)
+		return caSerial++;
+	RAND_pseudo_bytes(buf, SERIAL_LEN);
+	a1int serial;
+	serial.setRaw(buf, SERIAL_LEN);
+	return serial;
+}
+
 a1int pki_x509::hashInfo(const EVP_MD *md) const
 {
 	unsigned char digest[EVP_MAX_MD_SIZE];
 	unsigned len = 0;
 	if (!ASN1_item_digest(ASN1_ITEM_rptr(X509_CINF), md,
 				(char*)cert->cert_info,digest,&len))
-		openssl_error();
+		pki_openssl_error();
 	a1int a;
 	a.setRaw(digest,len);
 	return a;
@@ -159,9 +196,6 @@ a1int pki_x509::getQASerial(const a1int &secret) const
 
 void pki_x509::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 {
-	QStringList sl = p11.tokenInfo();
-	const unsigned char *p;
-	unsigned long s;
 	QString desc;
 
 	pk11_attr_ulong type(CKA_CERTIFICATE_TYPE);
@@ -180,10 +214,8 @@ void pki_x509::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 	}
 	pk11_attr_data x509(CKA_VALUE);
 	p11.loadAttribute(x509, object);
-	s = x509.getValue(&p);
-	if (cert)
-		X509_free(cert);
-	cert = d2i_X509(NULL, &p, s);
+	QByteArray der = x509.getData();
+	d2i(der);
 
 	if (desc.isEmpty()) {
 		try {
@@ -191,86 +223,167 @@ void pki_x509::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 
 			pk11_attr_data subj(CKA_SUBJECT);
 			p11.loadAttribute(subj, object);
-			s = subj.getValue(&p);
-			xn.d2i(p, s);
+			QByteArray der = subj.getData();
+			xn.d2i(der);
 			desc = xn.getMostPopular();
-			openssl_error();
+			pki_openssl_error();
 		} catch(errorEx &err) {
 			printf("No Cert Subject: %s\n", err.getCString());
 			// IGNORE
 		}
 	}
-	setIntName(sl[0] + " (" + desc + ")");
-	openssl_error();
+	setIntName(desc);
+	pki_openssl_error();
 }
 
-void pki_x509::store_token()
+void pki_x509::d2i(QByteArray &ba)
 {
-	pki_scard *card = (pki_scard *)privkey;
-	int slot, size, id_size;
-	unsigned char *p, *p1, *id;
+        X509 *c = (X509*)d2i_bytearray(D2I_VOID(d2i_X509), ba);
+	if (c) {
+		X509_free(cert);
+		cert = c;
+	}
+}
+
+QByteArray pki_x509::i2d()
+{
+	return i2d_bytearray(I2D_VOID(i2d_X509), cert);
+}
+
+void pki_x509::store_token(bool alwaysSelect)
+{
+	pki_scard *card = NULL;
+	slotid slot;
+	x509name xname;
 	QList<CK_OBJECT_HANDLE> objects;
 
-	if (!privkey || !privkey->isScard())
-		throw errorEx(tr("No associated Smart card"));
+	pkcs11 p11;
 
-	slot = card->prepare_card();
-
-	size = i2d_X509(cert, NULL);
-	openssl_error();
-	p = p1 = (unsigned char*)OPENSSL_malloc(size);
-	i2d_X509(cert, &p1);
-	openssl_error();
-
-	id_size = card->getIdBin(&id);
-	openssl_error();
+	if (!privkey || !privkey->isToken() || alwaysSelect) {
+		if (!p11.selectToken(&slot, NULL))
+			return;
+	} else {
+		card = (pki_scard *)privkey;
+		if (!card->prepare_card(&slot))
+			return;
+	}
 
 	pk11_attlist p11_atts;
 	p11_atts <<
 		pk11_attr_ulong(CKA_CLASS, CKO_CERTIFICATE) <<
 		pk11_attr_ulong(CKA_CERTIFICATE_TYPE, CKC_X_509) <<
-		pk11_attr_bool(CKA_TOKEN, true) <<
-		pk11_attr_data(CKA_VALUE, p, size) <<
-		pk11_attr_data(CKA_ID, id, id_size) <<
-		pk11_attr_data(CKA_LABEL, desc.toUtf8());
+		pk11_attr_data(CKA_VALUE, i2d());
 
-	free(p);
-	free(id);
-
-	pkcs11 p11;
 	p11.startSession(slot, true);
 
-	if (card->scardLogin(p11, false).isNull())
+	QList<CK_OBJECT_HANDLE> objs = p11.objectList(p11_atts);
+	if (objs.count() != 0) {
+		QMessageBox::warning(NULL, XCA_TITLE,
+		    tr("This certificate is already on the security token"));
+		return;
+	}
+
+	p11_atts <<
+		pk11_attr_bool(CKA_TOKEN, true) <<
+		pk11_attr_bool(CKA_PRIVATE, false) <<
+		pk11_attr_data(CKA_SUBJECT, getSubject().i2d()) <<
+		pk11_attr_data(CKA_LABEL, desc.toUtf8()) <<
+		(card ? card->getIdAttr() : p11.findUniqueID(CKO_CERTIFICATE));
+
+	if (p11.tokenLogin(p11.tokenInfo().label(), false).isNull())
 		return;
 
 	p11.createObject(p11_atts);
 }
 
+void pki_x509::deleteFromToken()
+{
+	pki_scard *card = (pki_scard *)privkey;
+	slotidList p11_slots;
+
+	if (!pkcs11::loaded())
+		return;
+
+	if (privkey && privkey->isToken()) {
+		slotid slot;
+		if (!card->prepare_card(&slot))
+			return;
+		p11_slots << slot;
+	} else {
+		pkcs11 p11;
+		p11_slots = p11.getSlotList();
+	}
+	for (int i=0; i<p11_slots.count(); i++) {
+		deleteFromToken(p11_slots[i]);
+	}
+}
+
+pk11_attlist pki_x509::objectAttributes()
+{
+	pk11_attlist attrs;
+        attrs <<
+                pk11_attr_ulong(CKA_CLASS, CKO_CERTIFICATE) <<
+                pk11_attr_ulong(CKA_CERTIFICATE_TYPE, CKC_X_509) <<
+                pk11_attr_data(CKA_VALUE, i2d());
+	return attrs;
+}
+
+void pki_x509::deleteFromToken(slotid slot)
+{
+	pkcs11 p11;
+	p11.startSession(slot, true);
+
+	pk11_attlist atts = objectAttributes();
+	QList<CK_OBJECT_HANDLE> objs = p11.objectList(atts);
+	if (!objs.count())
+		return;
+
+	tkInfo ti = p11.tokenInfo();
+	if (QMessageBox::question(NULL, XCA_TITLE,
+		tr("Delete the certificate '%1' from the token '%2 (#%3)'?").
+		arg(getIntName()).arg(ti.label()).arg(ti.serial()),
+		QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
+	{
+		return;
+	}
+	if (p11.tokenLogin(ti.label(), false).isNull())
+		return;
+
+	p11.deleteObjects(objs);
+}
+
+int pki_x509::renameOnToken(slotid slot, QString name)
+{
+
+	pkcs11 p11;
+	p11.startSession(slot, true);
+	pk11_attlist attrs = objectAttributes();
+
+	QList<CK_OBJECT_HANDLE> objs = p11.objectList(attrs);
+	if (!objs.count())
+		return 0;
+
+	pk11_attr_data label(CKA_LABEL, name.toUtf8());
+	tkInfo ti = p11.tokenInfo();
+	if (p11.tokenLogin(ti.label(), false).isNull())
+                return 0;
+	p11.storeAttribute(label, objs[0]);
+	return 1;
+}
+
 bool pki_x509::verifyQASerial(const a1int &secret) const
 {
 	return getQASerial(secret) == getSerial();
 }
 
-void pki_x509::set_date(ASN1_TIME **a, const a1time &a1)
+void pki_x509::setNotBefore(const a1time &a)
 {
-	if (*a != NULL )
-		ASN1_TIME_free(*a);
-
-	*a = a1.get_utc();
-	if (*a == NULL)
-		*a = a1.get();
-
-	openssl_error();
+	a.set_date(&X509_get_notBefore(cert));
 }
 
-void pki_x509::setNotBefore(const a1time &a1)
+void pki_x509::setNotAfter(const a1time &a)
 {
-	set_date(&X509_get_notBefore(cert), a1);
-}
-
-void pki_x509::setNotAfter(const a1time &a1)
-{
-	set_date(&X509_get_notAfter(cert), a1);
+	a.set_date(&X509_get_notAfter(cert));
 }
 
 a1time pki_x509::getNotBefore() const
@@ -288,14 +401,14 @@ a1time pki_x509::getNotAfter() const
 x509name pki_x509::getSubject() const
 {
 	x509name x(cert->cert_info->subject);
-	openssl_error();
+	pki_openssl_error();
 	return x;
 }
 
 x509name pki_x509::getIssuer() const
 {
 	x509name x(cert->cert_info->issuer);
-	openssl_error();
+	pki_openssl_error();
 	return x;
 }
 
@@ -319,7 +432,7 @@ void pki_x509::addV3ext(const x509v3ext &e)
 	X509_EXTENSION *ext = e.get();
 	X509_add_ext(cert, ext, -1);
 	X509_EXTENSION_free(ext);
-	openssl_error();
+	pki_openssl_error();
 }
 
 void pki_x509::delSigner(pki_base *s)
@@ -334,10 +447,10 @@ bool pki_x509::canSign()
 	int crit;
 	if (!privkey || privkey->isPubKey())
 		return false;
-	if (privkey->isScard() && !pkcs11::loaded())
+	if (privkey->isToken() && !pkcs11::loaded())
 		return false;
 	bc = (BASIC_CONSTRAINTS *)X509_get_ext_d2i(cert, NID_basic_constraints, &crit, NULL);
-	openssl_error();
+	pki_openssl_error();
 	if (!bc || !bc->ca)
 		return false;
 	return true;
@@ -345,16 +458,7 @@ bool pki_x509::canSign()
 
 bool pki_x509::hasExtension(int nid)
 {
-	int i;
-	STACK_OF(X509_EXTENSION) *x = cert->cert_info->extensions;
-
-	for(i = 0; i < sk_X509_EXTENSION_num(x); i++) {
-		X509_EXTENSION *ex = sk_X509_EXTENSION_value(x, i);
-		if (OBJ_obj2nid(ex->object) == nid) {
-			return true;
-		}
-	}
-	return false;
+	return getV3ext().idxByNid(nid) != -1;
 }
 
 void pki_x509::sign(pki_key *signkey, const EVP_MD *digest)
@@ -364,67 +468,67 @@ void pki_x509::sign(pki_key *signkey, const EVP_MD *digest)
 		my_error(tr("There is no key for signing !"));
 	}
 	tkey = signkey->decryptKey();
-	openssl_error();
+	pki_openssl_error();
 	X509_sign(cert, tkey, digest);
-	openssl_error();
+	pki_openssl_error();
 	EVP_PKEY_free(tkey);
-	openssl_error();
+	pki_openssl_error();
 }
 
 void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 {
 	int version, size;
-	const unsigned char *p1 = p;
 
-	X509 *cert_sik = cert;
 	version = head->version;
 	size = head->len - sizeof(db_header_t);
 
-	cert = D2I_CLASH(d2i_X509, NULL, &p1, size);
-	trust = db::intFromData(&p1);
-	isrevoked = db::boolFromData(&p1);
-	p1 = revoked.d2i(p1, size - (p1-p));
-	caSerial.setHex(db::stringFromData(&p1));
-	caTemplate = db::stringFromData(&p1);
-	crlDays = db::intFromData(&p1);
-	crlExpiry.d2i(p1, size - (p1-p));
+	QByteArray ba((const char*)p, size);
 
-	if (cert)
-		X509_free(cert_sik);
+	d2i(ba);
+	trust = db::intFromData(ba);
+	isrevoked = db::boolFromData(ba);
+	revoked.d2i(ba);
+	caSerial.setHex(db::stringFromData(ba));
+	caTemplate = db::stringFromData(ba);
+	crlDays = db::intFromData(ba);
+	crlExpiry.d2i(ba);
+	if (version > 1)
+		randomSerial = db::boolFromData(ba);
 	else
-		cert = cert_sik;
-	openssl_error();
+		randomSerial = false;
+	if (version > 2) {
+		crlNumber.setHex(db::stringFromData(ba));
+		revoke_reason = db::stringFromData(ba);
+		invalDate.d2i(ba);
+	}
+	if (ba.count() > 0) {
+		my_error(tr("Wrong Size %1").arg(ba.count()));
+	}
 }
 
 
-unsigned char *pki_x509::toData(int *size)
+QByteArray pki_x509::toData()
 {
-	unsigned char *p, *p1;
+	QByteArray ba;
 
-	// calculate the needed size
-	*size = i2d_X509(cert, NULL) + 11 + caSerial.toHex().length() +
-		caTemplate.length() + crlExpiry.derSize() + revoked.derSize();
-	openssl_error();
-	p = p1 = (unsigned char*)OPENSSL_malloc(*size);
-
-	i2d_X509(cert, &p1); // cert
-	db::intToData(&p1, trust); // trust
-	db::boolToData(&p1, isrevoked);
-	p1 = revoked.i2d(p1); // revokation date
+	ba += i2d(); // cert
+	ba += db::intToData(trust);
+	ba += db::boolToData(isrevoked);
+	ba += revoked.i2d(); // revokation date
 
 	// the serial if this is a CA
-	db::stringToData(&p1, caSerial.toHex());
+	ba += db::stringToData(caSerial.toHex());
 	// the name of the template to use for signing
-	db::stringToData(&p1, caTemplate);
+	ba += db::stringToData(caTemplate);
 	// version 3
-	db::intToData(&p1, crlDays); // the CRL period
-	p1 = crlExpiry.i2d(p1); // last CRL date
-	openssl_error();
-	if (*size != p1-p) {
-		printf("pki_x509::toData: Size = %d, real size=%zd\n", *size, p1-p);
-		//abort();
-	}
-	return p;
+	ba += db::intToData(crlDays); // the CRL period
+	ba += crlExpiry.i2d(); // last CRL date
+	ba += db::boolToData(randomSerial);
+	ba += db::stringToData(crlNumber.toHex());
+	ba += db::stringToData(revoke_reason);
+	ba += invalDate.i2d();
+	pki_openssl_error();
+	return ba;
 }
 
 void pki_x509::writeDefault(const QString fname)
@@ -448,22 +552,23 @@ void pki_x509::writeCert(const QString fname, bool PEM, bool append)
 				i2d_X509_fp(fp, cert);
 		}
 		fclose(fp);
-		openssl_error();
+		pki_openssl_error();
 	} else
 		fopen_error(fname);
 }
 
-bool pki_x509::compare(pki_base *ref)
+BIO *pki_x509::pem(BIO *b)
 {
-	bool ret = !X509_cmp(cert, ((pki_x509 *)ref)->cert);
-	ign_openssl_error();
-	return ret;
+	if (!b)
+		b = BIO_new(BIO_s_mem());
+	PEM_write_bio_X509(b, cert);
+	return b;
 }
 
 bool pki_x509::cmpIssuerAndSerial(pki_x509 *refcert)
 {
 	bool ret =  X509_issuer_and_serial_cmp(cert, refcert->cert);
-	openssl_error();
+	pki_openssl_error();
 	return ret;
 
 }
@@ -476,12 +581,17 @@ bool pki_x509::verify(pki_x509 *signer)
 		return false;
 	X509_NAME *subject =  X509_get_subject_name(signer->cert);
 	X509_NAME *issuer = X509_get_issuer_name(cert);
-	openssl_error();
+	pki_openssl_error();
 	if (X509_NAME_cmp(subject, issuer)) {
 		return false;
 	}
-	int i = X509_verify(cert, X509_get_pubkey(signer->cert));
-	ign_openssl_error();
+	EVP_PKEY *pub = X509_get_pubkey(signer->cert);
+	if (!pub) {
+		pki_ign_openssl_error();
+		return false;
+	}
+	int i = X509_verify(cert, pub);
+	pki_ign_openssl_error();
 	if (i>0) {
 		psigner = signer;
 		return true;
@@ -493,8 +603,11 @@ bool pki_x509::verify(pki_x509 *signer)
 pki_key *pki_x509::getPubKey() const
 {
 	EVP_PKEY *pkey = X509_get_pubkey(cert);
-	openssl_error();
+	pki_ign_openssl_error();
+	if (pkey == NULL)
+		return NULL;
 	pki_evp *key = new pki_evp(pkey);
+	pki_openssl_error();
 	return key;
 }
 
@@ -511,7 +624,7 @@ QString pki_x509::fingerprint(const EVP_MD *digest)
 	unsigned int n;
 	unsigned char md[EVP_MAX_MD_SIZE];
 	X509_digest(cert, digest, md, &n);
-	openssl_error();
+	pki_openssl_error();
 	for (j=0; j<(int)n; j++) {
 		sprintf(zs, "%02X%c",md[j], (j+1 == (int)n) ?'\0':':');
 		fp += zs;
@@ -525,25 +638,7 @@ int pki_x509::checkDate()
 	int ret = 0;
 	if (getNotAfter() <  a) ret = -1;
 	if (getNotBefore() > a) ret = 1;
-	openssl_error();
-	return ret;
-}
-
-int pki_x509::resetTimes(pki_x509 *signer)
-{
-	int ret = 0;
-	if (!signer) return -1;
-	if (getNotAfter() > signer->getNotAfter()) {
-		// client cert is longer valid....
-		setNotAfter(signer->getNotAfter());
-		ret=1;
-	}
-	if (getNotBefore() < signer->getNotBefore()) {
-		// client cert is longer valid....
-		setNotBefore(signer->getNotBefore());
-		ret=2;
-	}
-	openssl_error();
+	pki_openssl_error();
 	return ret;
 }
 
@@ -600,16 +695,17 @@ bool pki_x509::isRevoked()
 	return isrevoked ;
 }
 
-void pki_x509::setRevoked(bool rev)
+void pki_x509::setRevoked(bool rev, a1time inval, QString reason)
 {
 	if (rev) {
 		setEffTrust(0);
-		setTrust(0);
 		revoked.now();
-		openssl_error();
+		pki_openssl_error();
+		revoke_reason = reason;
+		invalDate = inval;
 	}
 	isrevoked = rev;
-	openssl_error();
+	pki_openssl_error();
 }
 
 a1time &pki_x509::getRevoked()
@@ -623,7 +719,7 @@ void pki_x509::setRevoked(const a1time &when)
 	revoked = when;
 	setEffTrust(0);
 	setTrust(0);
-	openssl_error();
+	pki_openssl_error();
 }
 
 int pki_x509::calcEffTrust()
@@ -633,6 +729,10 @@ int pki_x509::calcEffTrust()
 		efftrust = mytrust;
 		return mytrust;
 	}
+	if (isRevoked()) {
+		efftrust = 0;
+		return 0;
+	}
 	if (getSigner() == this && trust == 1) { // inherit trust, but self signed
 		trust=0;
 		efftrust=0;
@@ -650,71 +750,107 @@ int pki_x509::calcEffTrust()
 	return mytrust;
 }
 
-a1int pki_x509::getIncCaSerial() { return caSerial++; }
-
-a1int pki_x509::getCaSerial() { return caSerial; }
-
-void pki_x509::setCaSerial(a1int s) { caSerial = s; }
-
-int pki_x509::getCrlDays() {return crlDays;}
-
-void pki_x509::setCrlDays(int s){if (s>0) crlDays = s;}
-
-QString pki_x509::getTemplate(){ return caTemplate; }
-
-void pki_x509::setTemplate(QString s) {if (s.length()>0) caTemplate = s; }
-
 void pki_x509::setCrlExpiry(const a1time &time)
 {
 	crlExpiry = time;
-	openssl_error();
+	pki_openssl_error();
 }
 
-x509rev pki_x509::getRev()
+x509rev pki_x509::getRev(bool reason)
 {
 	x509rev a;
 	a.setDate(getRevoked());
 	a.setSerial(getSerial());
+	if (reason) {
+		a.setReason(revoke_reason);
+		a.setInvalDate(invalDate);
+	}
 	return a;
 }
 
-QVariant pki_x509::column_data(int col)
+bool pki_x509::caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen)
+{
+	x509v3ext e = getExtByNid(NID_basic_constraints);
+	if (e.nid() != NID_basic_constraints)
+		return false;
+	BASIC_CONSTRAINTS *bc = (BASIC_CONSTRAINTS *)e.d2i();
+	if (hasLen)
+		*hasLen = bc->pathlen ? true : false;
+	if (pathlen && bc->pathlen)
+		pathlen->set(bc->pathlen);
+	if (ca)
+		*ca = bc->ca;
+	BASIC_CONSTRAINTS_free(bc);
+	return true;
+}
+
+QVariant pki_x509::column_data(dbheader *hd)
 {
 	QString truststatus[] =
 		{ tr("Not trusted"), tr("Trust inherited"), tr("Always Trusted") };
 
-	switch (col) {
-		case 0:
-			return QVariant(getIntName());
-		case 1:
-			return QVariant(getSubject().getEntryByNid(NID_commonName));
-		case 2:
+	switch (hd->id) {
+		case HD_cert_serial:
 			return QVariant(getSerial().toHex());
-		case 3:
+		case HD_cert_notBefore:
+			return QVariant(getNotBefore().toSortable());
+		case HD_cert_notAfter:
 			return QVariant(getNotAfter().toSortable());
-		case 4:
-			return QVariant(truststatus[ getTrust() ]);
-		case 5:
+		case HD_cert_trust:
+			return QVariant(truststatus[getTrust()]);
+		case HD_cert_revokation:
 			if (isRevoked())
 				return QVariant(getRevoked().toSortable());
 			else if (canSign())
-				return QVariant(tr("CRL expires: ")+ crlExpiry.toSortable());
+				return QVariant(tr("CRL expires: %1").
+					arg(crlExpiry.toSortable()));
+			return QVariant();
+		case HD_cert_md5fp:
+			return QVariant(fingerprint(EVP_md5()));
+		case HD_cert_sha1fp:
+			return QVariant(fingerprint(EVP_sha1()));
+		case HD_cert_ca: {
+			a1int len;
+			bool ca, haslen;
+			if (caAndPathLen(&ca, &len, &haslen)) {
+				if (ca && haslen)
+					return QVariant(len.toDec());
+				if (!ca)
+					return QVariant(tr("No"));
+				else
+					return QVariant(tr("Yes"));
+			}
+			return QVariant("");
+		}
 	}
-	return QVariant();
-
+	return pki_x509super::column_data(hd);
 }
 
-QVariant pki_x509::getIcon(int column)
+QVariant pki_x509::getIcon(dbheader *hd)
 {
 	int pixnum = 0;
-	if (column != 0)
-		return QVariant();
+	bool ca;
+	pki_key *k;
 
-	if (getRefKey()) {
-		pixnum += 1;
-	}
-	if (calcEffTrust() == 0){
-		pixnum += 2;
+	switch (hd->id) {
+	case HD_cert_ca:
+		if (!caAndPathLen(&ca, NULL, NULL))
+			return QVariant();
+		if (!ca)
+			return QVariant();
+		pixnum = 5;
+		break;
+	case HD_internal_name:
+		k = getRefKey();
+		if (k && k->isPrivKey()) {
+			pixnum += 1;
+		}
+		if (calcEffTrust() == 0){
+			pixnum += 2;
+		}
+		break;
+	default:
+		return QVariant();
 	}
 	return QVariant(*icon[pixnum]);
 }
@@ -732,18 +868,19 @@ const EVP_MD *pki_x509::getDigest()
 
 void pki_x509::oldFromData(unsigned char *p, int size)
 {
-	int version, sCert, sRev, sLastCrl;
-	const unsigned char *p1 = p;
+	int version, sRev, sLastCrl;
+	QByteArray ba((char*)p, size);
 	X509 *cert_sik = cert;
-	version = intFromData(&p1);
+	cert = NULL;
+	version = intFromData(ba);
 	if (version >=1 && version <= 5) {
-		sCert = intFromData(&p1);
-		cert = D2I_CLASH(d2i_X509, NULL, &p1, sCert);
-		trust = intFromData(&p1);
-		sRev = intFromData(&p1);
+		intFromData(ba); /* sCert */
+		d2i(ba);
+		trust = intFromData(ba);
+		sRev = intFromData(ba);
 		if (sRev) {
 			if (version != 3) isrevoked = true;
-			p1 = revoked.d2i(p1, sRev);
+			revoked.d2i(ba);
 		}
 		else {
 		   isrevoked = false;
@@ -759,9 +896,9 @@ void pki_x509::oldFromData(unsigned char *p, int size)
 
 		if (version >= 2 ) {
 			if (version >= 5)
-				caSerial.setHex(db::stringFromData(&p1));
+				caSerial.setHex(db::stringFromData(ba));
 			else {
-				int i = intFromData(&p1);
+				int i = intFromData(ba);
 				if (i>=0)
 					caSerial = i;
 				else {
@@ -769,21 +906,20 @@ void pki_x509::oldFromData(unsigned char *p, int size)
 					++caSerial;
 				}
 			}
-			caTemplate = db::stringFromData(&p1);
+			caTemplate = db::stringFromData(ba);
 		}
 		if (version >= 3 ) {
-			crlDays = intFromData(&p1);
-			sLastCrl = intFromData(&p1);
+			crlDays = intFromData(ba);
+			sLastCrl = intFromData(ba);
 			if (sLastCrl) {
-			   crlExpiry.d2i(p1, sLastCrl);
+			   crlExpiry.d2i(ba);
 			}
 		}
 		// version 4 saves a NULL as revoked
 		// version 3 did save a recent date :-((
 	}
 	else { // old version
-		p1 = p;
-		cert = D2I_CLASH(d2i_X509, NULL, &p1, size);
+		d2i(ba);
 		revoked = NULL;
 		trust = 1;
 		efftrust = 1;
@@ -792,6 +928,9 @@ void pki_x509::oldFromData(unsigned char *p, int size)
 		X509_free(cert_sik);
 	else
 		cert = cert_sik;
-	openssl_error();
+
+	if (ba.count() > 0) {
+		my_error(tr("Wrong Size %1").arg(ba.count()));
+	}
 }
 
