@@ -1,12 +1,12 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
-#ifndef PKI_X509_H
-#define PKI_X509_H
+#ifndef __PKI_X509_H
+#define __PKI_X509_H
 
 #include <stdio.h>
 #include <openssl/x509.h>
@@ -23,20 +23,22 @@
 
 class pki_x509 : public pki_x509super
 {
+		Q_OBJECT
 	private:
 		pki_x509 *psigner;
-		a1time revoked, crlExpiry;
-		bool isrevoked;
+		a1time revoked, crlExpiry, invalDate;
+		bool isrevoked, randomSerial;
 		int trust;
 		int efftrust;
 		a1int caSerial;
+		a1int crlNumber;
 		int crlDays;
 		QString caTemplate;
 		X509 *cert;
+		QString revoke_reason;
 		void init();
-		void set_date(ASN1_TIME **a, const a1time &a1);
 	public:
-		static QPixmap *icon[5];
+		static QPixmap *icon[6];
 		pki_x509(X509 *c);
 		pki_x509(const pki_x509 *crt);
 		pki_x509(const QString name = "");
@@ -44,7 +46,7 @@ class pki_x509 : public pki_x509super
 
 		void fload(const QString fname);
 		void load_token(pkcs11 &p11, CK_OBJECT_HANDLE object);
-		void store_token();
+		void store_token(bool alwaysSelect);
 		void fromPEM_BIO(BIO *bio, QString name);
 		void writeDefault(const QString fname);
 		a1int hashInfo(const EVP_MD *md) const;
@@ -52,19 +54,19 @@ class pki_x509 : public pki_x509super
 		bool verifyQASerial(const a1int &secret) const;
 		void setSerial(const a1int &serial);
 		a1int getSerial() const;
-		void setNotBefore(const a1time &a1);
-		void setNotAfter(const a1time &a1);
+		void setNotBefore(const a1time &a);
+		void setNotAfter(const a1time &a);
 		a1time getNotBefore() const;
 		a1time getNotAfter() const;
 		x509name getSubject() const;
 		x509name getIssuer() const;
 		void setSubject(const x509name &n);
 		void setIssuer(const x509name &n);
+		bool caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen);
 
-		unsigned char *toData(int *size);
+		QByteArray toData();
 		void fromData(const unsigned char *p, db_header_t *head);
 		void oldFromData(unsigned char *p, int size);
-		bool compare(pki_base *refcert);
 		bool canSign();
 		void writeCert(const QString fname, bool PEM, bool append = false);
 		bool verify(pki_x509 *signer);
@@ -77,34 +79,77 @@ class pki_x509 : public pki_x509super
 		int checkDate();
 		void addV3ext(const x509v3ext &e);
 		void sign(pki_key *signkey, const EVP_MD *digest);
-		X509 *getCert(){ return cert;}
+		X509 *getCert()
+		{
+			return cert;
+		}
 		int getTrust();
 		void setTrust(int t);
 		int getEffTrust();
 		void setEffTrust(int t);
-		void setRevoked(bool rev);
+		void setRevoked(bool rev, a1time inval = a1time(),
+				QString reason = QString());
 		void setRevoked(const a1time &when);
 		a1time &getRevoked();
 		bool isRevoked();
 		int calcEffTrust();
 		a1int getIncCaSerial();
-		a1int getCaSerial();
-		void setCaSerial(a1int s);
-		void setTemplate(QString s);
-		QString getTemplate();
-		void setCrlDays(int s);
-		int getCrlDays();
+		a1int getCaSerial()
+		{
+			return caSerial;
+		}
+		void setCaSerial(a1int s)
+		{
+			caSerial = s;
+		}
+		a1int getIncCrlNumber()
+		{
+			return ++crlNumber;
+		}
+		void setTemplate(QString s)
+		{
+			if (s.length() > 0)
+				caTemplate = s;
+		}
+		QString getTemplate()
+		{
+			return caTemplate;
+		}
+		void setCrlDays(int s)
+		{
+			if (s > 0)
+				crlDays = s;
+		}
+		int getCrlDays()
+		{
+			return crlDays;
+		}
+		bool usesRandomSerial()
+		{
+			return randomSerial;
+		}
+		void setUseRandomSerial(bool r)
+		{
+			randomSerial = r;
+		}
+		pk11_attlist objectAttributes();
 		void setCrlExpiry(const a1time &time);
-		int resetTimes(pki_x509 *signer);
 		bool hasExtension(int nid);
 		bool cmpIssuerAndSerial(pki_x509 *refcert);
 		void updateView();
-		x509rev getRev();
+		x509rev getRev(bool reason = true);
 		QString getSigAlg();
 		x509v3ext getExtByNid(int nid);
 		const EVP_MD *getDigest();
-		QVariant column_data(int col);
-		QVariant getIcon(int column);
+		QVariant column_data(dbheader *hd);
+		QVariant getIcon(dbheader *hd);
+		QByteArray i2d();
+		void d2i(QByteArray &ba);
+		void deleteFromToken();
+		void deleteFromToken(slotid slot);
+		virtual QString getMsg(msg_type msg);
+		virtual int renameOnToken(slotid slot, QString name);
+		BIO *pem(BIO *);
 };
 
 #endif
