@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -18,26 +18,52 @@
 #include "widgets/CrlDetail.h"
 #include "widgets/CertDetail.h"
 #include "widgets/KeyDetail.h"
-#include "widgets/ReqDetail.h"
-#include <qpushbutton.h>
-#include <qmessagebox.h>
-#include <qlabel.h>
+#include <QtGui/QPushButton>
+#include <QtGui/QMessageBox>
+#include <QtGui/QLabel>
+#include <QtGui/QInputDialog>
+#include <QtCore/QUrl>
 
 ImportMulti::ImportMulti(MainWindow *parent)
 	:QDialog(parent)
 {
 	mainwin = parent;
 	setupUi(this);
-	setWindowTitle(tr(XCA_TITLE));
+	setWindowTitle(XCA_TITLE);
 	image->setPixmap(*MainWindow::certImg);
 	listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
-	mcont = new db_base("/dev/null", parent);
+	mcont = new db_token(QString(), parent);
 	listView->setModel(mcont);
 	listView->setIconSize(pki_evp::icon[0]->size());
 	listView->setSelectionMode(QAbstractItemView::ExtendedSelection);
 	connect( listView, SIGNAL(doubleClicked(const QModelIndex &)),
 		this, SLOT(on_butDetails_clicked()));
+	deleteToken->hide();
+	renameToken->hide();
+	slotInfo->hide();
+	setAcceptDrops(true);
+}
 
+void ImportMulti::tokenInfo(slotid s)
+{
+	slot = s;
+	mcont->setSlot(slot);
+	deleteToken->show();
+	renameToken->show();
+	slotInfo->show();
+	listView->setEditTriggers(QAbstractItemView::EditKeyPressed);
+
+	pkcs11 p11;
+
+	QString info = p11.driverInfo(slot);
+	tkInfo ti = p11.tokenInfo(slot);
+	info += tr("\nName: %1\nModel: %2\nSerial: %3").
+		arg(ti.label()).arg(ti.model()).arg(ti.serial());
+
+	slotInfo->setText(info);
+	image->setPixmap(*MainWindow::scardImg);
+	heading->setText(tr("Manage security token"));
+	setAcceptDrops(false);
 }
 
 void ImportMulti::addItem(pki_base *pki)
@@ -76,10 +102,35 @@ void ImportMulti::addItem(pki_base *pki)
 	}
 	else  {
 		QMessageBox::warning(this, XCA_TITLE,
-			tr("The type of the Item is not recognized: ") + cn, tr("OK"));
+			tr("The type of the Item '%1' is not recognized").
+			arg(cn));
 	}
 }
 
+void ImportMulti::dragEnterEvent(QDragEnterEvent *event)
+{
+	if (event->mimeData()->hasUrls())
+		event->acceptProposedAction();
+}
+
+void ImportMulti::dropEvent(QDropEvent *event)
+{
+	QList<QUrl> urls = event->mimeData()->urls();
+	QUrl u;
+	QStringList failed;
+	pki_multi *pki = new pki_multi();
+
+	foreach(u, urls) {
+		QString s = u.toLocalFile();
+		int count = pki->count();
+		pki->probeAnything(s);
+		if (pki->count() == count)
+			failed << s;
+	}
+	importError(failed);
+	addItem(pki);
+}
+
 void ImportMulti::on_butRemove_clicked()
 {
 	QItemSelectionModel *selectionModel = listView->selectionModel();
@@ -118,13 +169,66 @@ void ImportMulti::on_butImport_clicked()
 	}
 }
 
-void ImportMulti::import(QModelIndex &idx)
+void ImportMulti::on_deleteToken_clicked()
+{
+	QItemSelectionModel *selectionModel = listView->selectionModel();
+	QModelIndexList indexes = selectionModel->selectedIndexes();
+	QModelIndex index;
+	QString items;
+
+	foreach(index, indexes) {
+		if (index.column() != 0)
+			continue;
+		pki_base *pki = static_cast<pki_base*>(index.internalPointer());
+		try {
+			pki->deleteFromToken(slot);
+			mcont->remFromCont(index);
+			delete pki;
+		} catch (errorEx &err) {
+			mainwin->Error(err);
+		}
+	}
+}
+void ImportMulti::on_renameToken_clicked()
+{
+	QItemSelectionModel *selectionModel = listView->selectionModel();
+	QModelIndexList indexes = selectionModel->selectedIndexes();
+	QModelIndex index;
+	QString items;
+
+        foreach(index, indexes) {
+                if (index.column() != 0)
+                        continue;
+		listView->edit(index);
+		break;
+	}
+}
+
+static db_base *select_db(QString cn)
+{
+	if (cn == "pki_x509")
+		return MainWindow::certs;
+	if (cn == "pki_evp")
+		return MainWindow::keys;
+	if (cn == "pki_scard")
+		return MainWindow::keys;
+	if (cn == "pki_x509req")
+		return MainWindow::reqs;
+	if (cn == "pki_crl")
+		return MainWindow::crls;
+	if (cn == "pki_temp")
+		return MainWindow::temps;
+	return NULL;
+}
+
+pki_base *ImportMulti::import(QModelIndex &idx)
 {
 
 	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
+	db_base *db;
 
 	if (!pki)
-		return;
+		return NULL;
 	if (!mainwin->keys) {
 		mainwin->load_database();
 	}
@@ -133,27 +237,21 @@ void ImportMulti::import(QModelIndex &idx)
 	mcont->remFromCont(idx);
 	if (!mainwin->keys) {
 		delete pki;
-		return;
+		return NULL;
 	}
 
-	if (cn == "pki_x509") {
-		MainWindow::certs->insert(pki);
-	} else if (cn == "pki_evp") {
+	if (cn == "pki_evp")
 		((pki_evp*)pki)->setOwnPass(pki_evp::ptCommon);
-		MainWindow::keys->insert(pki);
-	} else if (cn == "pki_scard") {
-		MainWindow::keys->insert(pki);
-	} else if (cn == "pki_x509req") {
-		MainWindow::reqs->insert(pki);
-	} else if (cn == "pki_crl") {
-		MainWindow::crls->insert(pki);
-	} else if (cn == "pki_temp") {
-		MainWindow::temps->insert(pki);
-	} else  {
+
+	db = select_db(cn);
+	if (!db) {
 		QMessageBox::warning(this, XCA_TITLE,
-			tr("The type of the Item is not recognized: ") + cn, tr("OK"));
+			tr("The type of the Item '%1' is not recognized").
+			arg(cn));
 		delete pki;
+		return NULL;
 	}
+	return db->insert(pki);
 }
 
 void ImportMulti::on_butDetails_clicked()
@@ -184,8 +282,8 @@ void ImportMulti::on_butDetails_clicked()
 			dlg->exec();
 			delete dlg;
 		} else if (cn == "pki_x509req") {
-			ReqDetail *dlg;
-			dlg = new ReqDetail(mainwin);
+			CertDetail *dlg;
+			dlg = new CertDetail(mainwin);
 			dlg->setReq((pki_x509req *)pki);
 			dlg->exec();
 			delete dlg;
@@ -197,15 +295,15 @@ void ImportMulti::on_butDetails_clicked()
 			delete dlg;
 		} else if (cn == "pki_temp") {
 			QMessageBox::warning(this, XCA_TITLE,
-				tr("Details of this item cannot be shown: ") + cn, tr("OK"));
+				tr("Details of the item '%1' cannot be shown").
+				arg(cn));
 		} else
 			QMessageBox::warning(this, XCA_TITLE,
-				tr("The type of the Item is not recognized ") + cn, tr("OK"));
+				tr("The type of the item '%1' is not recognized").
+				arg(cn));
 	}
 	catch (errorEx &err) {
-		QMessageBox::warning(this, XCA_TITLE,
-			tr("Error") + pki->getClassName() +
-			err.getString(), tr("OK"));
+		mainwin->Error(err);
 	}
 
 }
@@ -228,16 +326,40 @@ int ImportMulti::entries()
 	return mcont->rootItem->childCount();
 }
 
-void ImportMulti::execute(int force)
+void ImportMulti::importError(QStringList failed)
 {
+	if (failed.count() == 1) {
+		QMessageBox::information(this, XCA_TITLE,
+			tr("The file '%1' did not contain PKI data").
+			arg(failed[0]));
+	} else if (failed.count() > 1) {
+		QMessageBox::information(this, XCA_TITLE,
+			tr("The %1 files: '%2' did not contain PKI data").
+			arg(failed.count()).
+			arg(failed.join("', '")));
+	}
+}
+
+void ImportMulti::execute(int force, QStringList failed)
+{
+	importError(failed);
+
 	/* if there is nothing to import don't pop up */
-	if (entries() == 0)
-		return;
-	/* if there is only 1 item and force is 0 import it silently */
-	if (entries() == 1 && force == 0) {
-		on_butOk_clicked();
+	if (entries() == 0) {
+		accept();
 		return;
 	}
-	/* the behavoiour for more than one item */
+	/* if there is only 1 item and force is 0 import it silently */
+	if (entries() == 1 && force == 0) {
+		QModelIndex idx = mcont->index(0, 0, QModelIndex());
+		pki_base *pki = import(idx);
+		if (pki && !pki_base::suppress_messages)
+			QMessageBox::information(this, XCA_TITLE,
+				pki->getMsg(pki_base::msg_import).
+				arg(pki->getIntName()));
+		accept();
+		return;
+	}
+	/* the behaviour for more than one item */
 	exec();
 }
