@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -10,23 +10,25 @@
 #include "lib/exception.h"
 #include "lib/pki_evp.h"
 #include "lib/pki_scard.h"
-#include <qdir.h>
-#include <qstatusbar.h>
-#include <qmessagebox.h>
+#include <QtCore/QDir>
+#include <QtGui/QStatusBar>
+#include <QtGui/QMessageBox>
 #include "lib/db_base.h"
 #include "lib/func.h"
 #include "widgets/ImportMulti.h"
 
-void MainWindow::init_database()
+int MainWindow::init_database()
 {
+	int ret = 2;
 	fprintf(stderr, "Opening database: %s\n", QString2filename(dbfile));
 	keys = NULL; reqs = NULL; certs = NULL; temps = NULL; crls = NULL;
 
 	certView->setRootIsDecorated(db_x509::treeview);
 
 	try {
-		if (!initPass())
-			return;
+		ret = initPass();
+		if (ret == 2)
+			return ret;
 		keys = new db_key(dbfile, this);
 		reqs = new db_x509req(dbfile, this);
 		certs = new db_x509(dbfile, this);
@@ -36,7 +38,20 @@ void MainWindow::init_database()
 	catch (errorEx &err) {
 		Error(err);
 		dbfile = "";
-		return;
+		return ret;
+	}
+
+	mandatory_dn = "";
+	string_opt = QString("MASK:0x2002");
+	ASN1_STRING_set_default_mask_asc((char*)CCHAR(string_opt));
+	hashBox::resetDefault();
+	pkcs11path = getDefaultPkcs11Lib();
+	workingdir = QDir::currentPath();
+	pki_base::suppress_messages = 0;
+	try {
+		pkcs11_lib p(pkcs11path);
+	} catch (errorEx &e) {
+		pkcs11path = QString();
 	}
 
 	connect( keys, SIGNAL(newKey(pki_key *)),
@@ -114,15 +129,47 @@ void MainWindow::init_database()
 				free(p);
 			}
 		}
+		mydb.first();
+		if (!mydb.find(setting, "suppress")) {
+			if ((p = (char *)mydb.load(NULL))) {
+				QString x = p;
+				free(p);
+				if (x == "1")
+					pki_base::suppress_messages = 1;
+			}
+		}
 		ASN1_STRING_set_default_mask_asc((char*)CCHAR(string_opt));
+		mydb.first();
+		if (!mydb.find(setting, "mw_geometry")) {
+			db_header_t h;
+			if ((p = (char *)mydb.load(&h))) {
+				if (h.version == 1) {
+					QByteArray ba;
+					ba = QByteArray::fromRawData(p, h.len);
+					int w, h, i;
+					w = db::intFromData(ba);
+					h = db::intFromData(ba);
+					i = db::intFromData(ba);
+					resize(w,h);
+					if (i != -1)
+						tabView->setCurrentIndex(i);
+					}
+				free(p);
+			}
+		}
 	} catch (errorEx &err) {
 		Error(err);
-		return;
+		return ret;
 	}
 	setWindowTitle(tr(XCA_TITLE));
 	setItemEnabled(true);
+	if (pki_evp::passwd.isEmpty())
+		QMessageBox::information(this, XCA_TITLE,
+			tr("Using or exporting private keys will not be possible without providing the correct password"));
+
 	dbindex->setText(tr("Database") + ":" + dbfile);
 	load_engine();
+	return ret;
 }
 
 void MainWindow::dump_database()
@@ -189,14 +236,61 @@ void MainWindow::undelete()
 	if (dlgi->entries() > 0) {
 		dlgi->execute(1);
 	} else {
-		QMessageBox::information(this, XCA_TITLE, tr("No deleted items found"));
+		QMessageBox::information(this, XCA_TITLE,
+			tr("No deleted items found"));
 	}
 	delete dlgi;
 }
 
+int MainWindow::open_default_db()
+{
+	if (!dbfile.isEmpty())
+		return 0;
+	FILE *fp = fopen(QString2filename(getUserSettingsDir() +
+					QDir::separator() + "defaultdb"), "r");
+	if (!fp)
+		return 0;
+
+	char buff[256];
+	size_t len = fread(buff, 1, 255, fp);
+	fclose(fp);
+	buff[len] = 0;
+	dbfile = filename2QString(buff).trimmed();
+	if (QFile::exists(dbfile))
+		return init_database();
+	return 0;
+}
+
+void MainWindow::default_database()
+{
+	QFileInfo fi(dbfile);
+	QString dir = getUserSettingsDir();
+	FILE *fp;
+
+	QDir d;
+	d.mkpath(dir);
+
+	fp = fopen(QString2filename(dir +QDir::separator() +"defaultdb"), "w");
+	if (fp) {
+		QByteArray ba;
+		ba = filename2bytearray(fi.canonicalFilePath() + "\n");
+		fwrite(ba.constData(), ba.size(), 1, fp);
+		fclose(fp);
+	}
+
+}
 
 void MainWindow::close_database()
 {
+	QByteArray ba;
+	if (!dbfile.isEmpty()) {
+		ba += db::intToData(size().width());
+		ba += db::intToData(size().height());
+		ba += db::intToData(tabView->currentIndex());
+		db mydb(dbfile);
+		mydb.set((const unsigned char *)ba.constData(), ba.size(), 1,
+			setting, "mw_geometry");
+	}
 	setItemEnabled(false);
 	dbindex->clear();
 
@@ -222,12 +316,13 @@ void MainWindow::close_database()
 	temps = NULL;
 	keys = NULL;
 
-	pki_evp::erasePasswd();
+	pki_evp::passwd.cleanse();
 
 	if (!crls)
 		return;
 	crls = NULL;
 
+
 	try {
 		db mydb(dbfile);
 		mydb.shrink( DBFLAG_OUTDATED | DBFLAG_DELETED );
@@ -235,6 +330,8 @@ void MainWindow::close_database()
 	catch (errorEx &err) {
 		MainWindow::Error(err);
 	}
+	pkcs11::remove_libs();
+	enableTokenMenu(pkcs11::loaded());
 }
 
 /* Asymetric Key buttons */
