@@ -1,12 +1,13 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
 
 #include "MainWindow.h"
+#include "PwDialog.h"
 #include "Options.h"
 #include "lib/load_obj.h"
 #include "lib/pass_info.h"
@@ -16,19 +17,26 @@
 #include "lib/func.h"
 #include "ui_Options.h"
 #include "widgets/hashBox.h"
-#include <qapplication.h>
-#include <qmenubar.h>
-#include <qmessagebox.h>
+#include <QtGui/QApplication>
+#include <QtGui/QClipboard>
+#include <QtGui/QMenuBar>
+#include <QtGui/QMessageBox>
 
 void MainWindow::init_menu()
 {
-	QMenu *file, *help, *import;
+	QMenu *file, *help, *import, *token;
 
 	file = menuBar()->addMenu(tr("&File"));
-	file->addAction(tr("&New DataBase"),  this, SLOT(new_database()));
-	file->addAction(tr("&Open DataBase"),  this, SLOT(load_database()));
+	file->addAction(tr("&New DataBase"), this, SLOT(new_database()),
+		QKeySequence::New);
+	file->addAction(tr("&Open DataBase"), this, SLOT(load_database()),
+		QKeySequence::Open);
+	file->addAction(tr("Generate DH parameter"), this,
+				 SLOT(generateDHparam()));
+	acList += file->addAction(tr("Set as default DataBase"), this,
+				SLOT(default_database()));
 	acList += file->addAction(tr("&Close DataBase"), this,
-			SLOT(close_database()));
+		SLOT(close_database()), QKeySequence(QKeySequence::Close));
 	acList += file->addAction(tr("&Dump DataBase"), this,
 				SLOT(dump_database()));
 	acList += file->addAction(tr("C&hange DataBase password"), this,
@@ -57,18 +65,40 @@ void MainWindow::init_menu()
 				SLOT(on_BNimportTemp_clicked()) );
 	import->addAction(tr("Revocation list"), this,
 				SLOT(on_BNimportCrl_clicked()) );
-	scardMenuAction = import->addAction(tr("Read Smard card"), this,
-				SLOT(importScard()));
 	import->addAction(tr("PEM file"), this,
 				SLOT(loadPem()) );
 	import->addAction(tr("paste PEM file"), this,
 				SLOT(pastePem()) );
 
+	token = menuBar()->addMenu(tr("&Token"));
+	token->addAction(tr("&Manage Security token"), this,
+				SLOT(manageToken()));
+	token->addAction(tr("&Init Security token"),  this,
+				SLOT(initToken()));
+	token->addAction(tr("&Change PIN"), this,
+				SLOT(changePin()) );
+	token->addAction(tr("Change &SO PIN"), this,
+				SLOT(changeSoPin()) );
+	token->addAction(tr("Init PIN"), this,
+				SLOT(initPin()) );
+
 	help = menuBar()->addMenu(tr("&Help") );
-	help->addAction(tr("&Content"), this, SLOT(help()), Qt::Key_F1 );
+	help->addAction(tr("&Content"), this, SLOT(help()),
+			QKeySequence::HelpContents);
 	help->addAction(tr("&About"), this, SLOT(about()) );
 	help->addAction(tr("Donations"), this, SLOT(donations()) );
 	wdList += import;
+	scardList += token;
+}
+
+int MainWindow::changeDB(QString fname)
+{
+	if (fname.isEmpty())
+		return 1;
+	close_database();
+	homedir = fname.mid(0, fname.lastIndexOf(QDir::separator()));
+	dbfile = fname;
+	return init_database();
 }
 
 void MainWindow::new_database()
@@ -77,18 +107,11 @@ void MainWindow::new_database()
 	QString selectedFilter;
 	QString fname = QFileDialog::getSaveFileName(this, l.caption, homedir,
 			l.filter, &selectedFilter, QFileDialog::DontConfirmOverwrite);
-
-	if (fname.isEmpty())
-		return;
-
-	close_database();
-	homedir = fname.mid(0, fname.lastIndexOf(QDir::separator()) );
 	// make sure that, if the 3 letter extension was left selected
 	// in Qt's OS X file open dialog,
 	// the filename actually ends with that extension.
 	// Otherwise usability breaks in jarring ways.
-	dbfile = getFullFilename(fname, selectedFilter);
-	init_database();
+	changeDB(getFullFilename(fname, selectedFilter));
 }
 
 void MainWindow::load_database()
@@ -96,25 +119,18 @@ void MainWindow::load_database()
 	load_db l;
 	QString fname = QFileDialog::getOpenFileName(this, l.caption, homedir,
 			l.filter);
-
-	if (fname.isEmpty())
-		return;
-
-	close_database();
-	homedir = fname.mid(0, fname.lastIndexOf(QDir::separator()) );
-	dbfile = fname;
-	init_database();
+	changeDB(fname);
 }
 
 void MainWindow::import_dbdump()
 {
 	extern int read_dump(const char *, db_base **, char *, int);
+	Passwd pass;
 	char buf[50];
 
 	db_base *dbl[] = { keys, reqs, certs, temps, crls };
 	if (!keys)
 		return;
-	QString pass;
 	QString file = QFileDialog::getOpenFileName(this, tr(XCA_TITLE), homedir,
 			tr("Database dump ( *.dump );;All files ( * )"));
 
@@ -123,21 +139,20 @@ void MainWindow::import_dbdump()
 
 	pass_info p(tr("Import password"),
 		tr("Please enter the password of the old database"), this);
-	if (passRead(buf, 50, 0, &p) <0)
+	if (PwDialog::execute(&p, &pass) != 1)
 		return;
-	pass = buf;
 	try {
-		read_dump(CCHAR(file), dbl, buf, 50);
-		if (pki_evp::md5passwd(CCHAR(pass)) != buf) {
+		read_dump(CCHAR(file), dbl, buf, sizeof(buf));
+		if (pki_evp::md5passwd(pass) != buf) {
 			int ret = QMessageBox::warning(this, tr(XCA_TITLE),
 				tr("Password verification error. Ignore keys ?"),
 				tr("Import anyway"), tr("Cancel"));
 			if (ret)
 				return;
 		}
-		pki_evp::setOldPasswd(CCHAR(pass));
+		pki_evp::oldpasswd = pass;
 		read_dump(CCHAR(file), dbl, NULL, 0);
-		pki_evp::eraseOldPasswd();
+		pki_evp::oldpasswd.cleanse();
 	} catch (errorEx &err) {
 		Error(err);
 	}
@@ -148,10 +163,14 @@ void MainWindow::setOptions()
 	Options *opt = new Options(this);
 
 	opt->setStringOpt(string_opt);
-	opt->pkcs11path->setText(pkcs11path);
-	if (!opt->exec())
+	opt->setupPkcs11Provider(pkcs11path);
+	opt->suppress->setCheckState(
+		pki_base::suppress_messages ? Qt::Checked : Qt::Unchecked);
+	if (!opt->exec()) {
+		delete opt;
+		enableTokenMenu(pkcs11::loaded());
 		return;
-
+	}
 	QString alg = opt->hashAlgo->currentHashName();
 	db mydb(dbfile);
 	mydb.set((const unsigned char *)CCHAR(alg), alg.length()+1, 1,
@@ -162,27 +181,23 @@ void MainWindow::setOptions()
 	mydb.set((const unsigned char *)CCHAR(mandatory_dn),
 			mandatory_dn.length()+1, 1, setting, "mandatory_dn");
 
+	bool suppress = opt->suppress->checkState();
+	pki_base::suppress_messages = suppress;
+	mydb.set((const unsigned char *)(suppress ? "1" : "0"),
+			2, 1, setting, "suppress");
+
 	if (opt->getStringOpt() != string_opt) {
 		string_opt = opt->getStringOpt();
 		ASN1_STRING_set_default_mask_asc((char *)CCHAR(string_opt));
 		mydb.set((const unsigned char *)CCHAR(string_opt),
 				string_opt.length()+1, 1, setting,"string_opt");
 	}
-	QString newpath = opt->pkcs11path->text();
+	QString newpath = opt->getPkcs11Provider();
 	if (newpath != pkcs11path) {
 		pkcs11path = newpath;
 		mydb.set((const unsigned char *) CCHAR(pkcs11path),
 			pkcs11path.length()+1, 1,setting, "pkcs11path");
-		try {
-			pki_scard::init_p11engine(pkcs11path, true);
-		} catch (errorEx &err) {
-			Error(err);
-		}
-#ifdef WIN32
-		QMessageBox::information(this, XCA_TITLE,
-			QString("You need to restart XCA to load the new library\n"),
-			tr("Ok"));
-#endif
 	}
-	scardMenuAction->setEnabled(pkcs11::loaded());
+	enableTokenMenu(pkcs11::loaded());
+	delete opt;
 }
