@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -9,19 +9,22 @@
 //#define MDEBUG
 #include "MainWindow.h"
 #include "ImportMulti.h"
-#include <qapplication.h>
-#include <qclipboard.h>
-#include <qmessagebox.h>
-#include <qlabel.h>
-#include <qpushbutton.h>
-#include <qlistview.h>
-#include <qlineedit.h>
-#include <qtextbrowser.h>
-#include <qstatusbar.h>
-#include <qlist.h>
-#include <qtemporaryfile.h>
+#include "lib/Passwd.h"
+
 #include <openssl/rand.h>
 
+#include <QtGui/QApplication>
+#include <QtGui/QClipboard>
+#include <QtGui/QMessageBox>
+#include <QtGui/QLabel>
+#include <QtGui/QPushButton>
+#include <QtGui/QListView>
+#include <QtGui/QLineEdit>
+#include <QtGui/QTextBrowser>
+#include <QtGui/QStatusBar>
+#include <QtCore/QList>
+#include <QtGui/QInputDialog>
+
 #include "lib/exception.h"
 #include "lib/pki_evp.h"
 #include "lib/pki_scard.h"
@@ -31,15 +34,14 @@
 #include "lib/pass_info.h"
 #include "lib/func.h"
 #include "lib/pkcs11.h"
-#include "ui_PassRead.h"
-#include "ui_PassWrite.h"
 #include "ui_About.h"
-
+#include "PwDialog.h"
 
 QPixmap *MainWindow::keyImg = NULL, *MainWindow::csrImg = NULL,
 	*MainWindow::certImg = NULL, *MainWindow::tempImg = NULL,
 	*MainWindow::nsImg = NULL, *MainWindow::revImg = NULL,
-	*MainWindow::appIco = NULL, *MainWindow::scardImg = NULL;
+	*MainWindow::appIco = NULL, *MainWindow::scardImg = NULL,
+	*MainWindow::doneIco = NULL;
 
 db_key *MainWindow::keys = NULL;
 db_x509req *MainWindow::reqs = NULL;
@@ -97,6 +99,7 @@ static const int other_curve_nids[] = {
 
 static void init_curves()
 {
+#ifndef OPENSSL_NO_EC
 	pki_evp::num_curves = EC_get_builtin_curves(NULL, 0);
 	pki_evp::curves = (EC_builtin_curve*)OPENSSL_malloc(
 			(int)(sizeof(EC_builtin_curve) *pki_evp::num_curves));
@@ -125,16 +128,24 @@ static void init_curves()
 			}
 		}
 	}
+#endif
+}
+
+void MainWindow::enableTokenMenu(bool enable)
+{
+	foreach(QWidget *w, scardList) {
+		w->setEnabled(enable);
+	}
 }
 
 void MainWindow::load_engine()
 {
 	try {
-		pki_scard::init_p11engine(pkcs11path, pkcs11path.isEmpty());
+		pkcs11::load_libs(pkcs11path, false);
 	} catch (errorEx &err) {
 		Error(err);
 	}
-	scardMenuAction->setEnabled(pkcs11::loaded());
+	enableTokenMenu(pkcs11::loaded());
 }
 
 static QByteArray fileNameEncoderFunc(const QString &fileName)
@@ -158,8 +169,6 @@ MainWindow::MainWindow(QWidget *parent )
 	dbindex->setMargin(6);
 
 	statusBar()->addWidget(dbindex, 1);
-	mandatory_dn = "";
-	string_opt = "pkix";
 
 	setupUi(this);
 	setWindowTitle(tr(XCA_TITLE));
@@ -189,6 +198,31 @@ MainWindow::MainWindow(QWidget *parent )
 	eku_nid = read_nidlist("eku.txt");
 	dn_nid = read_nidlist("dn.txt");
 	aia_nid = read_nidlist("aia.txt");
+	setAcceptDrops(true);
+}
+
+void MainWindow::dropEvent(QDropEvent *event)
+{
+	QList<QUrl> urls = event->mimeData()->urls();
+	QUrl u;
+	ImportMulti *dlgi = new ImportMulti(this);
+	QStringList failed;
+
+	foreach(u, urls) {
+		QString s = u.toLocalFile();
+	        pki_multi *pki = probeAnything(s);
+		if (pki && !pki->count())
+			failed << s;
+	        dlgi->addItem(pki);
+	}
+	dlgi->execute(1, failed);
+	delete dlgi;
+}
+
+void MainWindow::dragEnterEvent(QDragEnterEvent *event)
+{
+	if (event->mimeData()->hasUrls())
+		event->acceptProposedAction();
 }
 
 void MainWindow::setItemEnabled(bool enable)
@@ -199,7 +233,7 @@ void MainWindow::setItemEnabled(bool enable)
 	foreach(QAction *a, acList) {
 		a->setEnabled(enable);
 	}
-	scardMenuAction->setEnabled(pkcs11::loaded());
+	enableTokenMenu(pkcs11::loaded());
 }
 
 /* creates a new nid list from the given filename */
@@ -210,7 +244,7 @@ NIDlist *MainWindow::read_nidlist(QString name)
 
 #ifndef WIN32
 	/* first try $HOME/xca/ */
-	nl = readNIDlist(getUserSettingsDir() + QDir::separator() + name);
+	nl = readNIDlist(getUserSettingsDir() + name);
 #if !defined(Q_WS_MAC)
 	if (nl.count() == 0){
 		/* next is /etx/xca/... */
@@ -235,6 +269,7 @@ void MainWindow::init_images()
 	revImg = loadImg("bigcrl.png");
 	scardImg = loadImg("bigscard.png");
 	appIco = loadImg("key.xpm");
+	doneIco = loadImg("done.png");
 	bigKey->setPixmap(*keyImg);
 	bigCsr->setPixmap(*csrImg);
 	bigCert->setPixmap(*certImg);
@@ -247,12 +282,13 @@ void MainWindow::init_images()
 	pki_x509req::icon[0] = loadImg("req.png");
 	pki_x509req::icon[1] = loadImg("reqkey.png");
 	pki_x509req::icon[2] = loadImg("spki.png");
-	pki_x509req::icon[3] = loadImg("done.png");
+	pki_x509req::icon[3] = doneIco;
 	pki_x509::icon[0] = loadImg("validcert.png");
 	pki_x509::icon[1] = loadImg("validcertkey.png");
 	pki_x509::icon[2] = loadImg("invalidcert.png");
 	pki_x509::icon[3] = loadImg("invalidcertkey.png");
 	pki_x509::icon[4] = loadImg("revoked.png");
+	pki_x509::icon[5] = doneIco;
 	pki_temp::icon = loadImg("template.png");
 	pki_crl::icon = loadImg("crl.png");
 }
@@ -261,30 +297,40 @@ void MainWindow::read_cmdline()
 {
 	int cnt = 1, opt = 0, force_load = 0;
 	char *arg = NULL;
-	pki_base *item = NULL;
-	load_base *lb = NULL;
 	exitApp = 0;
+	QStringList failed;
 	ImportMulti *dlgi = new ImportMulti(this);
 	while (cnt < qApp->argc()) {
 		arg = qApp->argv()[cnt];
 		if (arg[0] == '-') { // option
-			if (lb)
-				delete lb;
-			opt = 1; lb = NULL;
+			opt = 1;
 			switch (arg[1]) {
-				case 'c' : lb = new load_cert(); break;
-				case 'r' : lb = new load_req(); break;
-				case 'k' : lb = new load_key(); break;
-				case 'p' : lb = new load_pkcs12(); break;
-				case '7' : lb = new load_pkcs7(); break;
-				case 'l' : lb = new load_crl(); break;
-				case 't' : lb = new load_temp(); break;
-				case 'P' : lb = new load_pem(); break;
-				case 'd' : force_load=1; break;
-				case 'v' : cmd_version(); opt=0; break;
-				case 'x' : exitApp = 1; opt=0; break;
-				case 'h' : cmd_help(NULL); opt=0; break;
-				default  : cmd_help(CCHAR(tr("no such option: ") + arg));
+				case 'c':
+				case 'r':
+				case 'k':
+				case 'p':
+				case '7':
+				case 'l':
+				case 't':
+				case 'P':
+					break;
+				case 'd':
+					force_load=1;
+					break;
+				case 'v':
+					cmd_version();
+					opt=0;
+					break;
+				case 'x':
+					exitApp = 1;
+					opt=0;
+					break;
+				case 'h':
+					cmd_help(NULL);
+					opt=0;
+					break;
+				default:
+					 cmd_help(CCHAR(tr("no such option: %1").arg(arg)));
 			}
 			if (arg[2] != '\0' && opt==1) {
 				 arg+=2;
@@ -294,31 +340,24 @@ void MainWindow::read_cmdline()
 			}
 		}
 		QString file = filename2QString(arg);
-		if (lb) {
-			item = NULL;
-			try {
-				item = lb->loadItem(file);
-				dlgi->addItem(item);
-			}
-			catch (errorEx &err) {
-				if (item) {
-					delete item;
-					item = NULL;
-				}
-			}
+		if (force_load) {
+			if (changeDB(file) == 2)
+				exitApp = 1;
+			force_load = 0;
 		} else {
-			if (QFile::exists(file) || force_load) {
-				dbfile = file;
-				homedir = dbfile.left(dbfile.lastIndexOf(QDir::separator()));
-				init_database();
-			} else {
-				cmd_help(CCHAR(tr("Database file does not exist: ") + arg));
-			}
+			int ret;
+			pki_multi *pki = probeAnything(file, &ret);
+			if (!pki && ret == 2)
+				 exitApp = 1;
+			if (pki && !pki->count())
+				failed << file;
+			dlgi->addItem(pki);
 		}
-
 		cnt++;
 	}
-	dlgi->execute(1); /* force showing of import dialog */
+	dlgi->execute(1, failed); /* force showing of import dialog */
+	if (dlgi->result() == QDialog::Rejected)
+		exitApp = 1;
 	delete dlgi;
 }
 
@@ -329,9 +368,48 @@ void MainWindow::loadPem()
 		keys->load_default(l);
 }
 
+bool MainWindow::pastePem(QString text)
+{
+	bool success = false;
+	QByteArray pemdata = text.toAscii();
+	BIO *b = BIO_QBA_mem_buf(pemdata);
+	check_oom(b);
+	pki_multi *pem = NULL;
+	ImportMulti *dlgi = NULL;
+	try {
+		pem = new pki_multi();
+		dlgi = new ImportMulti(this);
+		pem->fromPEM_BIO(b, QString("paste"));
+		success = pem->count() != 0;
+		dlgi->addItem(pem);
+		pem = NULL;
+		dlgi->execute(1);
+	}
+	catch (errorEx &err) {
+		Error(err);
+	}
+	if (dlgi)
+		delete dlgi;
+	if (pem)
+		delete pem;
+	BIO_free(b);
+	return success;
+}
+
 void MainWindow::pastePem()
 {
 	Ui::About ui;
+	QClipboard *cb = QApplication::clipboard();
+	QString text;
+
+	text = cb->text(QClipboard::Selection);
+	if (text.isEmpty())
+		text = cb->text(QClipboard::Clipboard);
+
+	if (!text.isEmpty())
+		if (pastePem(text))
+			return;
+
 	QDialog *input = new QDialog(this, 0);
 
 	ui.setupUi(input);
@@ -339,89 +417,162 @@ void MainWindow::pastePem()
 	QTextEdit *textbox = new QTextEdit(input);
 	ui.vboxLayout->addWidget(textbox);
 	ui.button->setText(tr("Import PEM data"));
-	input->setWindowTitle(tr(XCA_TITLE));
-	if (input->exec()) {
-		QString txt = textbox->toPlainText();
-		QTemporaryFile f;
-		f.open();
-		f.write(textbox->toPlainText().toAscii());
-		f.flush();
-		pki_multi *pem = NULL;
-		ImportMulti *dlgi = NULL;
-		try {
-			pem = new pki_multi();
-			dlgi = new ImportMulti(this);
-			pem->fload(f.fileName());
-			dlgi->addItem(pem);
-			dlgi->execute(1);
-		}
-		catch (errorEx &err) {
-			Error(err);
-		}
-		delete dlgi;
-	}
+	input->setWindowTitle(XCA_TITLE);
+	textbox->setPlainText(text);
+	if (input->exec())
+		text = textbox->toPlainText();
 	delete input;
+
+	if (!text.isEmpty())
+		pastePem(text);
 }
 
-void MainWindow::importScard()
+void MainWindow::initToken()
 {
-	pkcs11 p11;
-	QList<unsigned long> p11_slots;
-	int i;
-	pki_scard *card = NULL;
-	pki_x509 *cert = NULL;
-
+	bool ok;
 	if (!pkcs11::loaded())
 		return;
 	try {
-		ImportMulti *dlgi = new ImportMulti(this);
-		QList<CK_OBJECT_HANDLE> objects;
-		p11_slots = p11.getSlotList();
+		pkcs11 p11;
+		slotid slot;
+		Passwd pin;
+		int ret;
 
-		if (p11_slots.count() == 0)
-			QMessageBox::warning(this, XCA_TITLE,
-				tr("No Smart card found"));
-		for (i=0; i<p11_slots.count(); i++) {
-			p11.startSession(p11_slots[i]);
-			QList<CK_MECHANISM_TYPE> ml = p11.mechanismList(i);
-			if (ml.count() == 0)
-				ml << CKM_SHA1_RSA_PKCS;
-			pk11_attlist atts(pk11_attr_ulong(CKA_CLASS,
-					CKO_PUBLIC_KEY));
+		if (!p11.selectToken(&slot, this))
+			return;
 
-			objects = p11.objectList(atts);
+		tkInfo ti = p11.tokenInfo(slot);
+		QString slotname = QString("%1 (#%2)").
+			arg(ti.label()).arg(ti.serial());
 
-			for (int j=0; j< objects.count(); j++) {
-				card = new pki_scard("");
-				try {
-					card->load_token(p11, objects[j]);
-					card->setMech_list(ml);
-					dlgi->addItem(card);
-				} catch (errorEx &err) {
-					Error(err);
-					delete card;
-				}
-				card = NULL;
+		pass_info p(XCA_TITLE,
+			tr("Please enter the original SO PIN (PUK) of the token '%1'").
+			arg(slotname) + "\n" + ti.pinInfo());
+		p.setPin();
+		if (ti.tokenInitialized()) {
+			ret = PwDialog::execute(&p, &pin, false);
+		} else {
+			p.setDescription(tr("Please enter the new SO PIN (PUK) of the token '%1'").
+			arg(slotname) + "\n" + ti.pinInfo());
+			ret = PwDialog::execute(&p, &pin, true);
+		}
+		if (ret != 1)
+			return;
+		QString label = QInputDialog::getText(this, XCA_TITLE,
+			tr("The new label of the token '%1'").
+			arg(slotname), QLineEdit::Normal, QString(), &ok);
+		if (!ok)
+			return;
+		p11.initToken(slot, pin.constUchar(), pin.size(), label);
+	} catch (errorEx &err) {
+		Error(err);
+        }
+}
+
+void MainWindow::changePin(bool so)
+{
+	if (!pkcs11::loaded())
+		return;
+	try {
+		pkcs11 p11;
+		slotid slot;
+
+		if (!p11.selectToken(&slot, this))
+			return;
+		p11.changePin(slot, so);
+	} catch (errorEx &err) {
+		Error(err);
+        }
+}
+
+void MainWindow::changeSoPin()
+{
+	changePin(true);
+}
+
+void MainWindow::initPin()
+{
+	if (!pkcs11::loaded())
+		return;
+	try {
+		pkcs11 p11;
+		slotid slot;
+
+		if (!p11.selectToken(&slot, this))
+			return;
+		p11.initPin(slot);
+	} catch (errorEx &err) {
+		Error(err);
+        }
+}
+
+
+void MainWindow::manageToken()
+{
+	pkcs11 p11;
+	slotid slot;
+	pki_scard *card = NULL;
+	pki_x509 *cert = NULL;
+	ImportMulti *dlgi = NULL;
+
+	if (!pkcs11::loaded())
+		return;
+
+	try {
+		if (!p11.selectToken(&slot, this))
+			return;
+
+		ImportMulti *dlgi = new ImportMulti(this);
+
+		dlgi->tokenInfo(slot);
+		QList<CK_OBJECT_HANDLE> objects;
+
+		QList<CK_MECHANISM_TYPE> ml = p11.mechanismList(slot);
+		if (ml.count() == 0)
+			ml << CKM_SHA1_RSA_PKCS;
+		pk11_attlist atts(pk11_attr_ulong(CKA_CLASS,
+				CKO_PUBLIC_KEY));
+
+		p11.startSession(slot);
+		objects = p11.objectList(atts);
+
+		for (int j=0; j< objects.count(); j++) {
+			card = new pki_scard("");
+			try {
+				card->load_token(p11, objects[j]);
+				card->setMech_list(ml);
+				dlgi->addItem(card);
+			} catch (errorEx &err) {
+				Error(err);
+				delete card;
 			}
-			atts.reset();
-			atts << pk11_attr_ulong(CKA_CLASS, CKO_CERTIFICATE) <<
-				pk11_attr_ulong(CKA_CERTIFICATE_TYPE,CKC_X_509);
-			objects = p11.objectList(atts);
-
-			for (int j=0; j< objects.count(); j++) {
-				cert = new pki_x509("");
-				try {
-					cert->load_token(p11, objects[j]);
-					cert->setTrust(2);
-					dlgi->addItem(cert);
-				} catch (errorEx &err) {
-					Error(err);
-					delete cert;
-				}
-				cert = NULL;
+			card = NULL;
+		}
+		atts.reset();
+		atts << pk11_attr_ulong(CKA_CLASS, CKO_CERTIFICATE) <<
+			pk11_attr_ulong(CKA_CERTIFICATE_TYPE,CKC_X_509);
+		objects = p11.objectList(atts);
+
+		for (int j=0; j< objects.count(); j++) {
+			cert = new pki_x509("");
+			try {
+				cert->load_token(p11, objects[j]);
+				cert->setTrust(2);
+				dlgi->addItem(cert);
+			} catch (errorEx &err) {
+				Error(err);
+				delete cert;
 			}
+			cert = NULL;
+		}
+		if (dlgi->entries() == 0) {
+			tkInfo ti = p11.tokenInfo();
+			QMessageBox::information(this, XCA_TITLE,
+				tr("The token '%1' did not contain any keys or certificates").
+                                arg(ti.label()));
+		} else {
+			dlgi->execute(true);
 		}
-		dlgi->execute();
 	} catch (errorEx &err) {
 		Error(err);
         }
@@ -429,6 +580,8 @@ void MainWindow::importScard()
 		delete card;
 	if (cert)
 		delete cert;
+	if (dlgi)
+		delete dlgi;
 }
 
 MainWindow::~MainWindow()
@@ -463,18 +616,14 @@ QString makeSalt(void)
 void MainWindow::changeDbPass()
 {
 
-	char pass[MAX_PASS_LENGTH];
-	int keylen;
+	Passwd pass;
 
 	pass_info p(tr("New Password"), tr("Please enter the new password "
 			"to encrypt your private keys in the database-file"),
 			this);
 
-	keylen = passWrite(pass, MAX_PASS_LENGTH-1, 0, &p);
-	if (keylen < 0)
+	if (PwDialog::execute(&p, &pass, true) != 1)
 		return;
-	memset(pass +keylen, 0, MAX_PASS_LENGTH -keylen);
-
 	QString tempn = dbfile + "{new}";
 	try {
 		if (!QFile::copy(dbfile, tempn))
@@ -487,7 +636,7 @@ void MainWindow::changeDbPass()
 		mydb.mv(new_file);
 		close_database();
 		pki_evp::passHash = passhash;
-		memcpy(pki_evp::passwd, pass, MAX_PASS_LENGTH);
+		pki_evp::passwd = pass;
 		init_database();
 	} catch (errorEx &ex) {
 		QFile::remove(tempn);
@@ -495,7 +644,7 @@ void MainWindow::changeDbPass()
 	}
 }
 
-QString MainWindow::updateDbPassword(QString newdb, char *pass)
+QString MainWindow::updateDbPassword(QString newdb, Passwd pass)
 {
 	db mydb(newdb);
 
@@ -545,7 +694,7 @@ QString MainWindow::updateDbPassword(QString newdb, char *pass)
 		{
 			EVP_PKEY *evp = key->decryptKey();
 			key->set_evp_key(evp);
-			key->encryptKey(pass);
+			key->encryptKey(pass.constData());
 			klist << key;
 		} else if (key)
 			delete key;
@@ -554,14 +703,10 @@ next:
 			break;
 	}
 	for (int i=0; i< klist.count(); i++) {
-		int size;
-		unsigned char *p;
 		pki_evp *key = klist[i];
-		p = key->toData(&size);
-		check_oom(p);
-		mydb.set(p, size, key->getVersion(),
-			 key->getType(), key->getIntName());
-		OPENSSL_free(p);
+		QByteArray ba = key->toData();
+		mydb.set((const unsigned char*)ba.constData(), ba.count(),
+			key->getVersion(), key->getType(), key->getIntName());
 		delete key;
 	}
 	return passhash;
@@ -573,10 +718,13 @@ int MainWindow::initPass()
 	char *pass;
 	pki_evp::passHash = QString();
 	QString salt;
+	int ret;
 
 	pass_info p(tr("New Password"), tr("Please enter a password, "
 			"that will be used to encrypt your private keys "
-			"in the database-file"), this);
+			"in the database file:\n%1").
+			arg(compressFilename(dbfile)), this);
+
 	if (!mydb.find(setting, "pwhash")) {
 		if ((pass = (char *)mydb.load(NULL))) {
 			pki_evp::passHash = pass;
@@ -584,28 +732,27 @@ int MainWindow::initPass()
 		}
 	}
 	if (pki_evp::passHash.isEmpty()) {
-		int keylen = passWrite((char *)pki_evp::passwd,
-				MAX_PASS_LENGTH-1, 0, &p);
-		if (keylen < 0)
-			return 0;
-		pki_evp::passwd[keylen]='\0';
+		ret = PwDialog::execute(&p, &pki_evp::passwd, true, true);
+		if (ret != 1)
+			return ret;
 		salt = makeSalt();
 		pki_evp::passHash = pki_evp::sha512passwd(pki_evp::passwd,salt);
 		mydb.set((const unsigned char *)CCHAR(pki_evp::passHash),
 			pki_evp::passHash.length()+1, 1, setting, "pwhash");
 	} else {
-		int keylen=0;
+		ret = 0;
 		while (pki_evp::sha512passwd(pki_evp::passwd, pki_evp::passHash)
 				!= pki_evp::passHash)
 		{
-			if (keylen !=0) QMessageBox::warning(this,tr(XCA_TITLE),
+			if (ret)
+				QMessageBox::warning(this, XCA_TITLE,
 				tr("Password verify error, please try again"));
 			p.setTitle(tr("Password"));
-			p.setDescription(tr("Please enter the password for unlocking the database"));
-			keylen = passRead(pki_evp::passwd, MAX_PASS_LENGTH-1, 0, &p);
-			if (keylen < 0)
-				return 1;
-			pki_evp::passwd[keylen]='\0';
+			p.setDescription(tr("Please enter the password for unlocking the database:\n%1").arg(compressFilename(dbfile)));
+			ret = PwDialog::execute(&p, &pki_evp::passwd,
+						false, true);
+			if (ret != 1)
+				return ret;
 			if (pki_evp::passHash.left(1) == "S")
 				continue;
 			/* Start automatic update from md5 to salted sha512
@@ -627,115 +774,20 @@ int MainWindow::initPass()
 	return 1;
 }
 
-static int hex2bin(QString &x, char *buf, int buflen)
-{
-	int len = x.length();
-	bool ok = false;
-	if (len % 2)
-		return -1;
-	len /= 2;
-	if (len > buflen)
-		return -1;
-
-	for (int i=0; i<len; i++) {
-		buf[i] = x.mid(i*2, 2).toInt(&ok, 16);
-		if (!ok)
-			return -1;
-	}
-	return len;
-}
-
-static const QString hexwarn = MainWindow::tr("Hex password must only contain the characters '0' - '9' and 'a' - 'f' and it must consist of an even number of characters");
-// Static Password Callback functions
-int MainWindow::passRead(char *buf, int size, int, void *userdata)
-{
-	int ret = -1;
-	pass_info *p = (pass_info *)userdata;
-	Ui::PassRead ui;
-	QDialog *dlg = new QDialog(p->getWidget());
-	ui.setupUi(dlg);
-	if (p != NULL) {
-		ui.image->setPixmap(p->getImage());
-		ui.description->setText(p->getDescription());
-		ui.title->setText(p->getType());
-		ui.label->setText(p->getType());
-		dlg->setWindowTitle(p->getTitle());
-		if (p->getType() != "PIN")
-			ui.takeHex->hide();
-	}
-
-	while (dlg->exec()) {
-		QString x = ui.pass->text();
-		if (ui.takeHex->isChecked()) {
-			ret = hex2bin(x, buf, size);
-			if (ret != -1)
-				break;
-		} else {
-			strncpy(buf, x.toAscii(), size);
-			ret = x.length();
-			break;
-		}
-		QMessageBox::warning(p->getWidget(), XCA_TITLE, hexwarn);
-	}
-	delete dlg;
-	return ret;
-}
-
-int MainWindow::passWrite(char *buf, int size, int, void *userdata)
-{
-	int ret = -1;
-	pass_info *p = (pass_info *)userdata;
-	Ui::PassWrite ui;
-	QDialog *dlg = new QDialog(p->getWidget());
-	ui.setupUi(dlg);
-	if (p != NULL) {
-		ui.image->setPixmap(p->getImage()) ;
-		ui.description->setText(p->getDescription());
-		ui.title->setText(p->getType());
-		ui.label->setText(p->getType());
-		ui.repeatLabel->setText(tr("Repeat ") + p->getType());
-		dlg->setWindowTitle(p->getTitle());
-		if (p->getType() != "PIN")
-			ui.takeHex->hide();
-	}
-	dlg->show();
-	dlg->activateWindow();
-	ui.passA->setFocus();
-
-	while (dlg->exec()) {
-		QString A = ui.passA->text();
-		QString B = ui.passB->text();
-		if (A == B) {
-			if (ui.takeHex->isChecked()) {
-				ret = hex2bin(A, buf, size);
-				if (ret != -1)
-					break;
-			} else {
-				strncpy(buf, A.toAscii(), size);
-				ret = A.length();
-				break;
-			}
-			QMessageBox::warning(p->getWidget(), XCA_TITLE,
-						hexwarn);
-		} else {
-			QMessageBox::warning(p->getWidget(), XCA_TITLE,
-						p->getType() + tr(" missmatch"));
-		}
-	}
-	delete dlg;
-	return ret;
-}
-
 void MainWindow::Error(errorEx &err)
 {
 	if (err.isEmpty())
 		 return;
 	QString msg =  tr("The following error occured:") + "\n" + err.getString();
-	int ret = QMessageBox::warning(qApp->activeWindow(), XCA_TITLE,
-			msg, tr("&OK"), tr("Copy to Clipboard"));
-	if (ret == 1) {
+	QMessageBox box(QMessageBox::Warning, XCA_TITLE,
+		msg, QMessageBox::Ok, NULL);
+	box.addButton(QMessageBox::Apply)->setText(tr("Copy to Clipboard"));
+	if (box.exec() == QMessageBox::Apply) {
 		QClipboard *cb = QApplication::clipboard();
-		cb->setText(msg);
+		if (cb->supportsSelection())
+			cb->setText(msg, QClipboard::Selection);
+		else
+			cb->setText(msg);
 	}
 }
 
@@ -758,3 +810,85 @@ void MainWindow::connNewX509(NewX509 *nx)
 	connect( nx, SIGNAL(showReq(QString)), reqs, SLOT(showItem(QString)));
 }
 
+void MainWindow::importAnything(QString file)
+{
+	ImportMulti *dlgi = new ImportMulti(this);
+	QStringList failed;
+	pki_multi *pki = probeAnything(file);
+	if (pki && !pki->count())
+		failed << file;
+	dlgi->addItem(pki);
+	dlgi->execute(1, failed);
+	delete dlgi;
+}
+
+pki_multi *MainWindow::probeAnything(QString file, int *ret)
+{
+	pki_multi *pki = new pki_multi();
+
+	try {
+		if (file.endsWith(".xdb")) {
+			try {
+				int r;
+				db mydb(file);
+				mydb.verify_magic();
+				r = changeDB(file);
+				delete pki;
+				if (ret)
+					*ret = r;
+				return NULL;
+			} catch (errorEx &err) {
+			}
+		}
+		pki->probeAnything(file);
+	} catch (errorEx &err) {
+		Error(err);
+	}
+	return pki;
+}
+
+void MainWindow::generateDHparam()
+{
+	DH *dh = NULL;
+	FILE *fp = NULL;
+	QProgressBar *bar = NULL;
+	bool ok;
+	int num = QInputDialog::getDouble(this, XCA_TITLE, tr("Diffie-Hellman parameters are needed for different applications, but not handled by XCA.\nPlease enter the DH parameter bits"),
+		1024, 1024, 4096, 0, &ok);
+
+	if (!ok)
+		return;
+	try {
+		QStatusBar *status = statusBar();
+		bar = new QProgressBar();
+		check_oom(bar);
+		bar->setMinimum(0);
+		bar->setMaximum(100);
+		status->addPermanentWidget(bar, 1);
+		dh = DH_generate_parameters(num, 2, inc_progress_bar, bar);
+		status->removeWidget(bar);
+		openssl_error();
+
+		QString fname = QString("%1/dh%2.pem").arg(homedir).arg(num);
+		fname = QFileDialog::getSaveFileName(this, QString(),
+			fname, "All files ( * )", NULL);
+		if (fname == "")
+			throw errorEx("");
+		fp = fopen(QString2filename(fname), "w");
+		if (fp == NULL) {
+			throw errorEx(tr("Error opening file: '%1': %2").
+				arg(fname).arg(strerror(errno)));
+		}
+		PEM_write_DHparams(fp, dh);
+		openssl_error();
+	} catch (errorEx &err) {
+		Error(err);
+	}
+	if (dh)
+		DH_free(dh);
+	if (fp)
+		fclose(fp);
+	if (bar)
+		delete bar;
+}
+
