@@ -1,12 +1,12 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
-#ifndef _MAINWINDOW_H
-#define _MAINWINDOW_H
+#ifndef __MAINWINDOW_H
+#define __MAINWINDOW_H
 
 #include "NewX509.h"
 #include "ui_MainWindow.h"
@@ -17,14 +17,16 @@
 #include "lib/db_crl.h"
 #include "lib/exception.h"
 #include "lib/oid.h"
-#include <qpixmap.h>
-#include <qfiledialog.h>
-#include <qmenubar.h>
-#include <qlist.h>
+#include "lib/Passwd.h"
+#include <QtGui/QPixmap>
+#include <QtGui/QFileDialog>
+#include <QtGui/QMenuBar>
+#include <QtCore/QList>
 
 #define DBFILE "xca.xdb"
 
 class db_x509;
+class pki_multi;
 
 class MainWindow: public QMainWindow, public Ui::MainWindow
 {
@@ -35,8 +37,8 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		QString string_opt;
 		QString pkcs11path;
 		QList<QWidget*> wdList;
+		QList<QWidget*> scardList;
 		QList<QAction*> acList;
-		QAction *scardMenuAction;
 
 	protected:
 		void init_images();
@@ -45,6 +47,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		NIDlist *read_nidlist(QString name);
 		QLabel *statusLabel;
 		QString homedir;
+		int changeDB(QString fname);
 
 	public:
 		static db_x509 *certs;
@@ -53,7 +56,8 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		static db_temp *temps;
 		static db_crl *crls;
 		static QPixmap *keyImg, *csrImg, *certImg, *tempImg,
-				*nsImg, *revImg, *appIco, *scardImg;
+				*nsImg, *revImg, *appIco, *scardImg,
+				*doneIco;
 		static NIDlist *eku_nid, *dn_nid, *aia_nid;
 		static QString mandatory_dn;
 		int exitApp;
@@ -67,9 +71,6 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		int initPass();
 		void read_cmdline();
 		void load_engine();
-		static int passRead(char *buf, int size, int rwflag, void *userdata);
-		static int passWrite(char *buf, int size, int rwflag, void *userdata);
-		//static void Qt::SocketError(errorEx &err);
 		static void Error(errorEx &err);
 		void cmd_version();
 		void cmd_help(const char* msg);
@@ -78,14 +79,21 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void setPath(QString path);
 		bool mkDir(QString dir);
 		void setItemEnabled(bool enable);
-		QString updateDbPassword(QString newdb, char *pass);
+		QString updateDbPassword(QString newdb, Passwd pass);
+		void enableTokenMenu(bool enable);
+		pki_multi *probeAnything(QString file, int *ret = NULL);
+		void importAnything(QString file);
+		void dropEvent(QDropEvent *event);
+		void dragEnterEvent(QDragEnterEvent *event);
+		int open_default_db();
 
 	public slots:
-		void init_database();
+		int init_database();
 		void new_database();
 		void load_database();
 		void close_database();
 		void dump_database();
+		void default_database();
 		void connNewX509(NewX509 *nx);
 		void about();
 		void donations();
@@ -93,12 +101,19 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void import_dbdump();
 		void undelete();
 		void loadPem();
+		bool pastePem(QString text);
 		void pastePem();
 		void changeDbPass();
 
 	private slots:
 		void setOptions();
-		void importScard();
+		void manageToken();
+		void initToken();
+		void changePin(bool so=false);
+		void changeSoPin();
+		void initPin();
+		void generateDHparam();
+
 		void on_keyView_doubleClicked(const QModelIndex &m);
 		void on_reqView_doubleClicked(const QModelIndex &m);
 		void on_certView_doubleClicked(const QModelIndex &m);
