@@ -1,21 +1,21 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
 
 #include "NewX509.h"
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qradiobutton.h>
-#include <qmessagebox.h>
-#include <qlineedit.h>
-#include <qlabel.h>
-#include <qpixmap.h>
-#include <qpushbutton.h>
-#include <qvalidator.h>
+#include <QtGui/QCheckBox>
+#include <QtGui/QComboBox>
+#include <QtGui/QRadioButton>
+#include <QtGui/QMessageBox>
+#include <QtGui/QLineEdit>
+#include <QtGui/QLabel>
+#include <QtGui/QPixmap>
+#include <QtGui/QPushButton>
+#include <QtGui/QValidator>
 #include "MainWindow.h"
 #include "v3ext.h"
 #include "lib/x509name.h"
@@ -45,21 +45,29 @@ NewX509::NewX509(QWidget *parent)
 	aia_nid = *MainWindow::aia_nid;
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
 
-	QStringList sl;
+	QStringList keys;
 
 	setupUi(this);
 
 	/* temporary storage for creating temporary X509V3_CTX */
 	ctx_cert = NULL;
 
-	sl << "Type" << "Content";
-	extDNlist->setColumnCount(2);
-	extDNlist->setHorizontalHeaderLabels(sl);
-	setWindowTitle(tr(XCA_TITLE));
+	for (i=0; i < dn_nid.count(); i++)
+		keys << QString(OBJ_nid2ln(dn_nid[i]));
+
+	extDNlist->setKeys(keys);
+	extDNlist->setInfoLabel(extDNinfo);
+	connect(extDNlist->itemDelegateForColumn(1),
+		SIGNAL(setupLineEdit(const QString &, QLineEdit *)),
+		this, SLOT(setupExtDNwidget(const QString &, QLineEdit *)));
+	setWindowTitle(XCA_TITLE);
+
+	for (i=0; i<tabWidget->count(); i++) {
+		tabnames << tabWidget->tabText(i);
+	}
 
 	nsImg->setPixmap(*MainWindow::nsImg);
-	//setFont( tFont );
-	serialNr->setValidator( new QRegExpValidator(QRegExp("[0-9a-fA-F]*"), this));
+	serialNr->setValidator(new QRegExpValidator(QRegExp("[0-9a-fA-F]*"), this));
 	QStringList strings;
 
 	// are there any useable private keys  ?
@@ -93,7 +101,6 @@ NewX509::NewX509(QWidget *parent)
 	l->insertWidget(1, selfQASignRB);
 #endif
 	// set dates to now and now + 1 year
-	a1time a;
 	validNumber->setText("1");
 	validRange->setCurrentIndex(2);
 	on_applyTime_clicked();
@@ -107,10 +114,6 @@ NewX509::NewX509(QWidget *parent)
 	for (i=0; i < eku_nid.count(); i++)
 		ekeyUsage->addItem(OBJ_nid2ln(eku_nid[i]));
 
-	// setup Distinguished Name
-	for (i=0; i < dn_nid.count(); i++)
-		extDNobj->addItem(OBJ_nid2ln(dn_nid[i]));
-
 	// setup Authority Info Access
 	for (i=0; i < aia_nid.count(); i++)
 		aiaOid->addItem(OBJ_nid2ln(aia_nid[i]));
@@ -119,15 +122,19 @@ NewX509::NewX509(QWidget *parent)
 	X509V3_set_ctx(&ext_ctx, NULL , NULL, NULL, NULL, 0);
 	X509V3_set_ctx_nodb(&ext_ctx);
 
-	// setup the list of x509nameEntrys
-	name_ptr[0] = countryName;
-	name_ptr[1] = stateOrProvinceName;
-	name_ptr[2] = localityName;
-	name_ptr[3] = organisationName;
-	name_ptr[4] = organisationalUnitName;
-	name_ptr[5] = commonName;
-	name_ptr[6] = emailAddress;
+	QList<QLabel *> nameLabel;
+	nameLabel << LcountryName << LstateOrProvinceName << LlocalityName <<
+	LorganisationName << LorganisationalUnitName << LcommonName <<
+	LemailAddress;
 
+	for(int i=0; i<nameLabel.count(); i++) {
+		nameLabel[i]->setText(OBJ_nid2ln(name_nid[i]));
+		QString tt = nameLabel[i]->toolTip();
+		nameLabel[i]->setToolTip(QString("[%1] %2").
+			arg(OBJ_nid2sn(name_nid[i])).arg(tt));
+		name_ptr[i] = (QLineEdit *)nameLabel[i]->buddy();
+		setupLineEditByNid(name_nid[i], name_ptr[i]);
+	}
 	// Setup Request Attributes
 	if (attrWidget->layout())
 		delete attrWidget->layout();
@@ -147,6 +154,7 @@ NewX509::NewX509(QWidget *parent)
 		attr_edit << edit;
 		attrLayout->addWidget(label, i, 0);
 		attrLayout->addWidget(edit, i, 1);
+		setupLineEditByNid(nid, edit);
 	}
 	// last polish
 	on_certList_currentIndexChanged(0);
@@ -154,6 +162,7 @@ NewX509::NewX509(QWidget *parent)
 	tabWidget->setCurrentIndex(0);
 	attrWidget->hide();
 	pt = none;
+	notAfter->setEndDate(true);
 }
 
 void NewX509::setRequest()
@@ -175,6 +184,36 @@ NewX509::~NewX509()
 		delete(ctx_cert);
 }
 
+void NewX509::setupExtDNwidget(const QString &s, QLineEdit *l)
+{
+	setupLineEditByNid(OBJ_txt2nid(CCHAR(s)), l);
+}
+
+void NewX509::setupLineEditByNid(int nid, QLineEdit *l)
+{
+	ASN1_STRING_TABLE *tab = ASN1_STRING_TABLE_get(nid);
+	QValidator *validator = NULL;
+	QStringList info;
+
+	info << QString("[%1]").arg(OBJ_nid2sn(nid));
+
+	if (tab) {
+		if (tab->minsize > 1)
+			info << tr("minimum size: %1").arg(tab->minsize);
+		if (tab->maxsize != -1)
+			info << tr("maximum size: %1").arg(tab->maxsize);
+		if (tab->mask == B_ASN1_PRINTABLESTRING) {
+			info << tr("only a-z A-Z 0-9 '()+,-./:=?");
+			QRegExp rx("[a-zA-Z0-9'()+,-./:=?]+");
+			validator = new QRegExpValidator(rx, this);
+		} else if (tab->mask == B_ASN1_IA5STRING) {
+			info << tr("only 7-bit clean characters");
+		}
+	}
+	l->setToolTip(info.join(" "));
+	l->setValidator(validator);
+}
+
 void NewX509::addReqAttributes(pki_x509req *req)
 {
 	for (int i=0; i < attr_nid.count(); i++) {
@@ -214,7 +253,6 @@ void NewX509::defineTemplate(pki_temp *temp)
 {
 	fromTemplate(temp);
 	templateChanged(temp);
-	tempList->setEnabled(false);
 }
 
 void NewX509::defineRequest(pki_x509req *req)
@@ -297,9 +335,16 @@ static QString lb2QString(QListWidget *lb)
 	return sl.join(", ");
 }
 
-void NewX509::fromTemplate(pki_temp *temp)
+void NewX509::subjectFromTemplate(pki_temp *temp)
 {
-	setX509name(temp->xname);
+	if (temp)
+		setX509name(temp->xname);
+}
+
+void NewX509::extensionsFromTemplate(pki_temp *temp)
+{
+	if (!temp)
+		return;
 	subAltName->setText(temp->subAltName);
 	issAltName->setText(temp->issAltName);
 	crlDist->setText(temp->crlDist);
@@ -326,10 +371,15 @@ void NewX509::fromTemplate(pki_temp *temp)
 	basicPath->setText(temp->pathLen);
 	nconf_data->document()->setPlainText(temp->adv_ext);
 	noWellDefinedExpDate->setChecked(temp->noWellDefined);
-	notBefore->setNow();
 	on_applyTime_clicked();
 }
 
+void NewX509::fromTemplate(pki_temp *temp)
+{
+	subjectFromTemplate(temp);
+	extensionsFromTemplate(temp);
+}
+
 void NewX509::toTemplate(pki_temp *temp)
 {
 	temp->setIntName(description->text());
@@ -356,7 +406,9 @@ void NewX509::toTemplate(pki_temp *temp)
 	temp->eKeyUse = lb2QString(ekeyUsage);
 	temp->validN = validNumber->text().toInt();
 	temp->validM = validRange->currentIndex();
-	temp->pathLen = QString::number(basicPath->text().toInt());
+	temp->pathLen = basicPath->text();
+	if (!temp->pathLen.isEmpty())
+		temp->pathLen = QString::number(temp->pathLen.toInt());
 	temp->validMidn = midnightCB->isChecked();
 	if (nconf_data->isReadOnly()) {
 		temp->adv_ext = v3ext_backup;
@@ -489,36 +541,41 @@ void NewX509::templateChanged(pki_temp *templ)
 	templateChanged(tempname);
 }
 
-void NewX509::on_applyTemplate_clicked()
+pki_temp *NewX509::currentTemplate()
 {
-	pki_temp *temp = NULL;
 	if (!tempList->isEnabled())
-		return;
+		return NULL;
 	QString name = tempList->currentText();
 	if (name.isEmpty())
-		return;
-	temp = (pki_temp *)MainWindow::temps->getByName(name);
-	if (!temp)
-		return;
-	fromTemplate(temp);
+		return NULL;
+	return (pki_temp *)MainWindow::temps->getByName(name);
 }
 
-void NewX509::on_foreignSignRB_toggled(bool checked)
+void NewX509::on_applyTemplate_clicked()
+{
+	fromTemplate(currentTemplate());
+}
+
+void NewX509::on_applySubject_clicked()
+{
+	subjectFromTemplate(currentTemplate());
+}
+
+void NewX509::on_applyExtensions_clicked()
+{
+	extensionsFromTemplate(currentTemplate());
+}
+
+void NewX509::on_foreignSignRB_toggled(bool)
 {
 	switchHashAlgo();
-	certList->setEnabled(checked);
-}
-
-void NewX509::on_selfSignRB_toggled(bool checked)
-{
-	serialNr->setEnabled(checked);
 }
 
 void NewX509::newKeyDone(QString name)
 {
 	QStringList keys;
-	private_keys = MainWindow::keys->get0PrivateDesc(true);
-	private_keys0 = MainWindow::keys->get0PrivateDesc(false);
+	private_keys = MainWindow::keys->get0KeyDesc(true);
+	private_keys0 = MainWindow::keys->get0KeyDesc(false);
 	keyList->clear();
 	if (usedKeysToo->isChecked())
 		keys = private_keys;
@@ -531,11 +588,6 @@ void NewX509::newKeyDone(QString name)
 	keyList->setCurrentIndex(keys.indexOf(name));
 }
 
-void NewX509::on_noWellDefinedExpDate_toggled(bool)
-{
-	notAfter->setEnabled(!noWellDefinedExpDate->isChecked());
-}
-
 void NewX509::on_usedKeysToo_toggled(bool)
 {
 	QString cur = keyList->currentText();
@@ -578,8 +630,9 @@ x509name NewX509::getX509name(int _throw)
 		}
 		row = extDNlist->rowCount();
 		for (j=0; j<row; j++) {
-			nid = OBJ_ln2nid(CCHAR(extDNlist->item(j,0)->text()));
-			x.addEntryByNid(nid, extDNlist->item(j,1)->text());
+			QStringList l = extDNlist->getRow(j);
+			nid = OBJ_txt2nid(CCHAR(l[0]));
+			x.addEntryByNid(nid, l[1]);
 		}
 	} catch (errorEx &err) {
 		if (!err.isEmpty()) {
@@ -595,11 +648,8 @@ x509name NewX509::getX509name(int _throw)
 void NewX509::setX509name(const x509name &n)
 {
 	int i,j;
-#if QT_VERSION >= 0x040200
-	extDNlist->clearContents();
-#else
-	extDNlist->clear();
-#endif
+
+	extDNlist->deleteAllRows();
 	for (j=0; j<EXPLICIT_NAME_CNT; j++) {
 		name_ptr[j]->setText("");
 	}
@@ -613,50 +663,22 @@ void NewX509::setX509name(const x509name &n)
 			}
 		}
 		if (j == EXPLICIT_NAME_CNT) {
-			QTableWidgetItem *tw;
-			int row;
-
-			row = extDNlist->rowCount();
-			extDNlist->setRowCount(row+1);
-
-			for (int i=0; i<2; i++) {
-				tw = new QTableWidgetItem(sl[i+1]);
-				extDNlist->setItem(row, i, tw);
-			}
+			extDNlist->addRow(sl.mid(1, 2));
 		}
 	}
 }
 
-void NewX509::on_extDNadd_clicked()
-{
-	QTableWidgetItem *tw;
-	int row;
-
-	row = extDNlist->rowCount();
-	extDNlist->setRowCount(row+1);
-
-	tw = new QTableWidgetItem(extDNobj->currentText());
-	extDNlist->setItem(row, 0, tw);
-
-	tw = new QTableWidgetItem(extDNname->text());
-	extDNlist->setItem(row, 1, tw);
-}
-
-void NewX509::on_extDNdel_clicked()
-{
-	extDNlist->removeRow(extDNlist->currentRow());
-}
-
 void NewX509::on_applyTime_clicked()
 {
-	applyTD(this, validNumber->text().toInt(), validRange->currentIndex(),
-			midnightCB->isChecked(), notBefore, notAfter);
+	notAfter->setDiff(notBefore, validNumber->text().toInt(),
+				     validRange->currentIndex());
 }
 
 void NewX509::setupTmpCtx()
 {
 	pki_x509 *signcert;
 	pki_x509req *req = NULL;
+	pki_key *key = NULL;
 	a1int serial;
 	QString errtxt;
 
@@ -667,9 +689,14 @@ void NewX509::setupTmpCtx()
 	if (fromReqCB->isChecked()) {
 		req = getSelectedReq();
 		ctx_cert->setSubject(req->getSubject());
+		if (req)
+			key = req->getRefKey();
 	} else {
 		ctx_cert->setSubject(getX509name());
+		key = getSelectedKey();
 	}
+	if (key)
+		ctx_cert->setPubKey(key);
 	// Step 2 - select Signing
 	if (foreignSignRB->isChecked()) {
 		signcert = getSelectedSigner();
@@ -699,7 +726,7 @@ void NewX509::on_adv_validate_clicked()
 	if (!nconf_data->isReadOnly()) {
 		QString errtxt;
 		extList el;
-		pki_base::ign_openssl_error();
+		ign_openssl_error();
 		QString result;
 		setupTmpCtx();
 		v3ext_backup = nconf_data->toPlainText();
@@ -707,34 +734,44 @@ void NewX509::on_adv_validate_clicked()
 			el = getSelectedReq()->getV3ext();
 		}
 		if (el.size() > 0) {
-			result = "<h2><center>"
-				"From PKCS#10 request</center></h2><p>\n";
+			result = "<h2><center>";
+			result += tr("From PKCS#10 request") +
+				"</center></h2><p>\n";
 			result += el.getHtml("<br>");
 		}
-		el = getGuiExt();
-		el += getNetscapeExt();
-		el.delInvalid();
+		try {
+			el = getGuiExt();
+			el += getNetscapeExt();
+			el.delInvalid();
+		} catch (errorEx &err) {
+			errtxt = err.getString();
+			el.clear();
+		}
 		if (el.size() > 0) {
 			if (!result.isEmpty())
 				result += "\n<hr>\n";
-			result += "<h2><center>Other Tabs</center></h2><p>\n";
+			result += "<h2><center>";
+			result += tr("Other Tabs") + "</center></h2><p>\n";
 			result += el.getHtml("<br>");
 		}
-		el = getAdvanced();
+		try {
+			el = getAdvanced();
+		} catch (errorEx &err) {
+			errtxt += err.getString();
+			el.clear();
+		}
 		if (el.size() > 0) {
 			if (!result.isEmpty())
 				result += "\n<hr>\n";
-			result += "<h2><center>Advanced Tab</center></h2><p>\n";
+			result += "<h2><center>";
+			result += tr("Advanced Tab") + "</center></h2><p>\n";
 			result += el.getHtml("<br>");
 		}
-		while (int i = ERR_get_error() ) {
-			errtxt += ERR_error_string(i, NULL);
-			errtxt += "<br>\n";
-		}
 		if (!errtxt.isEmpty()) {
 			if (!result.isEmpty())
 				result += "\n<hr>\n";
-			result += "<h2><center>Errors</center></h2><p>\n";
+			result += "<h2><center>";
+			result += tr("Errors") + "</center></h2><p>\n";
 			result += errtxt;
 		}
 		nconf_data->document()->setHtml(result);
@@ -749,22 +786,18 @@ void NewX509::on_adv_validate_clicked()
 		adv_validate->setText(tr("Validate"));
 		valid_htmltext = "";
 	}
-	pki_base::ign_openssl_error();
+	ign_openssl_error();
 }
 
 void NewX509::on_editSubAlt_clicked()
 {
 	QString s = "email,RID,URI,DNS,IP,otherName";
-	if (pt != x509_req)
-		s = QString("email:copy,") + s;
 	editV3ext(subAltName, s, NID_subject_alt_name);
 }
 
 void NewX509::on_editIssAlt_clicked()
 {
-	QString s = "email,RID,URI,DNS,IP,otherName";
-	if (pt != x509_req)
-	        s = QString("issuer:copy,") +s ;
+	QString s = "email,RID,URI,DNS,IP,otherName,issuer";
 	editV3ext(issAltName, s, NID_issuer_alt_name);
 }
 
@@ -804,39 +837,74 @@ QString NewX509::mandatoryDnRemain()
 	return dnl.join(",");
 }
 
-void NewX509::on_okButton_clicked()
+void NewX509::gotoTab(int tab)
 {
-	on_tabWidget_currentChanged(0);
+	for (int i=0; i<tabWidget->count(); i++) {
+		if (tabWidget->tabText(i) == tabnames[tab]) {
+			tabWidget->setCurrentIndex(i);
+			break;
+		}
+	}
+}
 
+void NewX509::accept()
+{
+	x509name xn;
+	on_tabWidget_currentChanged(0);
 	try {
-		getX509name(1);
+		xn = getX509name(1);
 	} catch (errorEx &err) {
-		if (QMessageBox::warning(this, tr(XCA_TITLE), err.getString(),
-				tr("Ok"), tr("Abort rollout")) == 1)
-		{
+		gotoTab(1);
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE, err.getString(),
+			QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok);
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		if (msg.exec() == QMessageBox::Close) {
 			reject();
 		}
 		return;
 	}
-
-	if (fromReqCB->isChecked()) {
-		 if (!getSelectedReq()->verify()) {
-			if (QMessageBox::warning(this, tr(XCA_TITLE),
-					tr("The verification of the Certificate request failed.\n"
-					"The rollout should be aborted."),
-					tr("Continue anyway"), tr("Abort rollout")) == 1)
-			{
+	QString lenErr = xn.checkLength();
+	if (!lenErr.isEmpty()) {
+		gotoTab(1);
+		lenErr = tr("The following length restrictions of RFC3280 are violated:") +
+			"\n" + lenErr;
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+					lenErr, QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit subject"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		switch (msg.exec())
+		{
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
+				return;
+			case QMessageBox::Close:
 				reject();
-			}
+				return;
+			case QMessageBox::Apply:
+				break;
+		}
+	}
+	if (fromReqCB->isChecked() && !getSelectedReq()->verify()) {
+		gotoTab(0);
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+			tr("The verification of the Certificate request failed.\nThe rollout should be aborted."),
+			QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Continue anyway"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		if (msg.exec() == QMessageBox::Close) {
+			reject();
 		}
 	}
 	if (description->text().isEmpty() && !fromReqCB->isChecked()) {
 		if (commonName->text().isEmpty()) {
-			if (QMessageBox::warning(this, tr(XCA_TITLE),
-					tr("The internal name and the common name are empty.\n"
-					"Please set at least the internal name."),
-					tr("Ok"), tr("Abort rollout")) == 1)
-			{
+			gotoTab(1);
+			QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+				tr("The internal name and the common name are empty.\nPlease set at least the internal name."), QMessageBox::NoButton, this);
+			msg.addButton(QMessageBox::Ok)->setText(tr("Edit name"));
+			msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+			if (msg.exec() == QMessageBox::Close) {
 				reject();
 			}
 			return;
@@ -844,70 +912,152 @@ void NewX509::on_okButton_clicked()
 			description->setText(commonName->text());
 		}
 	}
-
-	if ( keyList->count() == 0 &&
-		keyList->isEnabled() &&
-		!fromReqCB->isChecked())
+	if (keyList->count() == 0 && keyList->isEnabled() &&
+				!fromReqCB->isChecked())
 	{
-		if (QMessageBox::warning(this, tr(XCA_TITLE),
-				tr("There is no Key selected for signing."),
-				tr("Ok"), tr("Abort rollout")) == 1)
-		{
+		gotoTab(1);
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+			tr("There is no Key selected for signing."),
+			QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Select key"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		if (msg.exec() == QMessageBox::Close) {
 			reject();
 		}
 		return;
 	}
-
 	QString unsetDN;
 	if (pt != tmpl)
 		unsetDN = mandatoryDnRemain();
-	if (!unsetDN.isEmpty())
-	{
-		switch (QMessageBox::warning(this, tr(XCA_TITLE),
-				tr("The following distinguished name entries are empty,\n"
-					"though you have declared them as mandatory "
-					"in the options menu:\n") + unsetDN,
-				tr("Ok"), tr("Abort rollout"),
-				tr("Continue rollout")))
+	if (!unsetDN.isEmpty()) {
+		gotoTab(1);
+		QString text = tr("The following distinguished name entries are empty:\n%1\nthough you have declared them as mandatory in the options menu.").arg(unsetDN);
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+					text, QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit subject"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		switch (msg.exec())
 		{
-			case -1:
-			case 0:
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
 				return;
-			case 1:
+			case QMessageBox::Close:
 				reject();
 				return;
+			case QMessageBox::Apply:
+				break;
 		}
 	}
-	if (notBefore->getDate() > notAfter->getDate() &&
-				!noWellDefinedExpDate->isChecked()) {
-		switch (QMessageBox::warning(this, tr(XCA_TITLE),
-			tr("The certificate will be out of date before it becomes valid. "
-				"You most probably mixed up both dates."),
-			tr("Ok"), tr("Abort rollout"), tr("Continue rollout")))
+	pki_x509 *signer = NULL;
+	if (!selfSignRB->isChecked())
+		signer = getSelectedSigner();
+
+	pki_key *signkey = getSelectedKey();
+	if (signer)
+		signkey = signer->getRefKey();
+
+	if ((!signkey || signkey->isPubKey()) && pt != tmpl) {
+		QString txt;
+		gotoTab(signer ? 0 : 1);
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+			tr("The key you selected for signing is not a private one."),
+			QMessageBox::NoButton, this);
+		txt = signer ? tr("Select other signer"):tr("Select other key");
+		msg.addButton(QMessageBox::Ok)->setText(txt);
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		if (msg.exec() == QMessageBox::Close) {
+			reject();
+		}
+		return;
+        }
+	if (signer && notBefore->getDate() < signer->getNotBefore()) {
+		gotoTab(2);
+		QString text = tr("The certificate will be earlier valid than the signer. This is probably not what you want.");
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+					text, QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Yes)->setText(tr("Adjust date and continue"));
+		switch (msg.exec())
 		{
-			case -1:
-			case 0:
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
 				return;
-			case 1:
+			case QMessageBox::Close:
 				reject();
 				return;
+			case QMessageBox::Apply:
+				break;
+			case QMessageBox::Yes:
+				notBefore->setDate(signer->getNotBefore());
+		}
+	}
+	if (signer && notAfter->getDate() > signer->getNotAfter() &&
+				!noWellDefinedExpDate->isChecked()) {
+		gotoTab(2);
+		QString text = tr("The certificate will be longer valid than the signer. This is probably not what you want.");
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+					text, QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		msg.addButton(QMessageBox::Yes)->setText(tr("Adjust date and continue"));
+		switch (msg.exec())
+		{
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
+				return;
+			case QMessageBox::Close:
+				reject();
+				return;
+			case QMessageBox::Apply:
+				break;
+			case QMessageBox::Yes:
+				notAfter->setDate(signer->getNotAfter());
+		}
+	}
+	if (notBefore->getDate() > notAfter->getDate()) {
+		gotoTab(2);
+		QString text = tr("The certificate will be out of date before it becomes valid. You most probably mixed up both dates.");
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+					text, QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText("Continue rollout");
+		switch (msg.exec())
+		{
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
+				return;
+			case QMessageBox::Close:
+				reject();
+				return;
+			case QMessageBox::Apply:
+				break;
 		}
 	}
 	on_adv_validate_clicked();
 	if (checkExtDuplicates()) {
-		switch (QMessageBox::warning(this, tr(XCA_TITLE),
-			tr("The certificate contains duplicated extensions. "
-				"Check the validation on the advanced tab."),
-			tr("Ok"), tr("Abort rollout"), tr("Continue rollout")))
+		gotoTab(5);
+		QString text = tr("The certificate contains duplicated extensions. Check the validation on the advanced tab.");
+		QMessageBox msg(QMessageBox::Warning, XCA_TITLE,
+					text, QMessageBox::NoButton, this);
+		msg.addButton(QMessageBox::Ok)->setText(tr("Edit extensions"));
+		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
+		switch (msg.exec())
 		{
-			case -1:
-			case 0:
+			case QMessageBox::Ok:
+			case QMessageBox::Cancel:
 				return;
-			case 1:
+			case QMessageBox::Close:
 				reject();
 				return;
+			case QMessageBox::Apply:
+				break;
 		}
 	}
-
-	accept();
+	QDialog::accept();
 }
