@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -12,8 +12,9 @@
 #include "ui_NewX509.h"
 #include "lib/oid.h"
 #include "lib/db.h"
+#include "widgets/kvView.h"
 #include <openssl/x509v3.h>
-#include <qlistwidget.h>
+#include <QtGui/QListWidget>
 
 class MainWindow;
 class extList;
@@ -43,10 +44,12 @@ class NewX509: public QDialog, public Ui::NewX509
 		enum pki_type pt;
 		void templateChanged(QString templatename);
 		QString mandatoryDnRemain();
-		QStringList private_keys, private_keys0;
+		QStringList private_keys, private_keys0, tabnames;
 		pki_x509 *ctx_cert;
 		QString v3ext_backup;
 		QString valid_htmltext;
+		kvmodel *extDNmodel;
+
 	public:
 		static int name_nid[EXPLICIT_NAME_CNT];
 		QRadioButton *selfQASignRB;
@@ -89,30 +92,35 @@ class NewX509: public QDialog, public Ui::NewX509
 		void switchHashAlgo();
 		void addReqAttributes(pki_x509req *req);
 		int checkExtDuplicates();
+		void subjectFromTemplate(pki_temp *temp);
+		void extensionsFromTemplate(pki_temp *temp);
+		pki_temp *currentTemplate();
+		void gotoTab(int tab);
+		void setupLineEditByNid(int nid, QLineEdit *l);
+
 	public slots:
 		void on_fromReqCB_clicked();
 		void on_keyList_currentIndexChanged(const QString &);
 		void on_reqList_currentIndexChanged(const QString &);
 		void newKeyDone(QString name);
-		void on_extDNadd_clicked();
-		void on_extDNdel_clicked();
 		void on_applyTime_clicked();
 		void on_editSubAlt_clicked();
 		void on_editIssAlt_clicked();
 		void on_editCrlDist_clicked();
 		void on_editAuthInfAcc_clicked();
 		void on_foreignSignRB_toggled(bool checked);
-		void on_selfSignRB_toggled(bool checked);
 		void on_genKeyBut_clicked();
 		void on_showReqBut_clicked();
 		void on_certList_currentIndexChanged(int index);
 		void on_applyTemplate_clicked();
-		void on_okButton_clicked();
+		void on_applySubject_clicked();
+		void on_applyExtensions_clicked();
 		void on_adv_validate_clicked();
 		void on_usedKeysToo_toggled(bool checked);
 		void on_tabWidget_currentChanged(int idx);
 		void on_reqSubChange_clicked();
-		void on_noWellDefinedExpDate_toggled(bool);
+		void accept();
+		void setupExtDNwidget(const QString &s, QLineEdit *w);
 
 	signals:
 		void genKey(QString);
