@@ -5,30 +5,38 @@
  * All rights reserved.
  */
 
-#ifndef OPTIONS_H
-#define OPTIONS_H
+#ifndef __OPTIONS_H
+#define __OPTIONS_H
 
 #include "ui_Options.h"
-#include <qdialog.h>
+#include <QtGui/QDialog>
 #include "lib/base.h"
-#include "widgets/MainWindow.h"
+#include "SearchPkcs11.h"
+#include "MainWindow.h"
 
 class Options: public QDialog, public Ui::Options
 {
 		Q_OBJECT
 	private:
+		SearchPkcs11 *searchP11;
 		QStringList string_opts;
 		MainWindow *mw;
 	public:
 		Options(MainWindow *parent);
+		~Options();
+		void setupPkcs11Provider(QString list);
+		void setStringOpt(const QString string_opt);
+		QString getDnString();
+		QString getStringOpt();
+		QString getPkcs11Provider();
+
 	public slots:
 		void on_extDNadd_clicked();
 		void on_extDNdel_clicked();
-		QString getDnString();
-		void setStringOpt(const QString string_opt);
-		QString getStringOpt();
-		void on_fileButton_clicked(void);
-		void on_tryLoadButton_clicked(void);
+		void on_addButton_clicked(void);
+		void on_removeButton_clicked(void);
+		void on_searchPkcs11_clicked(void);
+		void addLib(QString);
 };
 
 #endif
