@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -15,15 +15,19 @@ static struct {
 } hashalgos[] = {
 	{ "MD 5", EVP_md5() },
 	{ "SHA 1", EVP_sha1() },
-#ifdef HAS_SHA256
 	{ "SHA 256", EVP_sha256() },
 	{ "SHA 384", EVP_sha384() },
 	{ "SHA 512", EVP_sha512() },
-#endif
 	{ "RIPEMD 160", EVP_ripemd160() },
 };
 
-QString hashBox::default_md = QString(hashalgos[1].name); /* SHA1 */
+QString hashBox::default_md = QString();
+
+void hashBox::resetDefault()
+{
+	/* SHA1 */
+	default_md = QString(hashalgos[1].name);
+}
 
 hashBox::hashBox(QWidget *parent)
 	:QComboBox(parent)
@@ -42,8 +46,10 @@ const EVP_MD *hashBox::currentHash()
 	switch(key_type) {
 	case EVP_PKEY_DSA:
 		return EVP_dss1();
+#ifndef OPENSSL_NO_EC
 	case EVP_PKEY_EC:
 		return EVP_ecdsa();
+#endif
 	default:
 		QString hash = currentText();
 		for (unsigned i=0; i<ARRAY_SIZE(hashalgos); i++) {
@@ -51,11 +57,25 @@ const EVP_MD *hashBox::currentHash()
 				return hashalgos[i].md;
 		}
 	}
-	return NULL;
+	return hashalgos[1].md; /* SHA1 as fallback */
+}
+
+void hashBox::setCurrentString(QString md)
+{
+	int idx = findText(md);
+	if (idx != -1) {
+		setCurrentIndex(idx);
+		wanted_md = "";
+	} else {
+		wanted_md = md;
+	}
 }
 
 void hashBox::setupHashes(QList<int> nids)
 {
+	QString md = currentText();
+	if (!wanted_md.isEmpty())
+		md = wanted_md;
 	clear();
 	for (unsigned i=0; i<ARRAY_SIZE(hashalgos); i++) {
 		if (nids.contains(hashalgos[i].md->type)) {
@@ -63,14 +83,19 @@ void hashBox::setupHashes(QList<int> nids)
 		}
 	}
 	setDefaultHash();
+	setCurrentString(md);
 }
 
 void hashBox::setupAllHashes()
 {
+	QString md = currentText();
+	if (!wanted_md.isEmpty())
+		md = wanted_md;
 	clear();
 	for (unsigned i=0; i<ARRAY_SIZE(hashalgos); i++) {
 		addItem(QString(hashalgos[i].name));
 	}
+	setCurrentString(md);
 }
 
 QString hashBox::currentHashName()
@@ -80,7 +105,7 @@ QString hashBox::currentHashName()
 
 void hashBox::setDefaultHash()
 {
-	setCurrentIndex(findText(default_md));
+	setCurrentString(default_md);
 }
 
 void hashBox::setCurrentAsDefault()
