@@ -5,10 +5,10 @@
  * All rights reserved.
  */
 
-#ifndef HASH_BOX_H
-#define HASH_BOX_H
+#ifndef __HASH_BOX_H
+#define __HASH_BOX_H
 
-#include <qcombobox.h>
+#include <QtGui/QComboBox>
 #include <openssl/evp.h>
 
 class hashBox: public QComboBox
@@ -16,6 +16,7 @@ class hashBox: public QComboBox
 		Q_OBJECT
 	private:
 		static QString default_md;
+		QString wanted_md;
 		int backup;
 		int key_type;
 	public:
@@ -27,7 +28,9 @@ class hashBox: public QComboBox
 		void setDefaultHash();
 		void setupHashes(QList<int> nids);
 		void setupAllHashes();
+		void setCurrentString(QString md);
 		static void setDefault(QString def);
+		static void resetDefault();
 };
 
 #endif
