@@ -5,14 +5,15 @@
  * All rights reserved.
  */
 
-#ifndef IMPORTMULTI_H
-#define IMPORTMULTI_H
+#ifndef __V3EXT_H
+#define __V3EXT_H
 
 #include "ui_v3ext.h"
 #include "lib/pki_base.h"
-#include <qlineedit.h>
-#include <qstringlist.h>
+#include <QtGui/QLineEdit>
+#include <QtCore/QStringList>
 #include <openssl/x509v3.h>
+#include "widgets/kvView.h"
 
 class pki_x509;
 class pki_key;
@@ -27,7 +28,6 @@ class v3ext: public QDialog, public Ui::v3ext
 		bool __validate(bool showSuccess);
 	public:
 		v3ext( QWidget *parent);
-		~v3ext();
 		void addItem(QString list);
 		void addEntry(QString list);
 		QString toString();
@@ -35,10 +35,9 @@ class v3ext: public QDialog, public Ui::v3ext
 				X509V3_CTX *ctx);
 
 	public slots:
-		void on_delEntry_clicked();
-		void on_addEntry_clicked();
 		void on_apply_clicked();
 		void on_validate_clicked();
+		void setupLineEdit(const QString &s, QLineEdit *l);
 };
 
 #endif
